      SUBROUTINE ASSIM_TIMESEC(idate0,idate1,idateconvtot)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Time/date
C   Purpose   : Return seconds elapsed between two dates
C   Called by : 
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 03/1999
C
C Modified  :
C
CCC
C
C INPUT
C
C idate0 : initial date (1)=Year (2)=Month (3)=Day (4)=Hr (5)=Min
C idate1 : final date (1)=Year (2)=Month (3)=Day (4)=Hr (5)=Min
C
C OUTPUT
C
C idateconvtot : seconds elapsed between the two
C                (=-1 if date1 is before date0)
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: idateconvtot
	INTEGER , DIMENSION (5) :: idate0,idate1
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: J
      INTEGER , ALLOCATABLE :: tabbis(:)
      logical :: test
C
C
	ALLOCATE (tabbis(12))
      tabbis=(/31,28,31,30,31,30,31,31,30,31,30,31/)
c
c 1. COMPARE YEARS
c
      test=.FALSE.
c
c test : .TRUE. if the two dates correspond to the same year
c
      if (idate0(1).EQ.idate1(1)) then
c
c If dates correspond to the same year, add one year between the two
c
        idate1(1)=idate1(1)+1
        test=.TRUE.
      endif
c
      if (idate0(1).GT.idate1(1)) then
c
c Date1 is before date0 : return 
c
        idateconvtot=-1
        return
      endif
c
c 2. CONVERT YEARS
c
      idateconvtot=0
      do 200 j=idate0(1)+1,idate1(1)-1
c
        if (MOD(j,4).NE.0)then
c
c Year #j has 365 days
c
          idateconvtot=idateconvtot+31536000
        else
c
c Year #j has 366 days
c
          idateconvtot=idateconvtot+31622400
        endif
c
 200  continue
c
c 3. CONVERT MONTHS
c
      if (MOD(idate0(1),4).EQ.0) then
c
c February of year0 has 29 days 
c
        tabbis(2)=tabbis(2)+1
      endif
c
      do 300 j=idate0(2),12
c
c Contibution of year 0 
c
        idateconvtot=idateconvtot+tabbis(j)*86400
 300  continue
c
       if (MOD(idate0(1),4).EQ.0) then
c
c Restore 28 days for february
c
        tabbis(2)=tabbis(2)-1
      endif
c
      if (MOD(idate1(1),4).EQ.0) then
c
c February of year1 has 29 days
c
        tabbis(2)=tabbis(2)+1
      endif
c
      do 301 j=1,idate1(2)-1
c
c Contribution of year 1
c

        idateconvtot=idateconvtot+tabbis(j)*86400
 301  continue
c
       if (MOD(idate1(1),4).EQ.0) then
c
c Restore 28 days for february
c
        tabbis(2)=tabbis(2)-1
      endif
c
c 4. CONVERT DAYS, HOURS AND MINUTES 
c

      idateconvtot=idateconvtot+
     $             86400*(idate1(3)-idate0(3))+
     $              3600*(idate1(4)-idate0(4))+
     $                60*(idate1(5)-idate0(5))
c
c 5. RESTORE INITIAL YEAR DIFFERENCE
c
      if (TEST) then
c
c One year between the two dates was artificially added 
c
        if (idate1(2).LE.2) then
c
c Month of date 1 is before the end of february
c
          if (MOD(idate0(1),4).EQ.0) then
c
c February of year 0 has 29 days 
c
            idateconvtot=idateconvtot-31622400
          else
c
c February of year 0 has 28 days
c
            idateconvtot=idateconvtot-31536000
          endif
c
        else
c
c Month of date 1 is after the end of february
c
          if (MOD(idate1(1),4).EQ.0) then
c
c February of year 1 has 29 days 
c
            idateconvtot=idateconvtot-31622400
          else
c
c February of year 1 has 28 days
c
            idateconvtot=idateconvtot-31536000
          endif
c
        endif
c
c Restore idate1(1)
c
        idate1(1)=idate1(1)-1
c
      endif
c
c 6. RETURN
c
      if (idateconvtot.LT.0) then
          idateconvtot=-1
      endif
c
	DEALLOCATE (tabbis)
c
c	
      return
      end
