C
CCC
C
      SUBROUTINE H_NOV_ADJ()
C
      USE palmlib
C
C
CCCCC
C
C  Module Assimilation de DONnees et Analyse
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C
C  Centre National de Recherches Meteorologiques
C
CCCCCC
C
C Purpose   : Read observational data for model diagnostic
C             and data assimilation purposes.
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 03/1999
C
C Modified  : V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2001
C
CCC
C     PALM and PRIVATE MODULES
      USE palm_user_param
      USE DATDAT_MOD
C
C     NOVELTIS MODULES
      USE model_structure
      USE matrix_csr_structure
      USE convert_dates
      USE matrix_manipulations
      USE H_CSR
C
      IMPLICIT NONE
C
C INPUT
C
C daynum   : time from start (s)
C LASTTIME : if current call is the last one for "madona" (LASTTIME=.T.),
C            do not call "LECTOBS" and prepare GUESS*A
C
      INTEGER, DIMENSION(ip_ninstmax) :: ila_count_slot, ila_b, ila_e 
      INTEGER :: ICOUNT,IOBS,I,LOBS,IPRES,ICST, ILON, IYSTAR, il_n_niv
      INTEGER :: ib_inst, il_scount, il_count_slot
      INTEGER :: IB, INIV, IYSTAR0
C
      REAL (KIND=8),
     $    DIMENSION(ip_lono1*ip_lato1*ip_nivm1*2) ::
     $    rla_vector_out
      REAL (KIND=8),
     $    DIMENSION(ip_lono1,ip_lato1,ip_nivm1,ip_nassim) ::
     $    QQQM1ADJBEFORE
      REAL (KIND=8),
     $    DIMENSION(ip_lono1,ip_lato1,ip_nivm1,ip_nassim) ::
     $    QQQM1ADJAFTER
C
      REAL(KIND=8) :: QNOR
      REAL daynum, rl_DT0
      LOGICAL LASTTIME
C
C TEMPORARY VARIABLES AND ARRAYS
C
      INTEGER ILOBS(ip_nobstspmax)
C
      LOGICAL FIRSTTIME
C
CCC DECLARATION OF ystar = rga_Hx* WHERE OBSERVATIONS ARE FOUND 
C
C
      REAL, ALLOCATABLE :: ystar(:)
      REAL, ALLOCATABLE :: xstar(:)
C
      LOGICAL, DIMENSION(:), ALLOCATABLE :: lla_discard_obs
      REAL (KIND=8), DIMENSION(:,:), ALLOCATABLE :: rla_y
C
      TYPE(CSR_FORMAT) :: sl_Hres_csr
      TYPE(MODEL_DATA),DIMENSION(ip_nassim) :: sla_mod_data
      TYPE(PALM_H_CSR), DIMENSION(:), ALLOCATABLE :: sla_H_palm
      TYPE(PALM_HAVK_CSR), DIMENSION(:), ALLOCATABLE :: sla_Havk_palm
      LOGICAL, DIMENSION(ip_nobstspmax) :: lla_discard_palm
C
CCC PALM VARIABLES
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
      IF (ip_ninstmax_avk.lt.ip_ninstmax) ALLOCATE(sla_H_palm(1))
      IF (ip_ninstmax_avk.gt.0) ALLOCATE(sla_Havk_palm(1))
C
CCC PALM GET THE TIME
c
      cl_space = 'one_real'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'rd_daynum'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,daynum,il_err)
C
      cl_space = 'one_integer'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'id_count_slot'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_count_slot,
     $    il_err)
C
C
CCC PALM GET THE LOGICALS LASTTIME AND FIRSTTIME
C
      cl_space = 'one_logical'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'ld_lasttime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,LASTTIME,il_err)
C
      cl_space = 'one_logical'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'ld_firsttime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,FIRSTTIME,il_err)
C
      rl_DT0 = REAL(ip_obs_slot)
C      
C INITIALIZATION FOR ADJOINT CODE
C
      QQQM1ADJBEFORE(:,:,:,:)=0.
      QQQM1ADJAFTER(:,:,:,:)=0.
C
C Working for every assimilated tracer
C
      iystar0=0
      DO i = 1, ip_nassim
C
C SELECT OBSERVATIONS IN CURRENT TIMESTEP
C
CAP
CAP Should insert a tracer selection for when 
CAP obseving more than 1 species
CAP
        icount=0
        ila_count_slot(:)=0
        ila_b(:)=ig_nobs
        ila_e(:)=0
        DO iobs=1,ig_nobs
          IF (icount.GT.ip_nobstspmax) THEN
              WRITE(*,*) '+ TOO MANY OBS. FOR CURRENT TIMESTEP: ABORT!'
              STOP
          ENDIF
          IF ((INT(daynum).LE.iga_secobs(iobs)).AND.
     +        (INT(daynum+rl_DT0).GT.iga_secobs(iobs))) THEN
              icount=icount+1
              ILOBS(icount)=iobs
              ila_count_slot(iga_inst(iobs)) =
     $            ila_count_slot(iga_inst(iobs))+1
              IF (ila_b(iga_inst(iobs)) .gt. icount)
     $            ila_b(iga_inst(iobs))=icount
              IF (ila_e(iga_inst(iobs)) .lt. icount)
     $            ila_e(iga_inst(iobs))=icount
          ENDIF
        ENDDO
C
C SPATIAL INTERPOLATION : END OF CURRENT TIMESTEP / CURRENT OBSERVATIONS
C
CAP        IF (icount.NE.0) THEN
        IF (il_count_slot.NE.0) THEN
C
CCC PALM GET Y*
C
CAP We make the assumption that nbconstmax = ip_nassim !
CAP
            ALLOCATE(ystar(ip_nobstspmax*ip_nbnivpresmax*ip_nassim),
     $          STAT=il_err)
            IF (il_err .ne. 0) THEN
                WRITE(*,*) 'H_NOV_ADJ Error allocating ystar'
                WRITE(PL_OUT,*) 'H_NOV_ADJ Error allocating ystar'
                CALL PALM_Abort(il_err)
            ENDIF
            ystar(:)=0.
C
            cl_space = 'vectobs'
            il_time = int(daynum)
            il_usrtag = PL_NO_TAG
            cl_name = 'rda_ystar'
            CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,ystar,
     $          il_err)
C
CAP        END IF
C
C LOOP OVER THE INSTRUMENTS
C
        DO ib_inst = 1,ig_ninst
C
          il_scount = ila_count_slot(ib_inst)
C         
          IF (il_scount.NE.0) THEN
C
              il_n_niv = 1
              DO iobs=ila_b(ib_inst),ila_e(ib_inst)
                lobs=ILOBS(iobs)
                IF (iga_npreslev(lobs).gt.il_n_niv)
     $              il_n_niv = iga_npreslev(lobs)
              END DO
C
              IF (lga_avg_kernel(ib_inst)) THEN            
C
C GET THE H MATRIX
C
CCC Get sl_HPalm
                  cl_space = 'CSR_HAVK'
                  il_time = int(daynum)
                  il_usrtag = ib_inst
                  cl_name = 'Havk_csr'
                  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $                sla_Havk_palm(1),il_err)
C
                  IF (il_err.ne.0) THEN
                      CALL PALM_Error_explain(il_err, il_err2)
                  ENDIF
C
                  IF (associated(sl_Hres_csr%H)) THEN 
                      DEALLOCATE(sl_Hres_csr%H,STAT=il_err)
                      DEALLOCATE(sl_Hres_csr%i,STAT=il_err)
                      DEALLOCATE(sl_Hres_csr%j,STAT=il_err)
                  END IF
C
                  ALLOCATE(sl_Hres_csr%i(
     $                sla_Havk_palm(1)%nrow+1,sla_Havk_palm(1)%NPRF))
                  ALLOCATE(sl_Hres_csr%j(
     $                sla_Havk_palm(1)%nmax,sla_Havk_palm(1)%NPRF))
                  ALLOCATE(sl_Hres_csr%H(
     $                sla_Havk_palm(1)%nmax,sla_Havk_palm(1)%NPRF))
C
CCC Inject the structure
                  sl_Hres_csr%NPRF = sla_Havk_palm(1)%NPRF 
                  sl_Hres_csr%ncol = sla_Havk_palm(1)%ncol 
                  sl_Hres_csr%nrow = sla_Havk_palm(1)%nrow 
                  sl_Hres_csr%nmax = sla_Havk_palm(1)%nmax 
                  sl_Hres_csr%i(:,:) =
     $                sla_Havk_palm(1)%i(1:sl_Hres_csr%nrow+1,
     $                1:sl_Hres_csr%NPRF)
                  sl_Hres_csr%j(:,:) =
     $                sla_Havk_palm(1)%j(1:sl_Hres_csr%nmax,
     $                1:sl_Hres_csr%NPRF)
                  sl_Hres_csr%H(:,:) =
     $                sla_Havk_palm(1)%H(1:sl_Hres_csr%nmax,
     $                1:sl_Hres_csr%NPRF)
C
CCC Get discard, allocate the local one and inject the values
C
                  cl_space = 'CSR_HAVK_discard'
                  il_time = int(daynum)
                  il_usrtag = ib_inst
                  cl_name = 'Havk_discard'
                  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $                lla_discard_palm,il_err)
C
                  IF (il_err.ne.0) THEN
                      CALL PALM_Error_explain(il_err, il_err2)
                  ENDIF
C
                  ALLOCATE(lla_discard_obs(il_scount))
C
                  lla_discard_obs(1:il_scount) =
     $                lla_discard_palm(1:il_scount) 
C
              ELSE
C
C GET THE H MATRIX
C
CCC Get sl_HPalm
                  cl_space = 'CSR_H'
                  il_time = int(daynum)
                  il_usrtag = ib_inst
                  cl_name = 'H_csr'
                  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $                sla_H_palm(1),il_err)
C
                  IF (il_err.ne.0) THEN
                      CALL PALM_Error_explain(il_err, il_err2)
                  ENDIF
C
                  IF (associated(sl_Hres_csr%H)) THEN 
                      DEALLOCATE(sl_Hres_csr%H,STAT=il_err)
                      DEALLOCATE(sl_Hres_csr%i,STAT=il_err)
                      DEALLOCATE(sl_Hres_csr%j,STAT=il_err)
                  END IF
C
                  ALLOCATE(sl_Hres_csr%i(
     $                sla_H_palm(1)%nrow+1,sla_H_palm(1)%NPRF))
                  ALLOCATE(sl_Hres_csr%j(
     $                sla_H_palm(1)%nmax,sla_H_palm(1)%NPRF))
                  ALLOCATE(sl_Hres_csr%H(
     $                sla_H_palm(1)%nmax,sla_H_palm(1)%NPRF))
C
CCC Inject the structure
                  sl_Hres_csr%NPRF = sla_H_palm(1)%NPRF 
                  sl_Hres_csr%ncol = sla_H_palm(1)%ncol 
                  sl_Hres_csr%nrow = sla_H_palm(1)%nrow 
                  sl_Hres_csr%nmax = sla_H_palm(1)%nmax 
                  sl_Hres_csr%i(:,:) =
     $                sla_H_palm(1)%i(1:sl_Hres_csr%nrow+1,
     $                1:sl_Hres_csr%NPRF)
                  sl_Hres_csr%j(:,:) =
     $                sla_H_palm(1)%j(1:sl_Hres_csr%nmax,
     $                1:sl_Hres_csr%NPRF)
                  sl_Hres_csr%H(:,:) =
     $                sla_H_palm(1)%H(1:sl_Hres_csr%nmax,
     $                1:sl_Hres_csr%NPRF)
C
CCC Get discard, allocate the local one and inject the values
C
                  cl_space = 'CSR_H_discard'
                  il_time = int(daynum)
                  il_usrtag = ib_inst
                  cl_name = 'H_discard'
                  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $                lla_discard_palm,il_err)
C
                  IF (il_err.ne.0) THEN
                      CALL PALM_Error_explain(il_err, il_err2)
                  ENDIF
C
                  ALLOCATE(lla_discard_obs(il_scount))
C
                  lla_discard_obs(1:il_scount) =
     $                lla_discard_palm(1:il_scount) 
C
              ENDIF
C
C PREPARE RIGHT HAND SIDE FOR THE TRANS_MULTIPLICATION
C
              ALLOCATE(rla_y(il_n_niv,il_scount))
              rla_y(:,:)=0.
C
              DO iobs = 1, il_scount
                lobs=ILOBS(ila_b(ib_inst)-1+iobs)
                DO iniv = 1,iga_npreslev(lobs)
                  iystar=iystar0+(iniv-1)*ip_nobstspmax+
     $                ila_b(ib_inst)-1+iobs
                  rla_y(iniv,iobs) = ystar(iystar)
                END DO
              END DO
C
C
C MULTIPLY TRANSPOSED OBS OPERATOR
C
              IF (il_scount .NE. sl_Hres_csr%NPRF) THEN
                  WRITE(*,*) 'Number of obs not coherent with H dims'
                  WRITE(*,*) 'Icount = ',il_scount
                  WRITE(*,*) 'H NPRF = ',sl_Hres_csr%NPRF
                  WRITE(*,*) 'For time and instrument ',daynum,ib_inst
                  WRITE(PL_OUT,*) 'Number of obs not coherent with ',
     $                'H dims'
                  WRITE(PL_OUT,*) 'Icount = ',il_scount
                  WRITE(PL_OUT,*) 'H NPRF = ',sl_Hres_csr%NPRF
                  WRITE(PL_OUT,*) 'For time and instrument ',daynum,
     $                ib_inst
                  CALL PALM_Abort(il_err)
              ENDIF
C
              CALL addmult_matrixT_csr_vector(
     $            sl_Hres_csr,
     $            rla_y,
     $            il_n_niv,
     $            il_scount,
     $            rla_vector_out)
C
C STORE RESULTS IN PALM COMPLIANT VECTORS
C
#ifdef USE_BLAS
              CALL DAXPY(ip_lono1*ip_lato1*ip_nivm1,1.,
     $            rla_vector_out(1:ip_lono1*ip_lato1*ip_nivm1),1,
     $            QQQM1ADJBEFORE(1,1,1,i),1)
              CALL DAXPY(ip_lono1*ip_lato1*ip_nivm1,1.,
     $            rla_vector_out(ip_lono1*ip_lato1*ip_nivm1+1:
     $            2*ip_lono1*ip_lato1*ip_nivm1),1,
     $            QQQM1ADJAFTER(1,1,1,i),1)
#else
              QQQM1ADJBEFORE(:,:,:,i)=
     $            QQQM1ADJBEFORE(:,:,:,i)+
     $            RESHAPE(rla_vector_out(1:ip_lono1*ip_lato1*ip_nivm1),
     $            (/ip_lono1,ip_lato1,ip_nivm1/))
              QQQM1ADJAFTER(:,:,:,i)=
     $            QQQM1ADJAFTER(:,:,:,i)+
     $            RESHAPE(rla_vector_out(ip_lono1*ip_lato1*ip_nivm1+1:
     $            2*ip_lono1*ip_lato1*ip_nivm1),
     $            (/ip_lono1,ip_lato1,ip_nivm1/))
#endif
C
C DEALLOCATE USELESS VECTORS
C
              DEALLOCATE(rla_y,STAT=il_err)
              IF (il_err .ne. 0) THEN
                  WRITE(*,*) 'H_NOV_ADJ Error deallocating rla_y'
                  WRITE(PL_OUT,*)
     $                'H_NOV_ADJ Error deallocating rla_y'
                  CALL PALM_Abort(il_err)
              ENDIF
              DEALLOCATE(lla_discard_obs,STAT=il_err)
              IF (il_err .ne. 0) THEN
                  WRITE(*,*)'H_NOV_ADJ Error deallocating ',
     $                'lla_discard_obs'
                  WRITE(PL_OUT,*)
     $                'H_NOV_ADJ Error deallocating lla_doscard_obs'
                  CALL PALM_Abort(il_err)
              ENDIF
C            
          END IF        
        END DO
C
CAP        IF (icount .gt. 0) THEN
CAP        IF (il_count_slot .gt. 0) THEN
            DEALLOCATE(ystar,STAT=il_err)
            IF (il_err .ne. 0) THEN
                WRITE(*,*) 'H_NOV_ADJ Error deallocating ystar'
                WRITE(PL_OUT,*) 'H_NOV_ADJ Error deallocating ystar'
                CALL PALM_Abort(il_err)
            ENDIF
        END IF
C
        iystar0 = iystar0 + ip_nobstspmax*ip_nbnivpresmax
C
      END DO
C
C PALM PUT OF H*.ystar
C
      ALLOCATE(xstar(ip_dimx))
c        
#ifdef USE_BLAS
      CALL DCOPY(ip_dimx,QQQM1ADJBEFORE,1,xstar,1)
#else
      xstar(:) = RESHAPE(QQQM1ADJBEFORE,(/ip_dimx/))
#endif
c
      cl_space = 'observed_species'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_xstar_b'
      CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,xstar,
     +               il_err)
c        
#ifdef USE_BLAS
      CALL DCOPY(ip_dimx,QQQM1ADJAFTER,1,xstar,1)
#else
      xstar(:) = RESHAPE(QQQM1ADJAFTER,(/ip_dimx/))
#endif
c
      cl_space = 'observed_species'
      il_time = int(daynum)
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_xstar_e'
      CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,xstar,
     +    il_err)
c
      DEALLOCATE(xstar,STAT=il_err)
c
      IF (il_err .ne. 0) THEN
          WRITE(*,*) 'H_NOV_ADJ Error deallocating xstar'
          WRITE(PL_OUT,*) 'H_NOV_ADJ Error deallocating xstar'
          CALL PALM_Abort(il_err)
      ENDIF        
C
      IF (ip_ninstmax_avk.lt.ip_ninstmax) DEALLOCATE(sla_H_palm)
      IF (ip_ninstmax_avk.gt.0) DEALLOCATE(sla_Havk_palm)
C
      RETURN
      END
