      SUBROUTINE INITV()
C
C***
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C
C  Centre National de Recherches Meteorologiques
C
C***
C
C Purpose   : Put initial value of v and vb
C             Initialize log_pressure field for vertical diffusion
C
C Original  : S. Massart, CERFACS/GLOBC, 07/2003
C
C**
C
      USE palmlib
      USE palm_user_param
      USE SQRTB_MOD
C
      IMPLICIT NONE
C
C Local Declaration
C
      INTEGER assimtime
      REAL, DIMENSION(:), ALLOCATABLE :: rda_v
      REAL, DIMENSION(:,:), ALLOCATABLE :: rla_psol
      REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_pres_int
      REAL, DIMENSION(:,:,:), ALLOCATABLE :: rla_logp_3d
      REAL, DIMENSION(:), ALLOCATABLE, SAVE :: rla_ALM1, rla_BLM1
      REAL, DIMENSION(:), ALLOCATABLE :: rla_presgri
      LOGICAL, SAVE :: FIRSTTIME = .true.
      REAL :: XPAPLUS, XPAMOIN, rl_deg_2_rad
      INTEGER, DIMENSION(5) :: idate0
      INTEGER :: il_intdate0
      INTEGER :: ilon, ilat, iniv, il_vertcoord, il_modelgrid_angles
      INTEGER :: il_vcor_1D
      INTEGER :: il_myid, il_nprocs
      LOGICAL :: ll_hybrid_sigma
#ifndef B_COMPU
      REAL, DIMENSION(:), ALLOCATABLE :: rla_norm
      REAL :: rl_logbeta
      REAL :: rl_2ipflsz
      INTEGER :: il_loopstart, il_loopend, il_lon_on_p, il_niv_tri
      INTEGER :: isum, ib_idx
#endif
C
C PALM Declaration
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
C*** SPHERICAL HARMONIC INITIALIZATION
C
      INTEGER :: ldim,l1,l2
      INTEGER :: lwork,ldwork 
      REAL, DIMENSION(:), ALLOCATABLE :: work
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: dwork
      DOUBLE PRECISION :: dl_w
C
C*** PARALLELISM
C
      CALL MPI_COMM_RANK(PL_COMM_EXEC,il_myid,il_err)
      CALL MPI_COMM_SIZE(PL_COMM_EXEC,il_nprocs,il_err)
C
C** SWITCH BETWEEN HYBRID SIGMA AND PURE PRESSURE VERTICAL COORD
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'vert_coord'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_vertcoord,
     &    il_err)
C
      ll_hybrid_sigma = il_vertcoord .EQ. 1
C
C SELECT THE MODEL ANGLE REPRESENTATION
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'model_angles'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     &    il_modelgrid_angles,il_err)
C
C** Get assimilation time
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'id_assimtime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,assimtime,il_err)
C
C** Put the null initial control vector
C
      IF (il_myid .eq. 0) THEN
C
          ALLOCATE( rda_v(ip_dimx) )
C
C** Test if we are in the gradient test case 
C
          cl_space = 'crt_space'
          il_time = assimtime
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_dv'
          CALL PALM_Query_Put(cl_space,cl_name,il_time,il_usrtag,il_err)
C
          IF (il_ERR .eq. 0) THEN
C
C** In such a case send two random vectors for the test
C
              CALL RANDOM_NUMBER(rda_v)
              rda_v(:) = rda_v(:) * 1.e-3
C 
              CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,rda_v,
     $            il_err)
C
              CALL RANDOM_NUMBER(rda_v)
              rda_v(:) = rda_v(:) * 1.e-5
C
          ELSE
C
C** In the standard assimilation, initialize the control vector to 0
C
              rda_v(:) = 0.
          ENDIF
C 
          cl_space = 'crt_space'
          il_time = assimtime
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_v'
          CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,rda_v,il_err) 
C
          DEALLOCATE(rda_v)
      END IF
C
      IF (FIRSTTIME) THEN
C
          IF (il_myid .eq. 0) THEN
              idate0(:)=0
              cl_space = 'date'
              il_time = PL_NO_TIME
              il_usrtag = PL_NO_TAG
              cl_name = 'ida_date0'
              CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,idate0,
     $            il_err)
C
              il_intdate0 = 0
              CALL PALM_Time_convert(PL_TIME_DATE2INT, idate0(3),
     $            idate0(2),idate0(1),idate0(4),0,0,il_intdate0,il_err)
C
              cl_space = 'one_integer'
              il_time = PL_NO_TIME
              il_usrtag = PL_NO_TAG
              cl_name = 'il_intdate0'
              CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,
     $            il_intdate0,il_err)   
          ENDIF    
C
          IF (ip_iprint .gt. 0) THEN
              IF (ll_hybrid_sigma) THEN
                  IF (il_myid.eq.0) THEN
                      WRITE(*,'(A17,A26)')'++ USING A SIGMA ',
     $                    'HYBRID VERTICAL COORDINATE'
                      WRITE(*,'(A2)') '++'
                  ENDIF
                  WRITE(PL_OUT,'(A17,A26)')'++ USING A SIGMA ',
     $                    'HYBRID VERTICAL COORDINATE'
                  WRITE(PL_OUT,'(A2)') '++'
              ELSE
                  IF (il_myid.eq.0) THEN
                      WRITE(*,'(A11,A33)') '++ USING A ',
     $                    'PURE PRESSURE VERTICAL COORDINATE'
                      WRITE(*,'(A2)') '++'
                  ENDIF
                  WRITE(PL_OUT,'(A11,A33)')'++ USING A ',
     $                'PURE PRESSURE VERTICAL COORDINATE'
                  WRITE(PL_OUT,'(A2)') '++'
              ENDIF
          ENDIF
C         
C** Diffusion equations initializations
C
          cl_space = 'one_integer'
          il_time = PL_NO_TIME
          il_usrtag = PL_NO_TAG
          cl_name = 'filter_h'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,ig_flsh,
     $        il_err)
C
          cl_space = 'one_integer'
          il_time = PL_NO_TIME
          il_usrtag = PL_NO_TAG
          cl_name = 'filter_z'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,ig_flsz,
     $        il_err)
C
          IF (ig_flsh .NE. 0 .or. ig_flsz .NE. 0) THEN
              WRITE(PL_OUT,'(A37)')
     $            '++ INITIALIZING SQRT(B) AND SQRT(B)^T'
#ifndef B_COMPU
              WRITE(PL_OUT,'(A18)') '++ STORING VERSION'
#else
              WRITE(PL_OUT,'(A22)') '++ RECOMPUTING VERSION'
#endif
              WRITE(PL_OUT,'(A2)') '++'
              IF (il_myid .eq. 0) THEN
                  WRITE(*,'(A37)')
     $                '++ INITIALIZING SQRT(B) AND SQRT(B)^T'
#ifndef B_COMPU
                  WRITE(*,'(A18)') '++ STORING VERSION'
#else
                  WRITE(*,'(A22)') '++ RECOMPUTING VERSION'
#endif
                  WRITE(*,'(A2)') '++'
              END IF
          END IF
C
          IF (ig_flsz .NE. 0) THEN
C
              cl_space = 'one_integer'
              il_time = PL_NO_TIME
              il_usrtag = PL_NO_TAG
              cl_name = 'vert_coord_approx'
              CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     &            il_vcor_1D,il_err)
C
              lg_vcor_1D = (il_vcor_1D.eq.1)
C
              IF (.NOT.ALLOCATED(rga_norm_filv))
     $            ALLOCATE(rga_norm_filv(ip_lono1,
     $            ip_lato1,ip_nivm1))
              rga_norm_filv(:,:,:)=0.
              IF (.NOT.ALLOCATED(rga_logp)) THEN
                IF (lg_vcor_1D) THEN
                  ALLOCATE(rga_logp(1,1,ip_nivm1))
                ELSE
                  ALLOCATE(rga_logp(ip_lono1,ip_lato1,ip_nivm1))
                ENDIF
              ENDIF
              rga_logp(:,:,:)=0.
#ifndef B_COMPU
              IF (il_nprocs.GT.1) THEN
                  il_loopstart=il_myid*(ip_lono1/il_nprocs)+1
                  IF (il_myid .NE. il_nprocs-1) THEN
                      il_loopend=(il_myid+1)*(ip_lono1/il_nprocs)
                  ELSE
                      il_loopend=ip_lono1
                  ENDIF
              ELSE
                  il_loopstart = 1
                  il_loopend = ip_lono1
              ENDIF
              il_lon_on_p = il_loopend - il_loopstart + 1
              il_niv_tri = (ip_nivm1*(ip_nivm1+1))/2
              IF (.NOT.ALLOCATED(rga_exp_sto))
     $            ALLOCATE(rga_exp_sto(il_lon_on_p*ip_lato1*il_niv_tri))
#endif
C
              IF (ll_hybrid_sigma) THEN
C
C** Hybrid sigma coordinate : get alpha and beta coefficients
C
                  ALLOCATE (rla_ALM1(0:ip_nivm1))
                  ALLOCATE (rla_BLM1(0:ip_nivm1))
C
                  cl_space = 'sigma_coeff'
                  il_time = PL_NO_TIME
                  il_usrtag = PL_NO_TAG
                  cl_name = 'ALM1'
                  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $                rla_ALM1,il_err)
C
                  cl_space = 'sigma_coeff'
                  il_time = PL_NO_TIME
                  il_usrtag = PL_NO_TAG
                  cl_name = 'BLM1'
                  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $                rla_BLM1,il_err)
C
              ELSE
C
C** Pure pressure vertical coordinate
C
                  ALLOCATE (rla_presgri(ip_nivm1))
                  ALLOCATE(rla_logp_3d(ip_lono1,ip_lato1,ip_nivm1))
                  rla_logp_3d(:,:,:) = 0.
C
                  cl_space = 'grid_pres'
                  il_time = PL_NO_TIME
                  il_usrtag = PL_NO_TAG
                  cl_name = 'P_GRI'
                  CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $                rla_presgri,il_err)
C
                  DO ilon = 1,ip_lono1
                    DO ilat = 1,ip_lato1
                      DO iniv = 1,ip_nivm1
                        rla_logp_3d(ilon,ilat,iniv) =
     $                      log(rla_presgri(iniv))
                      END DO
                    END DO
                  END DO
C
                  IF (lg_vcor_1D) THEN
                    rga_logp(1,1,:) = SUM(SUM(rla_logp_3d,dim=2),dim=1)
     $                          /(ip_lono1 * ip_lato1)
                  ELSE
                    rga_logp = rla_logp_3d
                  ENDIF
C
                  DEALLOCATE (rla_presgri)
                  DEALLOCATE (rla_logp_3d)
C
                  CALL INIVCOR(il_nprocs,il_myid,assimtime)
C
              ENDIF
C
          ENDIF
C
C
C* Compute Gaussian points and weights
C
C
          ALLOCATE (dga_RLATM1(ip_lato1))
          ALLOCATE (dga_wts(ip_lato1))
          CALL gaqd(ip_lato1, dga_rlatm1, dga_wts, dl_w, lwork, il_err)
C
C* Print information
C
          IF (ig_flsh .NE. 0) THEN
C
              IF (il_myid.eq.0) THEN
                  WRITE(*,'(A24,A30)') '++ HORIZONTAL BKG. ERR. ',
     $                'CORRELATION TAKEN INTO ACCOUNT'
                  WRITE(*,'(A25,I3)') '++ LENGTH SCALE (deg.) = ',
     $                ig_flsh
                  WRITE(*,'(A2)') '++'
              ENDIF
              WRITE(PL_OUT,'(A24,A30)') '++ HORIZONTAL BKG. ERR. ',
     $            'CORRELATION TAKEN INTO ACCOUNT'
              WRITE(PL_OUT,'(A25,I3)') '++ LENGTH SCALE (deg.) = ',
     $            ig_flsh
              WRITE(PL_OUT,'(A2)') '++'
          ENDIF
C
C** Initialize arrays for horizontal diffusion (spherical harmonics) 
C
C* Dimensions to initialize spherepack on a Gaussian grid
C             l1 = min0(nlat,(nlon+2)/2) if nlon is even or
C             l1 = min0(nlat,(nlon+1)/2) if nlon is odd
              l1=min0(ip_latc1,(ip_lonc1+2-mod(ip_lonc1,2))/2)
C             l2 = nlat/2        if nlat is even or
C             l2 = (nlat+1)/2    if nlat is odd
              l2= (ip_latc1+mod(ip_latc1,2))/2
C
#ifndef B_COMPU
C*     Spherepack storing Legendre
              ldim = ip_latc1*(3*(l1+l2)-2)
     $            +(l1-1)*(l2*(2*ip_latc1-l1)-3*l1)/2+ip_lonc1+15
              lwork = 4*ip_latc1*(ip_latc1+2)+2
              ldwork = ip_latc1*(ip_latc1+4)
#else
C*     Spherepack recomputing Legendre
              ldim=ip_latc1*(2*l2+3*l1-2)+3*l1*(1-l1)/2+ip_lonc1+15
              ldwork = ip_latc1*(ip_latc1+4)
#endif
              ALLOCATE(rga_shaw(ldim) )  
              ALLOCATE(rga_shsw(ldim) )  
              ALLOCATE(dwork(ldwork) )
              rga_shaw(:)=0.
              dwork(:)=0.0
C* Initialize spherepack analysis on a Gaussian grid
#ifndef B_COMPU
C*     Spherepack storing Legendre
              ALLOCATE(work(lwork))
              work(:)=0.
              CALL shagsi(ip_latc1,ip_lonc1,rga_shaw,ldim,work,lwork,
     $            dwork,ldwork,il_err)
              DEALLOCATE(work)
#else
C*     Spherepack recomputing Legendre
              CALL shagci(ip_latc1,ip_lonc1,rga_shaw,ldim,dwork,
     $            ldwork,il_err)
#endif
              rga_shsw(:)=0.
              dwork(:)=0.0
C* Initialize spherepack synthesis on a Gaussian grid
#ifndef B_COMPU
C*     Spherepack storing Legendre
              ALLOCATE(work(lwork))
              work(:)=0.
              CALL shsgsi(ip_latc1,ip_lonc1,rga_shsw,ldim,work,lwork,
     $            dwork,ldwork,il_err)
              DEALLOCATE(work)              
#else
C*     Spherepack recomputing Legendre
              CALL shsgci(ip_latc1,ip_lonc1,rga_shsw,ldim,dwork,
     $            ldwork,il_err)
#endif
              DEALLOCATE(dwork)              
C
          ENDIF
C
          FIRSTTIME = .false.
C
C      ENDIF
C
C* Initialize diffusion norme
C
      rg_pi = 4. * ATAN(1.)
C*SM 2008/02/27
      IF (ig_flsh .EQ. 0) THEN
        rg_filno = REAL(ip_trunc + 0) / SQRT(2. * rg_pi) 
      ELSE  
        rg_filno = 2. * sin(REAL(ig_flsh)*rg_pi/360.) 
     $           / SQRT(2.)  *  REAL(ip_trunc + 0)
      ENDIF
C
      IF (ll_hybrid_sigma) THEN
C
C** Reconstruct the 3D log pressure fields for sigma hybrid coordinates 
C
          ALLOCATE(rla_psol(ip_lono1,ip_lato1))
c      
          cl_space = 'ground'
          il_time = assimtime
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_psolm1'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rla_psol,
     $        il_err)
C
          IF (ig_flsz .NE. 0)  THEN
C
              ALLOCATE(rla_pres_int(ip_lono1,ip_lato1,ip_nivm1))
              ALLOCATE(rla_logp_3d(ip_lono1,ip_lato1,ip_nivm1))
C
#ifdef NIV_60
              do ilon=1,ip_lono1
                DO ilat=1,ip_lato1
                  rla_pres_int(ilon,ilat,1)=rla_ALM1(1)+
     &              rla_BLM1(1)*rla_psol(ilon,ilat)
                  rla_logp_3d(ilon,ilat,1)=
     $                log(0.5*rla_pres_int(ilon,ilat,1))
                END DO
              enddo
          
              do iniv=2,ip_nivm1
                do ilon=1,ip_lono1
                  DO ilat=1,ip_lato1
                    XPAPLUS=rla_ALM1(iniv)+rla_BLM1(iniv)*
     $                  rla_psol(ilon,ilat)
                    rla_pres_int(ilon,ilat,iniv)=XPAPLUS
                    XPAMOIN=rla_ALM1(iniv-1)+
     $                  rla_BLM1(iniv-1)*rla_psol(ilon,ilat)
                    rla_logp_3d(ilon,ilat,iniv)=
     $                        log(0.5*(XPAPLUS+XPAMOIN))
                  END DO
                enddo
              enddo
          
#else
              do ilon=1,ip_lono1
                DO ilat=1,ip_lato1
                  rla_logp_3d(ilon,ilat,1)=
     $              ALOG(rla_ALM1(1)+rla_BLM1(1)*rla_psol(ilon,ilat))-1.
                  rla_pres_int(ilon,ilat,1)=rla_ALM1(1)+
     $                rla_BLM1(1)*rla_psol(ilon,ilat)
                END DO
              enddo
c
              do iniv=2,ip_nivm1
                do ilon=1,ip_lono1
                  DO ilat=1,ip_lato1
                    XPAPLUS=rla_ALM1(iniv)+rla_BLM1(iniv)*
     $                  rla_psol(ilon,ilat)
                    rla_pres_int(ilon,ilat,iniv)=XPAPLUS
                    XPAMOIN=rla_ALM1(iniv-1)+
     $                  rla_BLM1(iniv-1)*rla_psol(ilon,ilat)
                    rla_logp_3d(ilon,ilat,iniv)=
     $                  (XPAPLUS*ALOG(XPAPLUS)-XPAMOIN*ALOG(XPAMOIN))
     $                  /(XPAPLUS-XPAMOIN) - 1. 
                  END DO
                enddo
              enddo
c
#endif
              IF (lg_vcor_1D) THEN
                rga_logp(1,1,:) = SUM(SUM(rla_logp_3d,dim=2),dim=1)
     $                      /(ip_lono1 * ip_lato1)
              ELSE
                rga_logp = rla_logp_3d
              ENDIF
              DEALLOCATE(rla_logp_3d)
              DEALLOCATE(rla_pres_int)
C
              CALL INIVCOR(il_nprocs,il_myid,assimtime)
C
          ENDIF
C
          DEALLOCATE(rla_psol)
C
      ENDIF
C
      RETURN
      END
