      SUBROUTINE OBSINIT() 
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 06/12/2006
C
C   Type       : PALM_ROUTINE
C   Purpose    : INITIALISATION OF OBERVATION OPERATORS
C
C Last Modification : A.Piacentini 12/2006
C
CCC
C
      USE palmlib
      USE palm_user_param
      USE GRIDS_OBS_MOD
      USE MRAT_OBS_MOD
      USE MRATLT_MOD
      USE DATDAT_MOD
C
      IMPLICIT NONE
C
C PALM VARIABLES DECLARATION
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      CHARACTER(LEN=8) cl_domname
      REAL rl_rad_2_deg, rl_rlonori
      INTEGER il_time, il_usrtag, il_err, il_err2
C
C LOCAL VARIABLES DECLARATION
C
      INTEGER :: lwork
      DOUBLE PRECISION :: dl_w
      DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: dla_wts
C
      INTEGER il_vertcoord, il_periodic_lon
      INTEGER il_modelgrid_angles
      INTEGER ib
C
C SANITY CHECKS
C
      IF (ip_ninstmax_avk .gt. ip_ninstmax) THEN
          WRITE(*,*) '+ ERROR: number of instruments with av. kernel'
          WRITE(*,*) '  exceeds total number of instruments'
          WRITE(PL_OUT,*) '+ ERROR: number of instruments with
     $        av. kernel'
          WRITE(PL_OUT,*) '  exceeds total number of instruments'
          CALL Palm_Abort(il_err)
      ENDIF
C
      IF (ip_nbniv_avk_out .gt. ip_nbniv_avk_in) THEN
          WRITE(*,*) '+ ERROR: number of levels after averaging kernel'
          WRITE(*,*) '  exceeds number of levels before'
          WRITE(PL_OUT,*) '+ ERROR: number of levels after averaging
     $        kernel'
          WRITE(PL_OUT,*) '  exceeds number of levels before'
          CALL Palm_Abort(il_err)
      ENDIF      
C
      IF (ip_tc_as_prof.eq.1.and.ip_nbnivpresmax.lt.ip_nivm1) THEN
          WRITE(*,*) '+ ERROR: total columns treated as profiles but'
          WRITE(*,*) '  max nb of levels less than model levels'
          WRITE(PL_OUT,*) '+ ERROR: total columns treated as
     $        profiles but'
          WRITE(PL_OUT,*) '  max nb of levels less than model levels'
          CALL Palm_Abort(il_err)
      ENDIF      
C
      IF (ip_fenetre.lt.ip_obs_slot) THEN
          WRITE(*,*) '+ ERROR: observation slot exceeds window size'
          WRITE(PL_OUT,*) '+ ERROR: observation slot exceeds window
     $        size'
          CALL Palm_Abort(il_err)
      ENDIF      
C
      IF (ip_correctime.gt.ip_fenetre) THEN
          WRITE(*,*) '+ ERROR: correction time exceeds window size'
          WRITE(PL_OUT,*) '+ ERROR: correction time exceeds window
     $        size'
          CALL Palm_Abort(il_err)
      ENDIF      
C
      IF (((ip_ninstmax-ip_ninstmax_avk)*ip_nobsperinst_noavk_slot+
     $    ip_ninstmax_avk*ip_nobsperinst_avk_slot).gt.ip_nobstspmax
     $    ) THEN
          WRITE(*,*) '+ WARNING: ip_nobstspmax (max total nr of obs'
          WRITE(*,*) '  per slot) is smaller than the sum of the max'
          WRITE(*,*) '  nr of av. kern. obs per slot + max nr. of obs'
          WRITE(*,*) '  without av, kern. per slot'
          WRITE(PL_OUT,*) '+ WARNING: ip_nobstspmax (max total nr of
     $        obs'
          WRITE(PL_OUT,*) '  per slot) is smaller than the sum of the
     $        max'
          WRITE(PL_OUT,*) '  nr of av. kern. obs per slot + max nr.
     $        of obs'
          WRITE(PL_OUT,*) '  without av, kern. per slot'
      ENDIF      
C
C SWITCH BETWEEN HYBRID SIGMA AND PURE PRESSURE VERTICAL COORD
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'vert_coord'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_vertcoord,
     &    il_err)
C
      lg_hybrid_sigma = il_vertcoord .EQ. 1
C
      IF (ip_iprint .gt. 0) THEN
          IF (lg_hybrid_sigma) THEN
              WRITE(*,'(A24,A32)') '++ OBSINIT WORKING WITH ',
     $            'SIGMA HYBRID VERTICAL COORDINATE'
              WRITE(PL_OUT,'(A24,A32)') '++ OBSINIT WORKING WITH ',
     $            'SIGMA HYBRID VERTICAL COORDINATE'
          ELSE
              WRITE(*,'(A24,A33)') '++ OBSINIT WORKING WITH ',
     $            'PURE PRESSURE VERTICAL COORDINATE'
              WRITE(PL_OUT,'(A24,A33)')'++ OBSINIT WORKING WITH ',
     $            'PURE PRESSURE VERTICAL COORDINATE'
          ENDIF
      ENDIF
C
C SELECT THE MODEL ANGLE REPRESENTATION
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'model_angles'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     &    il_modelgrid_angles,il_err)
C
      IF (ip_iprint .gt. 0) THEN
          IF (il_modelgrid_angles .eq. 1) THEN
              WRITE(*,'(A24,A34)') '++ OBSINIT : MODEL GRID ',
     $            'IN RADIANS. CONVERTING TO DEGREES.'
              WRITE(PL_OUT,'(A24,A34)') '++ OBSINIT : MODEL GRID ',
     $            'IN RADIANS. CONVERTING TO DEGREES.'
          ENDIF
      ENDIF
C
C SWITCH BETWEEN PERIODIC OR LIMITED AREA LONGITUDE 
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'periodic_lon'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     &    il_periodic_lon,il_err)
C
      lg_periodic_lon = il_periodic_lon .EQ. 1
C
      IF (ip_iprint .gt. 0) THEN
          IF (lg_periodic_lon) THEN
              WRITE(*,'(A24,A19)') '++ OBSINIT WORKING WITH ',
     $            'PERIODIC LONGITUDES'
              WRITE(PL_OUT,'(A24,A19)') '++ OBSINIT WORKING WITH ',
     $            'PERIODIC LONGITUDES'
          ELSE
              WRITE(*,'(A24,A23)') '++ OBSINIT WORKING WITH ',
     $            'LIMITED AREA LONGITUDES'
              WRITE(PL_OUT,'(A24,A23)')'++ OBSINIT WORKING WITH ',
     $            'LIMITED AREA LONGITUDES'
          ENDIF
      ENDIF
C
C GLOBAL VARIABLE ALLOCATION
C
      CALL ALLOC_GRIDS_OBS
      CALL ALLOC_MRAT_OBS
      CALL ALLOC_MRATLT
      CALL ALLOC_DATDAT
C
C GEOMETRY FIELDS
C
      cl_space = 'dom_name'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'NAMDOM'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,cl_domname,
     &    il_err)
      cg_domname = TRIM(cl_domname)
C     
C* Compute Gaussian points and weights for observable grid latitudes
C* instead of getting latitude form a communication
C
      ALLOCATE (dla_wts(ip_lato1))
      CALL gaqd(ip_lato1, rga_RLATM1, dla_wts, dl_w, lwork, il_err)
      rga_RLATM1 = rga_RLATM1 - 2. * ATAN(1.)
      DEALLOCATE (dla_wts)
      rl_rad_2_deg = 45./ATAN(1.)
      rga_rlatm1 = rga_rlatm1*rl_rad_2_deg
C
      cl_space = 'one_real'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'RLONM1_ORI'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rl_RLONORI,
     &    il_err)
C
C* Convert to degrees if necessary
C
      IF (il_modelgrid_angles .eq. 1) THEN
          rl_rad_2_deg = 45./ATAN(1.)
          rl_rlonori = rl_rlonori*rl_rad_2_deg
      ENDIF
C
C* Compute observable grid longitudes (1st point as in model grid)
C
      rga_rlonm1(1) = rl_rlonori
      DO ib = 2, ip_lono1
        rga_rlonm1(ib) = rga_rlonm1(ib-1)+360./REAL(ip_lono1)
      END DO
C
C* Fit longitudes in [-180,180) period
C
      DO ib = 1, ip_lono1
        IF (rga_rlonm1(ib).lt.-180) THEN
            rga_rlonm1(ib) =  rga_rlonm1(ib)+360.
        ELSE IF (rga_rlonm1(ib).ge.180) THEN
            rga_rlonm1(ib) =  rga_rlonm1(ib)-360.
        ENDIF
      END DO
C
      IF (lg_hybrid_sigma) THEN
C
C* For sigma hybrid vertical coordinate get the alpha and beta coeffs
C
          cl_space = 'sigma_coeff'
          il_time = PL_NO_TIME
          il_usrtag = PL_NO_TAG
          cl_name = 'ALM1'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rga_ALM1,
     &        il_err)
C
          cl_space = 'sigma_coeff'
          il_time = PL_NO_TIME
          il_usrtag = PL_NO_TAG
          cl_name = 'BLM1'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rga_BLM1,
     &        il_err)
      ELSE
C
C* For pure pressure vertical coordinate get the pressures at grid cell
C* centers and at interfaces
C
          cl_space = 'grid_pres'
          il_time = PL_NO_TIME
          il_usrtag = PL_NO_TAG
          cl_name = 'P_GRI'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rga_PRESGRI,
     &        il_err)
C
          cl_space = 'grid_pres'
          il_time = PL_NO_TIME
          il_usrtag = PL_NO_TAG
          cl_name = 'P_INT'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rga_PRESINT,
     &        il_err)
C
      ENDIF
C
C ASSIMILATED SPECIES
C
#ifdef NIV_60
#ifdef CARIOLLE
#define CPP_O3 'O_3'
#else
#define CPP_O3 'O_x'
#endif
#else
#define CPP_O3 'O_x'
#endif
      WRITE(*,"('++ READING ASSIMILATED SPECIES')")
      WRITE(PL_OUT,"('++ READING ASSIMILATED SPECIES')")
      open( unit = 66,
     $    file = 'assimilated_species',
     $    form = 'formatted')
      DO ib = 1, ip_nassim
        iga_assim_idx(ib)=0
        READ(66,*) cga_assim_spe(ib)
        IF (TRIM(cga_assim_spe(ib)) .eq. 'O_3' .and.
     $      CPP_O3 .ne. 'O_3') THEN
            cga_assim_spe(ib)=CPP_O3
            WRITE(*,*) '+ WARNING CHANGING O_3 TO ',CPP_O3
            WRITE(PL_OUT,*) '+ WARNING CHANGING O_3 TO ',CPP_O3
        ENDIF
        IF (TRIM(cga_assim_spe(ib)) .eq. 'O_x' .and.
     $      CPP_O3 .ne. 'O_x') THEN
            cga_assim_spe(ib)=CPP_O3
            WRITE(*,*) '+ WARNING CHANGING O_x TO ',CPP_O3
            WRITE(PL_OUT,*) '+ WARNING CHANGING O_x TO ',CPP_O3
        ENDIF            
        IF (TRIM(cga_assim_spe(ib)) .eq. 'O_X') THEN
            cga_assim_spe(ib)=CPP_O3
            WRITE(*,*) '+ WARNING CHANGING O_X TO ',CPP_O3
            WRITE(PL_OUT,*) '+ WARNING CHANGING O_X TO ',CPP_O3
        ENDIF            
        TRANAM1obs(ib)=cga_assim_spe(ib)
        iga_assim_idx(ib) = ib
      END DO
      CLOSE(66)
C 
      WRITE(*,*) '+ ASSIMILATING THE FOLLOWING SPECIES :'
      WRITE(PL_OUT,*) '+ ASSIMILATING THE FOLLOWING SPECIES :'
      DO ib = 1, ip_nassim
        WRITE(*,*) '+ ', TRIM(cga_assim_spe(ib)),' in position ',
     $      iga_assim_idx(ib)
        WRITE(PL_OUT,*) '+ ', TRIM(cga_assim_spe(ib)),' in position ',
     $      iga_assim_idx(ib)
      END DO
C
      cl_space = 'assim_names'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'assim_species'
      CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,cga_assim_spe,
     $	   il_err)

C
CCC
C
      END SUBROUTINE OBSINIT
