  SUBROUTINE PLOT_INC()
!
  USE palmlib
  USE palm_user_param
!
!***
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!
!  Centre National de Recherches Meteorologiques
!
!***
!
! Purpose   : Compute dx=U.v where B=U.U^T
!
! Original  : S. Massart, CERFACS/GLOBC, 09/2003
!
!**
!
!      IMPLICIT NONE
!
! Local Declaration
!
  IMPLICIT NONE 
!
  REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: rda_dx
  INTEGER :: assimtime, nbday, nbhour
  INTEGER, DIMENSION(5) :: idate0, idate1
  CHARACTER(LEN=16) , ALLOCATABLE , SAVE :: cla_assim_spe(:)
  CHARACTER(LEN=26) :: clo1
  CHARACTER(LEN=8) , SAVE :: cl_domname
  REAL, DIMENSION(ip_latm1),SAVE :: rla_latm1_r 
  REAL, DIMENSION(ip_lonm1),SAVE :: rla_lonm1_r 
  REAL, DIMENSION(:,:), ALLOCATABLE, SAVE :: rla_psol
  REAL, DIMENSION(:),ALLOCATABLE,SAVE :: rla_ALM1, rla_BLM1
  REAL, DIMENSION(ip_nivm1),SAVE :: rla_presgri_r 
  REAL :: rl_rad_2_deg
  REAL :: rl_ja
  INTEGER :: il_nobs, il_sigmab, il_filterh, il_filterz
  INTEGER :: il_Bvar_rep, il_Bhcor_rep, il_Bvcor_rep, il_vcor_1Dapprox
  LOGICAL, SAVE :: FIRSTTIME = .true.
  LOGICAL, SAVE :: ll_hybrid_sigma
  INTEGER :: il_vertcoord, il_modelgrid_angles
  INTEGER nbjour(12)
  DATA nbjour /31,28,31,30,31,30,31,31,30,31,30,31/
!
! PALM Declaration
!
  CHARACTER(LEN=PL_LNAME) cl_name, cl_space
  INTEGER il_time, il_usrtag, il_err, il_err2
  INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
!
!***
!
!**  ALLOCATE MEMORY
!
    ALLOCATE( rda_dx(ip_lonm1,ip_latm1,ip_nivm1,ip_nassim) )
    IF (FIRSTTIME) THEN
!
! SWITCH BETWEEN HYBRID SIGMA AND PURE PRESSURE VERTICAL COORD
!
	cl_space = 'one_integer'
	il_time = PL_NO_TIME
	il_usrtag = PL_NO_TAG
	cl_name = 'vert_coord'
	CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_vertcoord,&
	    il_err)
!
	ll_hybrid_sigma = il_vertcoord .EQ. 1
!
	IF (ip_iprint .gt. 0) THEN
	    IF (ll_hybrid_sigma) THEN
		WRITE(*,'(A24,A32)') '++ PLOT_INC WORKING WITH ',&
		   'SIGMA HYBRID VERTICAL COORDINATE'
		WRITE(PL_OUT,'(A24,A32)') '++ PLOT_INC WORKING WITH ',&
		   'SIGMA HYBRID VERTICAL COORDINATE'
	    ELSE
		WRITE(*,'(A24,A33)') '++ PLOT_INC WORKING WITH ',&
		   'PURE PRESSURE VERTICAL COORDINATE'
		WRITE(PL_OUT,'(A24,A33)') '++ PLOT_INC WORKING WITH ',&
		   'PURE PRESSURE VERTICAL COORDINATE'
	    ENDIF
	ENDIF
!
! SELECT THE MODEL ANGLE REPRESENTATION
!
	cl_space = 'one_integer'
	il_time = PL_NO_TIME
	il_usrtag = PL_NO_TAG
	cl_name = 'model_angles'
	CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,&
	    il_modelgrid_angles,il_err)
!
	IF (il_modelgrid_angles .eq. 1 .and. ip_iprint .gt. 0) THEN
	    WRITE(*,'(A25,A34)') '++ PLOT_INC : MODEL GRID ',&
	            'IN RADIANS. CONVERTING TO DEGREES.'
	    WRITE(PL_OUT,'(A25,A34)') '++ PLOT_INC : MODEL GRID ',&
	            'IN RADIANS. CONVERTING TO DEGREES.'
	ENDIF
!
! GEOMETRY FIELDS
!
	cl_space = 'grid_lat'
	il_time = PL_NO_TIME
	il_usrtag = PL_NO_TAG
	cl_name = 'RLATM1'
	CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rla_latm1_r,&
	   il_err)
!
	cl_space = 'grid_lon'
	il_time = PL_NO_TIME
	il_usrtag = PL_NO_TAG
	cl_name = 'RLONM1'
	CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rla_lonm1_r,&
	    il_err)
!
!* Convert to degrees if necessary
!
	IF (il_modelgrid_angles .eq. 1) THEN
	    rl_rad_2_deg = 45./ATAN(1.)
	    rla_latm1_r = rla_latm1_r*rl_rad_2_deg
	    rla_lonm1_r = rla_lonm1_r*rl_rad_2_deg
	ENDIF
!
	IF (ll_hybrid_sigma) THEN
!
	    rla_presgri_r(:)=0.
!
	    ALLOCATE (rla_ALM1(0:ip_nivm1))
	    ALLOCATE (rla_BLM1(0:ip_nivm1))
            ALLOCATE (rla_psol(ip_lonm1,ip_latm1))
!
	    cl_space = 'sigma_coeff'
	    il_time = PL_NO_TIME
	    il_usrtag = PL_NO_TAG
	    cl_name = 'ALM1'
	    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag, &
	       rla_ALM1,il_err)
!
	    cl_space = 'sigma_coeff'
	    il_time = PL_NO_TIME
	    il_usrtag = PL_NO_TAG
	    cl_name = 'BLM1'
	    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag, &
	       rla_BLM1,il_err)	    
!
	ELSE
!	  
!* For pure pressure vertical coordinate get the pressures at grid cell
!* centers
!
	    cl_space = 'grid_pres'
	    il_time = PL_NO_TIME
	    il_usrtag = PL_NO_TAG
	    cl_name = 'P_GRI'
	    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,&
	       rla_presgri_r,il_err)
	END IF
!
! RECEIVE THE NAMES OF THE ASSIMILATED SPECIES
!
	ALLOCATE( cla_assim_spe(ip_nassim) )
!
	cl_space = 'dom_name'
	il_time = PL_NO_TIME
	il_usrtag = PL_NO_TAG
	cl_name = 'NAMDOM'
	CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,cl_domname,&
	   il_err)
!
	cl_space = 'assim_names'
	il_time = PL_NO_TIME
	il_usrtag = PL_NO_TAG
	cl_name = 'assim_species'
	CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,cla_assim_spe,&
	   il_err)
!
	FIRSTTIME=.false.
!
    ENDIF
!
!**  GET CONTROL VECTOR dx
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'id_assimtime'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,assimtime,il_err) 
!
    cl_space = 'date'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'ida_date0'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,idate0,il_err) 
!
    cl_space = 'increment_space'
    il_time = assimtime
    il_usrtag = PL_NO_TAG
    cl_name = 'rda_dx'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rda_dx,il_err) 
!
!**  GET SURFACE PRESSURE FIELD (if hybrid coordinate)
!
      IF (ll_hybrid_sigma) THEN
!
          cl_space = 'ground'
          il_time = assimtime
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_psolm1'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rla_psol, &
             il_err)
!
      END IF
!
!**  GET ASSIMILATION STATISTICS
!
    cl_space = 'one_real'
    il_time = assimtime
    il_usrtag = PL_NO_TAG
    cl_name = 'J_xa'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rl_ja,il_err) 
!
    cl_space = 'one_integer'
    il_time = assimtime
    il_usrtag = PL_NO_TAG
    cl_name = 'number_obs'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_nobs,il_err) 
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'il_Bvar_rep'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_Bvar_rep,il_err) 
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'il_sigmab'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_sigmab,il_err) 
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'il_Bhcor_rep'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_Bhcor_rep,il_err) 
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'filter_h'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_filterh,il_err) 
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'il_Bvcor_rep'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_Bvcor_rep,il_err) 
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'filter_z'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_filterz,il_err) 
!
    cl_space = 'one_integer'
    il_time = PL_NO_TIME
    il_usrtag = PL_NO_TAG
    cl_name = 'il_vcor_1Dapprox'
    CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_vcor_1Dapprox,il_err) 
!
!** WRITE NETCDF FILE
!
    NBDAY=INT(assimtime/86400.)
    NBHOUR=INT((assimtime-REAL(NBDAY)*86400.)/3600.)
    idate1(5)=0
    idate1(4)=idate0(4)+NBHOUR
    idate1(3)=idate0(3)+NBDAY
    idate1(2)=idate0(2)
    idate1(1)=idate0(1)
    IF (idate1(4).GE.24) THEN
	idate1(4)=idate1(4)-24
	idate1(3)=idate1(3)+1
    ENDIF
    IF (mod(idate0(1),4).EQ.0) THEN
	nbjour(2)=29
    ENDIF
    IF (idate1(3).GT.nbjour(idate1(2))) THEN
	idate1(3)=idate1(3)-nbjour(idate1(2))
	idate1(2)=idate1(2)+1
    ENDIF
    IF (idate1(2).GT.12) THEN
	idate1(1)=idate1(1)+1
	idate1(2)=1
    ENDIF

    WRITE(clo1, &
       fmt='(''DX'',A6,''+'',I4.4,I2.2,I2.2,I2.2,''+O'',I2.2,''.nc'')') &
       TRIM(cl_domname),idate1(1),idate1(2),idate1(3),idate1(4), &
       0
!
!** For the moment, the name file has a outer update number set to 0.
!   It will change when we'll implement the full incremental nested loops.
!
    CALL MCG2NC(clo1,idate0,assimtime,cla_assim_spe,'ppb',rda_dx, &
    ll_hybrid_sigma, rla_lonm1_r, rla_latm1_r, rla_presgri_r, &
    rla_ALM1, rla_BLM1, rla_psol, rl_ja, il_nobs, &
    il_sigmab, il_filterh, il_filterz, il_Bvar_rep, il_Bhcor_rep, &
    il_Bvcor_rep, il_vcor_1Dapprox)
!
!**  DEALLOCATE MEMORY
!
    DEALLOCATE(rda_dx)
!         
!**
!
  RETURN
  END
