      SUBROUTINE SQRTB()
C
      USE palmlib
      USE palm_user_param
      USE sqrtB_mod
C
C***
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C  MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C
C  Centre National de Recherches Meteorologiques
C
C***
C
C Purpose   : Compute dx=U.v where B=U.U^T
C
C Original  : S. Massart, CERFACS/GLOBC, 09/2003
C
C**
C
      IMPLICIT NONE
C
#include "mpif.h"
C
C Local Declaration
C
      INTEGER assimtime, il_restime
      REAL daynum, daynum0
      REAL, DIMENSION(:), ALLOCATABLE :: rda_v
      REAL, DIMENSION(:), ALLOCATABLE :: rda_sigmab
      INTEGER :: il_myid, il_nprocs
C
C PALM Declaration
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
C*** PARALLELISM
C
      CALL MPI_COMM_RANK(PL_COMM_EXEC,il_myid,il_err)
      CALL MPI_COMM_SIZE(PL_COMM_EXEC,il_nprocs,il_err)
C
C**  ALLOCATE MEMORY
C
      ALLOCATE( rda_v(ip_dimx) )
      ALLOCATE( rda_sigmab(ip_dimx) )
C
C**  GET CONTROL VECTOR v
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'id_assimtime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,assimtime,il_err)
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'id_resulttime'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,il_restime,
     $    il_err)
C
      rda_v(:) = 0
C
      IF (il_myid .eq. 0) THEN
          cl_space = 'crt_space'
          il_time = assimtime
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_v'
          CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,rda_v,il_err) 
      ENDIF
C 
C** Broadcast to other procs, since PALM_RESEARCH does not allow
C   SINGLE PROC comms to ALL target processes from an algebra unit
C
      IF (il_nprocs .gt. 1) THEN
          CALL MPI_Bcast(rda_v,ip_dimx,MPI_DOUBLE_PRECISION,0,
     $        PL_COMM_EXEC,il_err)
      ENDIF
C  
      cl_space = 'increment_space'
      il_time = assimtime
      il_usrtag = PL_NO_TAG
      cl_name = 'rda_sigmab'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,
     $    rda_sigmab,il_err) 
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'filter_h'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,ig_flsh,
     $    il_err)
C
      cl_space = 'one_integer'
      il_time = PL_NO_TIME
      il_usrtag = PL_NO_TAG
      cl_name = 'filter_z'
      CALL PALM_Get(cl_space,cl_name,il_time,il_usrtag,ig_flsz,
     $    il_err)
C  
C**  APPLY THE NORM W^(-1/2) ON THE SPHERE
C
      CALL NORME_SPHERE(rda_v)
C  
C**  COMPUTE THE HORIZONTAL CORRELATION
C
C*SM 2008/02/27
C      IF (ig_flsh .NE. 0) THEN
          IF (MAXVAL(ABS(rda_v)).gt.2.*TINY(1.d0)) THEN
              CALL HOR_COREL_DIR(rda_v,il_nprocs,il_myid)
          ENDIF
C      ENDIF  
C
C**  COMPUTE THE VERTICAL CORRELATION
C
      IF (ig_flsz .NE. 0) THEN
#ifndef B_COMPU
          IF (MAXVAL(ABS(rda_v)).gt.2.*TINY(1.d0)) THEN
              CALL VERT_COREL_DIR(rda_v,il_nprocs,il_myid)
          ENDIF
#else
          CALL VERT_COREL_DIR(rda_v,il_nprocs,il_myid)
#endif
      ENDIF
C
C**  COMPUTE dx=U.v (TO BE EXTENDED)
C
      rda_v(:) = rda_v(:) * rda_sigmab(:)
C
C**  PUT VECTOR dx TO COMPUTE H.dx 
C
      IF (il_myid .eq. 0) THEN
          cl_space = 'increment_space'
          il_time = il_restime
          il_usrtag = PL_NO_TAG
          cl_name = 'rda_dx'
          CALL PALM_Put(cl_space,cl_name,il_time,il_usrtag,rda_v,
     $        il_err) 
      END IF
C
C**  DEALLOCATE MEMORY
C
      DEALLOCATE(rda_v)
      DEALLOCATE(rda_sigmab)
C         
C**
C
      RETURN
      END
