SUBROUTINE PL_Time_inter_real(cd_space, id_time, id_time1, rda_op1, &
                        id_time2, rda_op2, id_size)
!
!**** PL_Time_inter_real - Custom defined time interpolation for real objects
!
!     Purpose:
!     --------
!       This subroutine must be defined by PALM users when they want to define
!       their own time interpolation algorithm for real objects
!       THIS FILE IS JUST AN EXAMPLE FILE
!      
!***  Interface: 
!     ----------
!       CALL PL_Time_inter_real(cd_space, id_time, id_time1, rda_op1, 
!                                 id_time2, rda_op2, id_size)
!
!     Input:
!     ------
!   egin{itemize}
!     item cd_space         Handle of the space
!     item id_time          Time of the computed result
!     item id_time1         Time stamp of the first object
!     item rda_op1          First operand
!     item id_time2         Time stamp of the secnd object
!     item rda_op2          Second operand
!     item id_size          Size of the array
!   end{itemize}
!
!     Output:
!     -------
!     rda_op1          The result of the operation
!
!     Method:
!     -------
!
!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib
!
!     History:
!     --------
!       Version     Programmer      Date        Description
!       -------     ----------      ----        -----------
!       1.0         massart         04/14/05     Created
!
!*-----------------------------------------------------------------------
!
!** 1. DECLARATIONS
!      ------------
!
!** 1.1 MODULES
!
  USE palmlib
!
!** 1.2 Dummy arguments
!
  IMPLICIT NONE
!
  CHARACTER(LEN=PL_LNAME), INTENT(in) :: cd_space
  INTEGER, INTENT(in) :: id_size
  INTEGER, INTENT(in)    :: id_time
  INTEGER, INTENT(in)    :: id_time1
  REAL, DIMENSION(id_size), INTENT(inout) :: rda_op1
  INTEGER, INTENT(in)    :: id_time2
  REAL, DIMENSION(id_size), INTENT(in) :: rda_op2
!
!*----------------------------------------------------------------------
!
!** 2. Do something for space cd_space with time and objects
!      -----------------------------------------------------
!
!**     USER DEFINED
!
  WRITE(PL_OUT,*) 'interpolation for reals'
  CALL flush(PL_OUT)
  SELECT CASE (cd_space)
  CASE('vect_space.producer')
      rda_op1 = (rda_op1 + rda_op2)/2.
  CASE default
      rda_op1 = rda_op2
  END SELECT
  WRITE(PL_OUT,*) 'interpolation done for reals'
  CALL flush(PL_OUT)
!
!*----------------------------------------------------------------------
!
END SUBROUTINE PL_Time_inter_real
!
!*======================================================================
!
SUBROUTINE PL_Time_inter_dble(cd_space, id_time, id_time1, dda_op1, &
                       id_time2, dda_op2, id_size)
!
!**** PL_Time_inter_dble - Custom defined time int. for dble objects 
!
!     Purpose:
!     --------
!       for double objects
!       This subroutine must be defined by PALM users when they want to define
!       their own time interpolation algorithm for double objects 
!       THIS FILE IS JUST AN EXAMPLE FILE
!
!***  Interface: 
!     ----------
!      CALL PL_Time_inter_dble(cd_space, id_time, id_time1, dda_op1, 
!                id_time2, dda_op2, id_size)
!
!     Input:
!     ------
!  egin{itemize}
!     item  cd_space         Handle of the space
!     item  id_time          Time of the computed result
!     item  id_time1         Time stamp of the first object
!     item  dda_op1          First operand
!     item  id_time2         Time stamp of the secnd object
!     item  dda_op2          Second operand
!     item  id_size          Size of the array
!  end{itemize}
!
!     Output:
!     -------
!     dda_op1          The result of the operation
!
!     Method:
!     -------
!
!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib
!
!     History:
!     --------
!       Version     Programmer      Date        Description
!       -------     ----------      ----        -----------
!       1.0         AP         1998/12/24  Created
!
!*-----------------------------------------------------------------------
!
!** 1. DECLARATIONS
!      ------------
!
!** 1.1 MODULES
!
  USE palmlib
!
!** 1.2 Dummy arguments
!
  IMPLICIT NONE
!
  CHARACTER(LEN=PL_LNAME), INTENT(in) :: cd_space
  INTEGER, INTENT(in) :: id_size
  INTEGER, INTENT(in)    :: id_time
  INTEGER, INTENT(in)    :: id_time1
  REAL(KIND=KIND(1.d0)), DIMENSION(id_size), INTENT(inout) :: dda_op1
  INTEGER, INTENT(in)    :: id_time2
  REAL(KIND=KIND(1.d0)), DIMENSION(id_size), INTENT(in) :: dda_op2
!
!*----------------------------------------------------------------------
!
!** 2. Do something for space cd_space with time and objects
!      -----------------------------------------------------
!
!     USER DEFINED
!
  WRITE(PL_OUT,*) 'interpolation for doubles'
  CALL flush(PL_OUT)
  dda_op1 = (dda_op1 + dda_op2)/2.
  WRITE(PL_OUT,*) 'interpolation done for doubles'
  CALL flush(PL_OUT)
!
!*----------------------------------------------------------------------
!
END SUBROUTINE PL_Time_inter_dble
!
!*======================================================================
!
SUBROUTINE PL_Time_inter_cpx(cd_space, id_time,id_time1,yda_op1,id_time2, &
           yda_op2, id_size)
!
!****  PL_Time_int_cpx - Custom defined time int. for complex objects
!     Purpose:
!     --------
!       This subroutine must be defined by PALM users when they want to define
!       their own time interpolation algorithm for complex objects 
!       THIS FILE IS JUST AN EXAMPLE FILE
!
!
!***  Interface: 
!     ----------
!       CALL PL_Time_inter_real(cd_space, id_time, id_time1, yda_op1, 
!                 id_time2, yda_op2, id_size)
!
!     Input:
!     ------
!    egin{itemize}
!     item cd_space         Handle of the space
!     item id_time          Time of the computed result
!     item id_time1         Time stamp of the first object
!     item yda_op1          First operand
!     item id_time2         Time stamp of the secnd object
!     item yda_op2          Second operand
!     item id_size          Size of the array
!  end{itemize}
!
!     Output:
!     -------
!     yda_op1          The result of the operation
!
!     Method:
!     -------
!
!     Externals:
!     ----------
!       Modules:
!       --------
!          palmlib
!
!     History:
!     --------
!       Version     Programmer      Date        Description
!       -------     ----------      ----        -----------
!       1.0         AP         1998/12/24  Created
!
!*-----------------------------------------------------------------------
!
!** 1. DECLARATIONS
!      ------------
!
!** 1.1 MODULES
!
  USE palmlib
!
!** 1.2 Dummy arguments
!
  IMPLICIT NONE
!
  CHARACTER(LEN=PL_LNAME), INTENT(in) :: cd_space
  INTEGER, INTENT(in) :: id_size
  INTEGER, INTENT(in)    :: id_time
  INTEGER, INTENT(in)    :: id_time1
  COMPLEX, DIMENSION(id_size), INTENT(inout) :: yda_op1
  INTEGER, INTENT(in)    :: id_time2
  COMPLEX, DIMENSION(id_size), INTENT(in) :: yda_op2
!
!*----------------------------------------------------------------------
!
!** 2. Do something for space cd_space with time and objects
!      -----------------------------------------------------
!
!     USER DEFINED
!
  WRITE(PL_OUT,*) 'interpolation for complexes'
  CALL flush(PL_OUT)
  yda_op1 = (yda_op1 + yda_op2)/2.
  WRITE(PL_OUT,*) 'interpolation done for complexes'
  CALL flush(PL_OUT)
!
!*----------------------------------------------------------------------
!
END SUBROUTINE PL_Time_inter_cpx
!
!*======================================================================
!

