MODULE palm_user_param
!
!**** palm_user_param - Declaration of user parameters
!
!     Purpose:
!     --------
!     To provide the mnemonic keys for user subroutine call
!
!     Interface:
!     ---------
!       USE palm_user_param
!
!***
!     History:
!     --------
!       Version     Programmer      Date        Description
!       -------     ----------      ----        -----------
!       1.0         Anne           1999/08/08    Created
!       2.0         Thierry        2000/10/17    added type of variable
!*----------------------------------------------------------------------
!
!**  1. DECLARATIONS
!
!
!**  1.1  Dimension
!
  INTEGER, PARAMETER :: ip_lonm1 = 180 ! Number of model grid longitudes 
  INTEGER, PARAMETER :: ip_latm1 = 90 ! Number of model grid latitudes 
  INTEGER, PARAMETER :: ip_nivm1 = 60 ! Number of model grid vertical levels 
  INTEGER, PARAMETER :: ip_lonc1 = ip_lonm1 ! Number of control grid longitudes 
  INTEGER, PARAMETER :: ip_latc1 = ip_latm1 ! Number of control grid latitudes 
  INTEGER, PARAMETER :: ip_gauss_modelgrid = 0 ! Model grid type: 0 if regular, 1 if Gauss (e.g. MOCAGE T42) 
  INTEGER, PARAMETER :: ip_ntra1 = 1 ! Number of transported species 
  INTEGER, PARAMETER :: ip_nsls1 = 0 ! Number of short-lived species 
  INTEGER, PARAMETER :: ip_nassim = 1 ! Number of assimilated transported species 
  INTEGER, PARAMETER :: ip_ninstmax = 1 ! Max number of instruments for one species 
  INTEGER, PARAMETER :: ip_ninstmax_avk = 0 ! Max number of instruments with avg. kernel (AVKERN, INTQTY) 
  INTEGER, PARAMETER :: ip_nobsmax = 2000 ! Maximun number of observation each day 
  INTEGER, PARAMETER :: ip_nobstspmax = 200 ! Maximun number of observation each slot 
  INTEGER, PARAMETER :: ip_nobsperinst_noavk_slot = 200 ! Maximun nb of obs in a slot per instr. without av. kern. 
  INTEGER, PARAMETER :: ip_nobsperinst_avk_slot = 0 ! Maximun nb of obs in a slot per instr. with av. kern. 
  INTEGER, PARAMETER :: ip_nbnivpresmax = 18 ! Maximum number of pressure levels per profile 
  INTEGER, PARAMETER :: ip_nbniv_avk_in = 7 ! Maximum number of levels before application of avg. kern. 
  INTEGER, PARAMETER :: ip_nbniv_avk_out = 7 ! Maximum number of levels after application of avg. kern. 
  INTEGER, PARAMETER :: ip_tc_as_prof = 0 ! Tot col treatment: 0 as scalar, 1 as shifted profiles 
  INTEGER, PARAMETER :: ip_nbconstmax = 1 ! Maximum number of constituants in obs 
  INTEGER, PARAMETER :: ip_latobsmin = -90 ! Observations north of this latitude are discarded 
  INTEGER, PARAMETER :: ip_latobsmax = 90 ! Observations south of this latitude are discarded 
  INTEGER, PARAMETER :: ip_lonobsmin = -180 ! Observation west of this lon. are discarded. Use [-180,180] period 
  INTEGER, PARAMETER :: ip_lonobsmax = 180 ! Observation east of this lon. are discarded. Use [-180,180] period 
  INTEGER, PARAMETER :: ip_presobsmin = 0 ! Observations over this pressure (Pa) are discarded 
  INTEGER, PARAMETER :: ip_presobsmax = 150000 ! Observations under this pressure (Pa) are discarded 
  INTEGER, PARAMETER :: ip_enforced_obs_min = 0 ! Min enforced observation value  ** in ppb ** 
  INTEGER, PARAMETER :: ip_enforced_ana_min = 0 ! Min enforced analysis field value  ** in ppb ** 
  INTEGER, PARAMETER :: ip_scrip = 0 ! Grid remapping: 1 for SCRIP, 0 for SPHEREPACK 
  INTEGER, PARAMETER :: ip_nbfenet = 1 ! Number of assimilation windows 
  INTEGER, PARAMETER :: ip_fenetre = 3*3600 ! Size (in sec.) of each assmilation window 
  INTEGER, PARAMETER :: ip_obs_slot = 1*3600 ! Observ. time slot i.e. frequency of model puts (secs.) 
  INTEGER, PARAMETER :: ip_correctime = ip_fenetre * 3/3 ! Time where the correction is added 
  INTEGER, PARAMETER :: ip_insincr = 0 ! If ip_insincr = 1, the increment is added linearly over the assimilation windows 
  INTEGER, PARAMETER :: ip_niter = 100 ! Max number of iterations of the minimizer 
  INTEGER, PARAMETER :: ip_omf = 1 ! ip_omf = 1 to output obs. minus forecast diags 
  INTEGER, PARAMETER :: ip_oma = 0 ! ip_oma = 1 to output obs. minus analysis diags 
  INTEGER, PARAMETER :: ip_omd = 0 ! ip_omd = 1 to output obs. minus dry run spanning the whole assim. exp. diags 
  INTEGER, PARAMETER :: ip_restart_on_dsk = 0 ! if = 0 keep old i.c. in palm, if =1 store it on disk 
  INTEGER, PARAMETER :: ip_iprint = 0 ! Toggle diagnostic output 
  INTEGER, PARAMETER :: ip_loader_verbosity = 0 ! Obs loader verbosity : 0 = silent, 1 = rejection info, 2 complete info 
  INTEGER, PARAMETER :: ip_assimsize = ip_nbfenet*ip_fenetre ! Size (in sec.) of global assmilation window 
  INTEGER, PARAMETER :: ip_lono1 = ip_lonc1 ! Number of observable grid longitudes 
  INTEGER, PARAMETER :: ip_lato1 = ip_latc1 ! Number of observable grid latitudes 
  INTEGER, PARAMETER :: ip_dimx = ip_lonc1*ip_latc1*ip_nivm1*ip_nassim ! Vector size of transported species 
  INTEGER, PARAMETER :: ip_dimobs = ip_nobstspmax*ip_nbnivpresmax*ip_nbconstmax ! Dimension of observation space 

END MODULE palm_user_param
!
!*======================================================================
!

