      PROGRAM top_boundary_files
C
CCCCC
C
C   Lecture des fichiers climatologiques REPROBUS-2D
C   [WMO, 1998] pour l'atmosphere de 1995 et ecriture
C   au format MOCAGE afin de contraindre les flux pour
C   les especes chimiques stratospheriques
C
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C   Centre National de Recherches Meteorologiques
C   Groupe de Modelisation de Grande Echelle et du Climat
C   Equipe de Recherches sur l'Atmosphere Moyenne
C
CCCCC
C
C  Original : H. Teyssedre, 02/2001
C
C  Modified :
C
CCCCC
C
C
CCC
C
C  Parametres principaux
C
C  Entrees : DATEIN  (date du debut de la simulation MOCAGE)
C  --------  DATEOUT (date de fin)
C
C  Sorties :
C  -------- 
C
      parameter (nlon = 64, nlat= 34, nlev =41)
      parameter (ntra=40)

      parameter (pbottom=500., ptop=400.)

      integer datein(4), dateout(4)
      integer lenmonths(12)

      data lenmonths / 31,28,31,30,31,30,31,31,30,31,30,31 /

      character*2  cmma, cmmb
      character*20 cnamea, cnameb

      character*18 cbegname

      data cbegname / 'CHEMA1995_19941998' /

      real xlat(nlat)

      data xlat /
     + -90.00,
     + -85.76,-80.26,-74.74,-69.22,-63.68,-58.14,-52.60,-47.07,
     + -41.53,-35.99,-30.46,-24.92,-19.38,-13.84, -8.31, -2.77,
     +   2.77,  8.31, 13.84, 19.38, 24.92, 30.46, 35.99, 41.53,
     +  47.07, 52.60, 58.14, 63.68, 69.22, 74.74, 80.26, 85.76,
     +  90.00/

      real pma(nlat,nlev), qqma(nlat,nlev,ntra)
      real pmb(nlat,nlev), qqmb(nlat,nlev,ntra)
      real pm (nlat,nlev), qqm (nlat,nlev,ntra)
      real fqm(nlat,ntra)
C
C LIST OF CHEMICAL SPECIES USED TO CONSTRAIN MODEL TOP
C THE RESULTS ARE FROM A CLIMATOLOGY CALCULATED WITH
C ARPEGE/REPROBUS FOR 1995 ATMOSPHERE AND ZONALLY AVERAGED.
C MORE DETAILS ABOUT THIS SIMULATION CAN BE FOUND IN :
C WMO-98, CHAPTER 12.
C
      character*15 tranam(ntra)

      data tranam / 'N_2O           ','CH_4           ',
     +              'H_2O           ','NO_y           ',
     +              'HNO_3          ','N_2O_5         ',
     +              'Cl_x           ','O_x            ',
     +              'CO             ','OClO           ',
     +              'PSC            ','H_2SO_4        ',
     +              'HCl            ','ClONO_2        ',
     +              'HOCl           ','Cl_2           ',
     +              'H_2O_2         ','ClNO_2         ',
     +              'HBr            ','BrONO_2        ',
     +              'NO_x           ','HNO_4          ',
     +              'ClO_x          ','BrO_x          ',
     +              'Cl_2O_2        ','HOBr           ',
     +              'BrCl           ','HCHO           ',
     +              'MO2            ','OP1            ',
     +              'CFC11          ','CFC12          ',
     +              'CFC113         ','CCl_4          ',
     +              'CH_3CCl_3      ','CH_3Cl         ',
     +              'HCFC22         ','CH_3Br         ',
     +              'H1211          ','H1301          '
     +            /

      logical lreverse

      lreverse = .FALSE.

      if (xlat(nlat).lt.xlat(1)) lreverse = .TRUE.

      read (*,'(i4,3i2)') (datein (i),i=1,4)
      read (*,'(i4,3i2)') (dateout(i),i=1,4)

      if (mod(datein(1),4).eq.0) lenmonths(2) = 29
C
C determine date in the middle of the time period
C
      if (dateout(2).eq.datein(2)) then
        xmday = real(dateout(3)+datein(3))*0.5
      else
        idayout = dateout(3) + lenmonths(datein(2))
        xmday   = real(idayout+datein(3))*0.5
      endif

      mday = int(xmday)

      if (mday.le.lenmonths(datein(2))) then
        mmonth = datein(2)
      else
        mmonth = datein(2) + 1
        if (mmonth.gt.12) mmonth = 1
        mday = mday - lenmonths(datein(2))
      endif

      if (mday.le.15) then
        mma = mmonth - 1
        if (mma.lt.1) mma = 12
        mmb = mmonth
      else
        mma = mmonth
        mmb = mmonth + 1
        if (mmb.gt.12) mmb = 1
      endif

      write (cmma,'(i2.2)') mma
      write (cmmb,'(i2.2)') mmb

      cnamea = cbegname//cmma
      cnameb = cbegname//cmmb

      call readata(cnamea,pma,qqma)
      call readata(cnameb,pmb,qqmb)
c
c interpolation temporelle
c
      dday = real(lenmonths(mma))

      if (mday.gt.15) then
        fact = real(mday-15)/dday
      else
        fact = real(lenmonths(mma)-15-mday)/dday
      endif

      do ilat=1,nlat*nlev
        pm(ilat,1) = fact *pma(ilat,1) +
     +           (1.-fact)*pmb(ilat,1)
      enddo

      do ilat=1,nlat*nlev*ntra
        qqm(ilat,1,1) = fact *qqma(ilat,1,1) +
     +              (1.-fact)*qqmb(ilat,1,1)
      enddo

c
c ecriture du fichier
c
      do ilat=1,nlat

        ifm = 0
        do itra=1,ntra
          fqm(ilat,itra) = 0.
        enddo

        do ilev=1,nlev
          if ((pm(ilat,ilev).ge.ptop   ).and.
     +        (pm(ilat,ilev).le.pbottom)) then

            ifm = ifm + 1
            do itra=1,ntra
              fqm(ilat,itra) = fqm(ilat,itra) + qqm(ilat,ilev,itra)
            enddo

          endif
        enddo

        if (ifm.gt.0) then
          do itra=1,ntra
            fqm(ilat,itra) = fqm(ilat,itra) / real(ifm)
          enddo
        else
          do itra=1,ntra
            fqm(ilat,itra) = -999.
          enddo
        endif

      enddo

      XPI=4.*ATAN(1.)
      do ilat=1,nlat
        xlat(ilat)=xlat(ilat)*XPI/180.
      enddo

      open  (10, file='topbd', form='unformatted')

      write (10) nlat,ntra
      write (10) tranam

      if (lreverse) then
        write (10) (xlat(ilat),ilat=nlat,1,-1)
     +  ,((fqm(ilat,itra),ilat=nlat,1,-1),itra=1,ntra)
      else
        write (10) xlat,fqm
      endif

      close (10)

      STOP
      END
c
ccccc
c
      subroutine readata(cfilename,zpm,zqm)

      parameter (nlon = 64, nlat= 34, nlev =41)
      parameter (ntra=40)

      character*20 cfilename
c
c tableaux de la grille ARPROBUS
c
      real psol(nlon,nlat), zpsol(nlat), zpm(nlat,nlev)
      real um(nlon,nlat,nlev), vm(nlon,nlat,nlev),
     +     wm(nlon,nlat,nlev), tm(nlon,nlat,nlev)
      real qm(nlon,nlat,nlev,ntra), zqm(nlat,nlev,ntra)
c
c tableaux de definition des niveaux ARPROBUS (suffixe 41)
c
      real zva41(0:nlev), zvb41(0:nlev)

      data zva41 /
     +     0.,
     +     2.,    37.,   107.,   195.,   294.,   405.,   528.,   667.,
     +   823.,  1000.,  1202.,  1432.,  1695.,  1997.,  2342.,  2739.,
     +  3193.,  3715.,  4313.,  5000.,  5627.,  6051.,  6289.,  6362.,
     +  6286.,  6081.,  5765.,  5357.,  4875.,  4338.,  3764.,  3171.,
     +  2579.,  2006.,  1470.,   990.,   585.,   272.,    71.,     0.,
     +     0./

      data zvb41 /
     + 0.0000,
     + 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
     + 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000, 0.0000,
     + 0.0000, 0.0000, 0.0000, 0.0000, 0.0070, 0.0260, 0.0560, 0.0970,
     + 0.1450, 0.2010, 0.2630, 0.3290, 0.3990, 0.4700, 0.5420, 0.6130,
     + 0.6810, 0.7470, 0.8070, 0.8610, 0.9080, 0.9460, 0.9740, 0.9900,
     + 1.0000/

      open (1, file=cfilename,form="unformatted")
c
c lecture de la climatologie ARPROBUS
c
      read (1) ian,imois,psol,um,vm,wm,tm,qm

      close (1)

      write (*,*) "fin de lecture du fichier ",cfilename
c
c ATTENTION : psol est exprime en hPa dans ARPROBUS, convertion en Pa
c
      do j=1,nlon
         do k=1,nlat
            psol(j,k) = 100. * psol(j,k)
         enddo
      enddo
c
c calcul des moyennes zonales
c
c ATTENTION : dans ARPROBUS, les variables sont rangees du 
c             pole Nord vers le pole Sud contrairement a MOCAGE
c
      do k=1,nlat
         zpsol(k) = 0.
         do j=1,nlon
            zpsol(k) = zpsol(k) + psol(j,nlat-k+1)/real(nlon)
         enddo
      enddo

      do n=1,ntra
         do l=1,nlev
            do k=1,nlat
               zqm(k,l,n) = 0.
               do j=1,nlon
                  zqm(k,l,n) = zqm(k,l,n) +
     +                         qm(j,nlat-k+1,l,n)/real(nlon)
               enddo
            enddo
         enddo
      enddo
c
c definition de la grille ARPROBUS (41 niveaux)
c
      do k=1,nlat
         do l=1,nlev
            aaa = 0.5 * (zva41(l-1) + zva41(l))
            bbb = 0.5 * (zvb41(l-1) + zvb41(l))
            zpm(k,l) = aaa + bbb*zpsol(k)
         enddo
      enddo
C
      RETURN
      END
C
CCC
C
