C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This module defines the F90 kind parameter for common data types.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: kinds_mod.f,v 1.2 2000/04/19 21:56:25 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module kinds_mod

C-----------------------------------------------------------------------

      implicit none
      save

C-----------------------------------------------------------------------

      integer, parameter :: char_len  = 80,
     &                      int_kind  = kind(1),
     &                      log_kind  = kind(.true.),
     &                      real_kind = selected_real_kind(6),
     &                      dbl_kind  = selected_real_kind(13)

C-----------------------------------------------------------------------

      end module kinds_mod

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This module defines common constants used in many routines.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: constants.f,v 1.2 2000/04/19 21:56:25 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module constants

C-----------------------------------------------------------------------

      use kinds_mod  ! defines common data types

      implicit none

      save

C-----------------------------------------------------------------------

      real (kind = dbl_kind), parameter :: 
     &                        zero   = 0.0_dbl_kind,
     &                        one    = 1.0_dbl_kind,
     &                        two    = 2.0_dbl_kind,
     &                        three  = 3.0_dbl_kind,
     &                        four   = 4.0_dbl_kind,
     &                        five   = 5.0_dbl_kind,
     &                        half   = 0.5_dbl_kind,
     &                        quart  = 0.25_dbl_kind,
     &                        bignum = 1.e+20_dbl_kind,
     &                        tiny   = 1.e-14_dbl_kind,
     &                        pi     = 3.14159265359_dbl_kind,
     &                        pi2    = two*pi,
     &                        pih    = half*pi

C-----------------------------------------------------------------------

      end module constants

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This module contains the netCDF include file and a netcdf error
C     handling routine.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: netcdf.f,v 1.2 2000/04/19 21:56:25 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module netcdf_mod

C-----------------------------------------------------------------------

      use kinds_mod
      use constants
      USE palmlib

      implicit none

      include 'netcdf.inc'

C***********************************************************************

      contains

C***********************************************************************

      subroutine netcdf_error_handler(istat)

C-----------------------------------------------------------------------
C
C     This routine provides a simple interface to netCDF error message
C     routine.
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &    istat   ! integer status returned by netCDF function call
      INTEGER :: il_err

C-----------------------------------------------------------------------

      if (istat /= NF_NOERR) then
        print *,'Error in netCDF: ',nf_strerror(istat)
        WRITE(PL_OUT,*) 'Error in netCDF: ',nf_strerror(istat)
        CALL PALM_Abort(il_err)
      endif

C-----------------------------------------------------------------------

      end subroutine netcdf_error_handler

C***********************************************************************

      end module netcdf_mod

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This module is a dynamic I/O unit manager.  It keeps track of
C     which units are in use and reserves units for stdin, stdout, and
C     stderr.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: iounits.f,v 1.2 2000/04/19 21:56:25 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module iounits

C-----------------------------------------------------------------------

      use kinds_mod   ! defines data types

      implicit none

C-----------------------------------------------------------------------

      logical (kind=log_kind), dimension(99), save ::
     &    unit_free   ! flags to determine whether unit is free for use

      integer (kind=int_kind), parameter ::
     &    stdin  = 5, ! reserves unit for standard input
     &    stdout = 6, ! reserves unit for standard output
     &    stderr = 6  ! reserves unit for standard error

C***********************************************************************

      contains

C***********************************************************************

      subroutine get_unit(iunit)

C-----------------------------------------------------------------------
C
C     This routine returns the next available I/O unit number.
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     output variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(out) ::
     &     iunit   ! next free I/O unit

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: n

      logical (kind=log_kind), save :: first_call = .true.

C-----------------------------------------------------------------------
C
C     if this is the first call, reserve stdout, stdin and stderr
C
C-----------------------------------------------------------------------

      if (first_call) then
        unit_free = .true.
        unit_free(stdin)  = .false.
        unit_free(stdout) = .false.
        unit_free(stderr) = .false.
        first_call = .false.
      endif

C-----------------------------------------------------------------------
C
C     search for next available unit
C
C-----------------------------------------------------------------------

      srch_unit: do n=1,99
        if (unit_free(n)) then
          iunit = n
          unit_free(n) = .false.
          exit srch_unit
        endif
      end do srch_unit

C-----------------------------------------------------------------------

      end subroutine get_unit

C***********************************************************************

      subroutine release_unit(iunit)

C-----------------------------------------------------------------------
C
C     This routine releases the specified unit and closes the file.
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &     iunit   ! I/O unit to release

C-----------------------------------------------------------------------
C
C     closes I/O unit and declares it free
C
C-----------------------------------------------------------------------

      unit_free(iunit) = .true.
      close(iunit)

C-----------------------------------------------------------------------

      end subroutine release_unit

C***********************************************************************

      end module iounits

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This module reads in and initializes two grids for remapping.
C     NOTE: grid1 must be the master grid -- the grid that determines
C           which cells participate (e.g. land mask) and the fractional
C           area of grid2 cells that participate in the remapping.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: grids.f,v 1.6 2001/08/21 21:06:41 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module grids

C-----------------------------------------------------------------------

      use kinds_mod    ! defines data types
      use constants    ! common constants
      use iounits      ! I/O unit manager
      use netcdf_mod   ! netCDF stuff

      implicit none

C-----------------------------------------------------------------------
C
C     variables that describe each grid
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), save ::
     &             grid1_size, grid2_size, ! total points on each grid
     &             grid1_rank, grid2_rank, ! rank of each grid
     &             grid1_corners, grid2_corners ! number of corners
                                                ! for each grid cell

      integer (kind=int_kind), dimension(:), allocatable, save ::
     &             grid1_dims, grid2_dims  ! size of each grid dimension

      character(char_len), save :: 
     &             grid1_name, grid2_name  ! name for each grid

      character (char_len), save :: 
     &             grid1_units, ! units for grid coords (degs/radians)
     &             grid2_units  ! units for grid coords

      real (kind=dbl_kind), parameter ::
     &      deg2rad = pi/180.   ! conversion for deg to rads

C-----------------------------------------------------------------------
C
C     grid coordinates and masks
C
C-----------------------------------------------------------------------

      logical (kind=log_kind), dimension(:), allocatable, save ::
     &             grid1_mask,        ! flag which cells participate
     &             grid2_mask         ! flag which cells participate

      real (kind=dbl_kind), dimension(:), allocatable, save ::
     &             grid1_center_lat,  ! lat/lon coordinates for
     &             grid1_center_lon,  ! each grid center in radians
     &             grid2_center_lat, 
     &             grid2_center_lon,
     &             grid1_area,        ! tot area of each grid1 cell
     &             grid2_area,        ! tot area of each grid2 cell
     &             grid1_area_in,     ! area of grid1 cell from file
     &             grid2_area_in,     ! area of grid2 cell from file
     &             grid1_frac,        ! fractional area of grid cells
     &             grid2_frac         ! participating in remapping

      real (kind=dbl_kind), dimension(:,:), allocatable, save ::
     &             grid1_corner_lat,  ! lat/lon coordinates for
     &             grid1_corner_lon,  ! each grid corner in radians
     &             grid2_corner_lat, 
     &             grid2_corner_lon

      logical (kind=log_kind), save ::
     &             luse_grid_centers ! use centers for bounding boxes
     &,            luse_grid1_area   ! use area from grid file
     &,            luse_grid2_area   ! use area from grid file

      real (kind=dbl_kind), dimension(:,:), allocatable, save ::
     &             grid1_bound_box,  ! lat/lon bounding box for use
     &             grid2_bound_box   ! in restricting grid searches

C-----------------------------------------------------------------------
C
C     bins for restricting searches
C
C-----------------------------------------------------------------------

      character (char_len), save ::
     &        restrict_type  ! type of bins to use

      integer (kind=int_kind), save ::
     &        num_srch_bins  ! num of bins for restricted srch

      integer (kind=int_kind), dimension(:,:), allocatable, save ::
     &        bin_addr1, ! min,max adds for grid1 cells in this lat bin
     &        bin_addr2  ! min,max adds for grid2 cells in this lat bin

      real(kind=dbl_kind), dimension(:,:), allocatable, save ::
     &        bin_lats   ! min,max latitude for each search bin
     &,       bin_lons   ! min,max longitude for each search bin

C***********************************************************************

      contains

C***********************************************************************

      subroutine grid_init(grid1_file, grid2_file)

C-----------------------------------------------------------------------
C
C     this routine reads grid info from grid files and makes any
C     necessary changes (e.g. for 0,2pi longitude range)
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(in) :: 
     &             grid1_file, grid2_file  ! grid data files

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: 
     &  n      ! loop counter
     &, nele   ! element loop counter
     &, iunit  ! unit number for opening files
     &, i,j    ! logical 2d addresses
     &, ip1,jp1
     &, n_add, e_add, ne_add
     &, nx, ny

      integer (kind=int_kind) :: 
     &         ncstat,           ! netCDF status variable
     &         nc_grid1_id,       ! netCDF grid file id
     &         nc_grid2_id,       ! netCDF grid file id
     &         nc_grid1size_id,   ! netCDF grid size dim id
     &         nc_grid2size_id,   ! netCDF grid size dim id
     &         nc_grid1corn_id,   ! netCDF grid corner dim id
     &         nc_grid2corn_id,   ! netCDF grid corner dim id
     &         nc_grid1rank_id,   ! netCDF grid rank dim id
     &         nc_grid2rank_id,   ! netCDF grid rank dim id
     &         nc_grid1area_id,   ! netCDF grid rank dim id
     &         nc_grid2area_id,   ! netCDF grid rank dim id
     &         nc_grid1dims_id,   ! netCDF grid dimension size id
     &         nc_grid2dims_id,   ! netCDF grid dimension size id
     &         nc_grd1imask_id,   ! netCDF grid imask var id
     &         nc_grd2imask_id,   ! netCDF grid imask var id
     &         nc_grd1crnrlat_id, ! netCDF grid corner lat var id
     &         nc_grd2crnrlat_id, ! netCDF grid corner lat var id
     &         nc_grd1crnrlon_id, ! netCDF grid corner lon var id
     &         nc_grd2crnrlon_id, ! netCDF grid corner lon var id
     &         nc_grd1cntrlat_id, ! netCDF grid center lat var id
     &         nc_grd2cntrlat_id, ! netCDF grid center lat var id
     &         nc_grd1cntrlon_id, ! netCDF grid center lon var id
     &         nc_grd2cntrlon_id  ! netCDF grid center lon var id

      integer (kind=int_kind), dimension(:), allocatable :: 
     &                            imask ! integer mask read from file

      real (kind=dbl_kind) :: 
     &  dlat,dlon           ! lat/lon intervals for search bins

      real (kind=dbl_kind), dimension(4) ::
     &  tmp_lats, tmp_lons  ! temps for computing bounding boxes

C-----------------------------------------------------------------------
C
C     open grid files and read grid size/name data
C
C-----------------------------------------------------------------------

      ncstat = nf_open(grid1_file, NF_NOWRITE, nc_grid1_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_open(grid2_file, NF_NOWRITE, nc_grid2_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_grid1_id, 'grid_size', nc_grid1size_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_grid1_id, nc_grid1size_id, grid1_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_grid2_id, 'grid_size', nc_grid2size_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_grid2_id, nc_grid2size_id, grid2_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_grid1_id, 'grid_rank', nc_grid1rank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_grid1_id, nc_grid1rank_id, grid1_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_grid2_id, 'grid_rank', nc_grid2rank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_grid2_id, nc_grid2rank_id, grid2_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_grid1_id,'grid_corners',nc_grid1corn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_grid1_id,nc_grid1corn_id,grid1_corners)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_grid2_id,'grid_corners',nc_grid2corn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_grid2_id,nc_grid2corn_id,grid2_corners)
      call netcdf_error_handler(ncstat)

      IF (.not.allocated(grid1_dims)) allocate( grid1_dims(grid1_rank))
      IF (.not.allocated(grid2_dims)) allocate( grid2_dims(grid2_rank))

      ncstat = nf_get_att_text(nc_grid1_id, nf_global, 'title',
     &                         grid1_name)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_att_text(nc_grid2_id, nf_global, 'title',
     &                         grid2_name)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     allocate grid coordinates/masks and read data
C
C-----------------------------------------------------------------------
      IF(.NOT.allocated(grid1_mask))
     &    allocate(grid1_mask      (grid1_size))
      IF(.NOT.allocated(grid2_mask))
     &    allocate(grid2_mask      (grid2_size))
      IF(.NOT.allocated(grid1_center_lat))
     &    allocate(grid1_center_lat(grid1_size)) 
      IF(.NOT.allocated(grid1_center_lon))
     &    allocate(grid1_center_lon(grid1_size))
      IF(.NOT.allocated(grid2_center_lat))
     &    allocate(grid2_center_lat(grid2_size)) 
      IF(.NOT.allocated(grid2_center_lon))
     &    allocate(grid2_center_lon(grid2_size))
      IF(.NOT.allocated(grid1_area))
     &    allocate(grid1_area      (grid1_size))
      IF(.NOT.allocated(grid2_area))
     &    allocate(grid2_area      (grid2_size))
      IF(.NOT.allocated(grid1_frac))
     &    allocate(grid1_frac      (grid1_size))
      IF(.NOT.allocated(grid2_frac))
     &    allocate(grid2_frac      (grid2_size))
      IF(.NOT.allocated(grid1_corner_lat))
     &    allocate(grid1_corner_lat(grid1_corners, grid1_size))
      IF(.NOT.allocated(grid1_corner_lon))
     &    allocate(grid1_corner_lon(grid1_corners, grid1_size))
      IF(.NOT.allocated(grid2_corner_lat))
     &    allocate(grid2_corner_lat(grid2_corners, grid2_size))
      IF(.NOT.allocated(grid2_corner_lon))
     &    allocate(grid2_corner_lon(grid2_corners, grid2_size))
      IF(.NOT.allocated(grid1_bound_box))
     &    allocate(grid1_bound_box (4            , grid1_size))
      IF(.NOT.allocated(grid2_bound_box))
     &    allocate(grid2_bound_box (4            , grid2_size))

      IF(.NOT.allocated(imask))
     &    allocate(imask(grid1_size))

      ncstat = nf_inq_varid(nc_grid1_id, 'grid_dims', nc_grid1dims_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid1_id, 'grid_imask', nc_grd1imask_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid1_id, 'grid_center_lat', 
     &                                   nc_grd1cntrlat_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid1_id, 'grid_center_lon', 
     &                                   nc_grd1cntrlon_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid1_id, 'grid_corner_lat', 
     &                                   nc_grd1crnrlat_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid1_id, 'grid_corner_lon', 
     &                                   nc_grd1crnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_grid1_id, nc_grid1dims_id, grid1_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_grid1_id, nc_grd1imask_id, imask)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid1_id, nc_grd1cntrlat_id, 
     &                                       grid1_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid1_id, nc_grd1cntrlon_id, 
     &                                       grid1_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid1_id, nc_grd1crnrlat_id, 
     &                                       grid1_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid1_id, nc_grd1crnrlon_id, 
     &                                       grid1_corner_lon)
      call netcdf_error_handler(ncstat)

      if (luse_grid1_area) then
        IF(.NOT.allocated(grid1_area_in))
     &        allocate (grid1_area_in(grid1_size))
        ncstat = nf_inq_varid(nc_grid1_id, 'grid_area', nc_grid1area_id)
        call netcdf_error_handler(ncstat)
        ncstat = nf_get_var_double(nc_grid1_id, nc_grid1area_id, 
     &                                          grid1_area_in)
        call netcdf_error_handler(ncstat)
      endif

      grid1_area = zero
      grid1_frac = zero

C-----------------------------------------------------------------------
C
C     initialize logical mask and convert lat/lon units if required
C
C-----------------------------------------------------------------------

      where (imask == 1)
        grid1_mask = .true.
      elsewhere
        grid1_mask = .false.
      endwhere
      deallocate(imask)

      grid1_units = ' '
      ncstat = nf_get_att_text(nc_grid1_id, nc_grd1cntrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')

        grid1_center_lat = grid1_center_lat*deg2rad
        grid1_center_lon = grid1_center_lon*deg2rad

      case ('radians')

        !*** no conversion necessary

      case default

        print *,'unknown units supplied for grid1 center lat/lon: '
        print *,'proceeding assuming radians'

      end select

      grid1_units = ' '
      ncstat = nf_get_att_text(nc_grid1_id, nc_grd1crnrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')

        grid1_corner_lat = grid1_corner_lat*deg2rad
        grid1_corner_lon = grid1_corner_lon*deg2rad

      case ('radians')

        !*** no conversion necessary

      case default

        print *,'unknown units supplied for grid1 corner lat/lon: '
        print *,'proceeding assuming radians'

      end select

      ncstat = nf_close(nc_grid1_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     read data for grid 2
C
C-----------------------------------------------------------------------

      IF(.NOT.allocated(imask)) allocate(imask(grid2_size))

      ncstat = nf_inq_varid(nc_grid2_id, 'grid_dims', nc_grid2dims_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid2_id, 'grid_imask', nc_grd2imask_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid2_id, 'grid_center_lat', 
     &                                   nc_grd2cntrlat_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid2_id, 'grid_center_lon', 
     &                                   nc_grd2cntrlon_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid2_id, 'grid_corner_lat', 
     &                                   nc_grd2crnrlat_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_varid(nc_grid2_id, 'grid_corner_lon', 
     &                                   nc_grd2crnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_grid2_id, nc_grid2dims_id, grid2_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_grid2_id, nc_grd2imask_id, imask)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid2_id, nc_grd2cntrlat_id, 
     &                                       grid2_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid2_id, nc_grd2cntrlon_id, 
     &                                       grid2_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid2_id, nc_grd2crnrlat_id, 
     &                                       grid2_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_grid2_id, nc_grd2crnrlon_id, 
     &                                       grid2_corner_lon)
      call netcdf_error_handler(ncstat)

      if (luse_grid2_area) THEN
        IF (.not.allocated(grid2_area_in))  
     &        allocate (grid2_area_in(grid2_size))
        ncstat = nf_inq_varid(nc_grid2_id, 'grid_area', nc_grid2area_id)
        call netcdf_error_handler(ncstat)
        ncstat = nf_get_var_double(nc_grid2_id, nc_grid2area_id, 
     &                                          grid2_area_in)
        call netcdf_error_handler(ncstat)
      endif

      grid2_area = zero
      grid2_frac = zero

C-----------------------------------------------------------------------
C
C     initialize logical mask and convert lat/lon units if required
C
C-----------------------------------------------------------------------

      where (imask == 1)
        grid2_mask = .true.
      elsewhere
        grid2_mask = .false.
      endwhere
      deallocate(imask)

      grid2_units = ' '
      ncstat = nf_get_att_text(nc_grid2_id, nc_grd2cntrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')

        grid2_center_lat = grid2_center_lat*deg2rad
        grid2_center_lon = grid2_center_lon*deg2rad

      case ('radians')

        !*** no conversion necessary

      case default

        print *,'unknown units supplied for grid2 center lat/lon: '
        print *,'proceeding assuming radians'

      end select

      grid2_units = ' '
      ncstat = nf_get_att_text(nc_grid2_id, nc_grd2crnrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')

        grid2_corner_lat = grid2_corner_lat*deg2rad
        grid2_corner_lon = grid2_corner_lon*deg2rad

      case ('radians')

        !*** no conversion necessary

      case default

        print *,'no units supplied for grid2 corner lat/lon: '
        print *,'proceeding assuming radians'

      end select

      ncstat = nf_close(nc_grid2_id)
      call netcdf_error_handler(ncstat)


C-----------------------------------------------------------------------
C
C     convert longitudes to 0,2pi interval
C
C-----------------------------------------------------------------------

      where (grid1_center_lon .gt. pi2)  grid1_center_lon =
     &                                   grid1_center_lon - pi2
      where (grid1_center_lon .lt. zero) grid1_center_lon =
     &                                   grid1_center_lon + pi2
      where (grid2_center_lon .gt. pi2)  grid2_center_lon =
     &                                   grid2_center_lon - pi2
      where (grid2_center_lon .lt. zero) grid2_center_lon =
     &                                   grid2_center_lon + pi2
      where (grid1_corner_lon .gt. pi2)  grid1_corner_lon =
     &                                   grid1_corner_lon - pi2
      where (grid1_corner_lon .lt. zero) grid1_corner_lon =
     &                                   grid1_corner_lon + pi2
      where (grid2_corner_lon .gt. pi2)  grid2_corner_lon =
     &                                   grid2_corner_lon - pi2
      where (grid2_corner_lon .lt. zero) grid2_corner_lon =
     &                                   grid2_corner_lon + pi2

C-----------------------------------------------------------------------
C
C     make sure input latitude range is within the machine values
C     for +/- pi/2 
C
C-----------------------------------------------------------------------

      where (grid1_center_lat >  pih) grid1_center_lat =  pih
      where (grid1_corner_lat >  pih) grid1_corner_lat =  pih
      where (grid1_center_lat < -pih) grid1_center_lat = -pih
      where (grid1_corner_lat < -pih) grid1_corner_lat = -pih

      where (grid2_center_lat >  pih) grid2_center_lat =  pih
      where (grid2_corner_lat >  pih) grid2_corner_lat =  pih
      where (grid2_center_lat < -pih) grid2_center_lat = -pih
      where (grid2_corner_lat < -pih) grid2_corner_lat = -pih

C-----------------------------------------------------------------------
C
C     compute bounding boxes for restricting future grid searches
C
C-----------------------------------------------------------------------

      if (.not. luse_grid_centers) then
        grid1_bound_box(1,:) = minval(grid1_corner_lat, DIM=1)
        grid1_bound_box(2,:) = maxval(grid1_corner_lat, DIM=1)
        grid1_bound_box(3,:) = minval(grid1_corner_lon, DIM=1)
        grid1_bound_box(4,:) = maxval(grid1_corner_lon, DIM=1)

        grid2_bound_box(1,:) = minval(grid2_corner_lat, DIM=1)
        grid2_bound_box(2,:) = maxval(grid2_corner_lat, DIM=1)
        grid2_bound_box(3,:) = minval(grid2_corner_lon, DIM=1)
        grid2_bound_box(4,:) = maxval(grid2_corner_lon, DIM=1)

      else

        nx = grid1_dims(1)
        ny = grid1_dims(2)

        do n=1,grid1_size

          !*** find N,S and NE points to this grid point

          j = (n - 1)/nx +1
          i = n - (j-1)*nx

          if (i < nx) then
            ip1 = i + 1
          else
            !*** assume cyclic
            ip1 = 1
            !*** but if it is not, correct
            e_add = (j - 1)*nx + ip1
            if (abs(grid1_center_lat(e_add) - 
     &              grid1_center_lat(n   )) > pih) then
              ip1 = i
            endif
          endif

          if (j < ny) then
            jp1 = j+1
          else
            !*** assume cyclic
            jp1 = 1
            !*** but if it is not, correct
            n_add = (jp1 - 1)*nx + i
            if (abs(grid1_center_lat(n_add) - 
     &              grid1_center_lat(n   )) > pih) then
              jp1 = j
            endif
          endif

          n_add = (jp1 - 1)*nx + i
          e_add = (j - 1)*nx + ip1
          ne_add = (jp1 - 1)*nx + ip1

          !*** find N,S and NE lat/lon coords and check bounding box

          tmp_lats(1) = grid1_center_lat(n)
          tmp_lats(2) = grid1_center_lat(e_add)
          tmp_lats(3) = grid1_center_lat(ne_add)
          tmp_lats(4) = grid1_center_lat(n_add)

          tmp_lons(1) = grid1_center_lon(n)
          tmp_lons(2) = grid1_center_lon(e_add)
          tmp_lons(3) = grid1_center_lon(ne_add)
          tmp_lons(4) = grid1_center_lon(n_add)

          grid1_bound_box(1,n) = minval(tmp_lats)
          grid1_bound_box(2,n) = maxval(tmp_lats)
          grid1_bound_box(3,n) = minval(tmp_lons)
          grid1_bound_box(4,n) = maxval(tmp_lons)
        end do

        nx = grid2_dims(1)
        ny = grid2_dims(2)

        do n=1,grid2_size

          !*** find N,S and NE points to this grid point

          j = (n - 1)/nx +1
          i = n - (j-1)*nx

          if (i < nx) then
            ip1 = i + 1
          else
            !*** assume cyclic
            ip1 = 1
            !*** but if it is not, correct
            e_add = (j - 1)*nx + ip1
            if (abs(grid2_center_lat(e_add) - 
     &              grid2_center_lat(n   )) > pih) then
              ip1 = i
            endif
          endif

          if (j < ny) then
            jp1 = j+1
          else
            !*** assume cyclic
            jp1 = 1
            !*** but if it is not, correct
            n_add = (jp1 - 1)*nx + i
            if (abs(grid2_center_lat(n_add) - 
     &              grid2_center_lat(n   )) > pih) then
              jp1 = j
            endif
          endif

          n_add = (jp1 - 1)*nx + i
          e_add = (j - 1)*nx + ip1
          ne_add = (jp1 - 1)*nx + ip1

          !*** find N,S and NE lat/lon coords and check bounding box

          tmp_lats(1) = grid2_center_lat(n)
          tmp_lats(2) = grid2_center_lat(e_add)
          tmp_lats(3) = grid2_center_lat(ne_add)
          tmp_lats(4) = grid2_center_lat(n_add)

          tmp_lons(1) = grid2_center_lon(n)
          tmp_lons(2) = grid2_center_lon(e_add)
          tmp_lons(3) = grid2_center_lon(ne_add)
          tmp_lons(4) = grid2_center_lon(n_add)

          grid2_bound_box(1,n) = minval(tmp_lats)
          grid2_bound_box(2,n) = maxval(tmp_lats)
          grid2_bound_box(3,n) = minval(tmp_lons)
          grid2_bound_box(4,n) = maxval(tmp_lons)
        end do

      endif

      where (abs(grid1_bound_box(4,:) - grid1_bound_box(3,:)) > pi)
        grid1_bound_box(3,:) = zero
        grid1_bound_box(4,:) = pi2
      end where

      where (abs(grid2_bound_box(4,:) - grid2_bound_box(3,:)) > pi)
        grid2_bound_box(3,:) = zero
        grid2_bound_box(4,:) = pi2
      end where

      !***
      !*** try to check for cells that overlap poles
      !***

      where (grid1_center_lat > grid1_bound_box(2,:))
     &  grid1_bound_box(2,:) = pih

      where (grid1_center_lat < grid1_bound_box(1,:))
     &  grid1_bound_box(1,:) = -pih

      where (grid2_center_lat > grid2_bound_box(2,:))
     &  grid2_bound_box(2,:) = pih

      where (grid2_center_lat < grid2_bound_box(1,:))
     &  grid2_bound_box(1,:) = -pih

C-----------------------------------------------------------------------
C
C     set up and assign address ranges to search bins in order to 
C     further restrict later searches
C
C-----------------------------------------------------------------------

      select case (restrict_type)

      case ('latitude')
C        write(stdout,*) 'Using latitude bins to restrict search.'
        IF(.NOT.allocated(bin_addr1))
     &  allocate(bin_addr1(2,num_srch_bins))
        IF(.NOT.allocated(bin_addr2))
     &  allocate(bin_addr2(2,num_srch_bins))
        IF(.NOT.allocated(bin_lats))
     &  allocate(bin_lats (2,num_srch_bins))
        IF(.NOT.allocated(bin_lons))
     &  allocate(bin_lons (2,num_srch_bins))

        dlat = pi/num_srch_bins

        do n=1,num_srch_bins
          bin_lats(1,n) = (n-1)*dlat - pih
          bin_lats(2,n) =     n*dlat - pih
          bin_lons(1,n) = zero
          bin_lons(2,n) = pi2
          bin_addr1(1,n) = grid1_size + 1
          bin_addr1(2,n) = 0
          bin_addr2(1,n) = grid2_size + 1
          bin_addr2(2,n) = 0
        end do

        do nele=1,grid1_size
          do n=1,num_srch_bins
            if (grid1_bound_box(1,nele) <= bin_lats(2,n) .and.
     &          grid1_bound_box(2,nele) >= bin_lats(1,n)) then
              bin_addr1(1,n) = min(nele,bin_addr1(1,n))
              bin_addr1(2,n) = max(nele,bin_addr1(2,n))
            endif
          end do
        end do

        do nele=1,grid2_size
          do n=1,num_srch_bins
            if (grid2_bound_box(1,nele) <= bin_lats(2,n) .and.
     &          grid2_bound_box(2,nele) >= bin_lats(1,n)) then
              bin_addr2(1,n) = min(nele,bin_addr2(1,n))
              bin_addr2(2,n) = max(nele,bin_addr2(2,n))
            endif
          end do
        end do

      case ('latlon')
C        write(stdout,*) 'Using lat/lon boxes to restrict search.'

        dlat = pi /num_srch_bins
        dlon = pi2/num_srch_bins

        IF(.NOT.allocated(bin_addr1))
     &  allocate(bin_addr1(2,num_srch_bins*num_srch_bins))
        IF(.NOT.allocated(bin_addr2))
     &  allocate(bin_addr2(2,num_srch_bins*num_srch_bins))
        IF(.NOT.allocated(bin_lats))
     &  allocate(bin_lats (2,num_srch_bins*num_srch_bins))
        IF(.NOT.allocated(bin_lons))
     &  allocate(bin_lons (2,num_srch_bins*num_srch_bins))

        n = 0
        do j=1,num_srch_bins
        do i=1,num_srch_bins
          n = n + 1

          bin_lats(1,n) = (j-1)*dlat - pih
          bin_lats(2,n) =     j*dlat - pih
          bin_lons(1,n) = (i-1)*dlon
          bin_lons(2,n) =     i*dlon
          bin_addr1(1,n) = grid1_size + 1
          bin_addr1(2,n) = 0
          bin_addr2(1,n) = grid2_size + 1
          bin_addr2(2,n) = 0
        end do
        end do

        num_srch_bins = num_srch_bins**2

        do nele=1,grid1_size
          do n=1,num_srch_bins
            if (grid1_bound_box(1,nele) <= bin_lats(2,n) .and.
     &          grid1_bound_box(2,nele) >= bin_lats(1,n) .and.
     &          grid1_bound_box(3,nele) <= bin_lons(2,n) .and.
     &          grid1_bound_box(4,nele) >= bin_lons(1,n)) then
              bin_addr1(1,n) = min(nele,bin_addr1(1,n))
              bin_addr1(2,n) = max(nele,bin_addr1(2,n))
            endif
          end do
        end do

        do nele=1,grid2_size
          do n=1,num_srch_bins
            if (grid2_bound_box(1,nele) <= bin_lats(2,n) .and.
     &          grid2_bound_box(2,nele) >= bin_lats(1,n) .and.
     &          grid2_bound_box(3,nele) <= bin_lons(2,n) .and.
     &          grid2_bound_box(4,nele) >= bin_lons(1,n)) then
              bin_addr2(1,n) = min(nele,bin_addr2(1,n))
              bin_addr2(2,n) = max(nele,bin_addr2(2,n))
            endif
          end do
        end do

      case default
        stop 'unknown search restriction method'
      end select

C-----------------------------------------------------------------------

      end subroutine grid_init

C***********************************************************************

      end module grids

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     this module contains necessary variables for remapping between
C     two grids.  also routines for resizing and initializing these
C     variables.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: remap_vars.f,v 1.5 2000/04/19 21:56:26 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module remap_vars

      use kinds_mod
      use constants
      use grids
      USE palmlib

      implicit none

C-----------------------------------------------------------------------
C
C     module variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &      norm_opt_none    = 1
     &,     norm_opt_dstarea = 2
     &,     norm_opt_frcarea = 3

      integer (kind=int_kind), parameter ::
     &      map_type_conserv  = 1
     &,     map_type_bilinear = 2
     &,     map_type_bicubic  = 3
     &,     map_type_distwgt  = 4

      integer (kind=int_kind), save :: 
     &      max_links_map1  ! current size of link arrays
     &,     num_links_map1  ! actual number of links for remapping
     &,     max_links_map2  ! current size of link arrays
     &,     num_links_map2  ! actual number of links for remapping
     &,     num_maps        ! num of remappings for this grid pair
     &,     num_wts         ! num of weights used in remapping
     &,     map_type        ! identifier for remapping method
     &,     norm_opt        ! option for normalization (conserv only)
     &,     resize_increment ! default amount to increase array size

      integer (kind=int_kind), dimension(:), allocatable, save ::
     &      grid1_add_map1, ! grid1 address for each link in mapping 1
     &      grid2_add_map1, ! grid2 address for each link in mapping 1
     &      grid1_add_map2, ! grid1 address for each link in mapping 2
     &      grid2_add_map2  ! grid2 address for each link in mapping 2

      real (kind=dbl_kind), dimension(:,:), allocatable, save ::
     &      wts_map1, ! map weights for each link (num_wts,max_links)
     &      wts_map2  ! map weights for each link (num_wts,max_links)

C***********************************************************************

      contains

C***********************************************************************

      subroutine init_remap_vars

C-----------------------------------------------------------------------
C
C     this routine initializes some variables and provides an initial
C     allocation of arrays (fairly large so frequent resizing 
C     unnecessary).
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     determine the number of weights
C
C-----------------------------------------------------------------------

      select case (map_type)
      case(map_type_conserv)
        num_wts = 3
      case(map_type_bilinear)
        num_wts = 1
      case(map_type_bicubic)
        num_wts = 4
      case(map_type_distwgt)
        num_wts = 1
      end select

C-----------------------------------------------------------------------
C
C     initialize num_links and set max_links to four times the largest 
C     of the destination grid sizes initially (can be changed later).
C     set a default resize increment to increase the size of link
C     arrays if the number of links exceeds the initial size
C   
C-----------------------------------------------------------------------

      num_links_map1 = 0
      max_links_map1 = 4*grid2_size
      if (num_maps > 1) then
        num_links_map2 = 0
        max_links_map1 = max(4*grid1_size,4*grid2_size)
        max_links_map2 = max_links_map1
      endif

      resize_increment = 0.1*max(grid1_size,grid2_size)

C-----------------------------------------------------------------------
C
C     allocate address and weight arrays for mapping 1
C   
C-----------------------------------------------------------------------
      IF(.NOT.allocated(grid1_add_map1))
     &  allocate (grid1_add_map1(max_links_map1))
      IF(.NOT.allocated(grid2_add_map1))
     & allocate(grid2_add_map1(max_links_map1))
      IF(.NOT.allocated(wts_map1))
     & allocate(wts_map1(num_wts, max_links_map1))

C-----------------------------------------------------------------------
C
C     allocate address and weight arrays for mapping 2 if necessary 
C   
C-----------------------------------------------------------------------

      if (num_maps > 1) then
        IF(.NOT.allocated(grid1_add_map2))
     &  allocate (grid1_add_map2(max_links_map2))
        IF(.NOT.allocated(grid2_add_map2))
     &  allocate (grid2_add_map2(max_links_map2))
        IF(.NOT.allocated(wts_map2))
     &  allocate (wts_map2(num_wts, max_links_map2))
      endif

C-----------------------------------------------------------------------

      end subroutine init_remap_vars

C***********************************************************************

      subroutine resize_remap_vars(nmap, increment)

C-----------------------------------------------------------------------
C
C     this routine resizes remapping arrays by increasing(decreasing)
C     the max_links by increment
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &     nmap,      ! identifies which mapping array to resize
     &     increment  ! the number of links to add(subtract) to arrays

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &   ierr,     ! error flag
     &   mxlinks   ! size of link arrays

      integer (kind=int_kind), dimension(:), allocatable ::
     &   add1_tmp, ! temp array for resizing address arrays
     &   add2_tmp  ! temp array for resizing address arrays

      real (kind=dbl_kind), dimension(:,:), allocatable ::
     &   wts_tmp   ! temp array for resizing weight arrays

C-----------------------------------------------------------------------
C
C     resize map 1 arrays if required.
C
C-----------------------------------------------------------------------

      select case (nmap)
      case(1)

        !***
        !*** allocate temporaries to hold original values
        !***

        mxlinks = size(grid1_add_map1)
        IF(.not.allocated(add1_tmp))
     &      allocate(add1_tmp(mxlinks))
        IF(.not.allocated(add2_tmp))
     &      allocate(add2_tmp(mxlinks)) 
        IF(.not.allocated(wts_tmp))
     &      allocate(wts_tmp(num_wts,mxlinks))

        add1_tmp = grid1_add_map1
        add2_tmp = grid2_add_map1
        wts_tmp  = wts_map1
        
        !***
        !*** deallocate originals and increment max_links then
        !*** reallocate arrays at new size
        !***

        deallocate (grid1_add_map1, grid2_add_map1, wts_map1)
        max_links_map1 = mxlinks + increment
        allocate (grid1_add_map1(max_links_map1),
     &            grid2_add_map1(max_links_map1),
     &            wts_map1(num_wts,max_links_map1))

        !***
        !*** restore original values from temp arrays and
        !*** deallocate temps
        !***

        mxlinks = min(mxlinks, max_links_map1)
        grid1_add_map1(1:mxlinks) = add1_tmp (1:mxlinks)
        grid2_add_map1(1:mxlinks) = add2_tmp (1:mxlinks)
        wts_map1    (:,1:mxlinks) = wts_tmp(:,1:mxlinks)
        deallocate(add1_tmp, add2_tmp, wts_tmp)

C-----------------------------------------------------------------------
C
C     resize map 2 arrays if required.
C
C-----------------------------------------------------------------------

      case(2)

        !***
        !*** allocate temporaries to hold original values
        !***

        mxlinks = size(grid1_add_map2)
        IF(.not.allocated(add1_tmp)) allocate(add1_tmp(mxlinks))
        IF(.not.allocated(add2_tmp)) allocate(add2_tmp(mxlinks)) 
        IF(.not.allocated(wts_tmp))
     &      allocate(wts_tmp(num_wts,mxlinks),stat=ierr)
        if (ierr .ne. 0) then
          print *,'error allocating temps in resize: ',ierr
          WRITE(PL_OUT,*) 'error allocating temps in resize: ',ierr
          CALL PALM_Abort(ierr)
        endif

        add1_tmp = grid1_add_map2
        add2_tmp = grid2_add_map2
        wts_tmp  = wts_map2
        
        !***
        !*** deallocate originals and increment max_links then
        !*** reallocate arrays at new size
        !***

        deallocate (grid1_add_map2, grid2_add_map2, wts_map2)
        max_links_map2 = mxlinks + increment
        allocate (grid1_add_map2(max_links_map2),
     &            grid2_add_map2(max_links_map2),
     &            wts_map2(num_wts,max_links_map2),stat=ierr)
        if (ierr .ne. 0) then
          print *,'error allocating new arrays in resize: ',ierr
          WRITE(PL_OUT,*) 'error allocating new arrays in resize: ',ierr
          CALL PALM_Abort(ierr)
        endif


        !***
        !*** restore original values from temp arrays and
        !*** deallocate temps
        !***

        mxlinks = min(mxlinks, max_links_map2)
        grid1_add_map2(1:mxlinks) = add1_tmp (1:mxlinks)
        grid2_add_map2(1:mxlinks) = add2_tmp (1:mxlinks)
        wts_map2    (:,1:mxlinks) = wts_tmp(:,1:mxlinks)
        deallocate(add1_tmp, add2_tmp, wts_tmp)

      end select

C-----------------------------------------------------------------------

      end subroutine resize_remap_vars

C***********************************************************************

      end module remap_vars

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This routine reads remapping information from files written
C     by remap_setup.  If remapping in both directions are required,
C     two input files must be specified.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: remap_read.f,v 1.6 2000/04/19 21:56:26 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module remap_read

C-----------------------------------------------------------------------
C
C     contains routines for reading a remap file
C
C-----------------------------------------------------------------------

      use kinds_mod     ! defines common data types
      use constants     ! defines useful constants
      use grids         ! includes all grid information
      use netcdf_mod    ! module with netcdf vars and utilities
      use remap_vars    ! module for all required remapping variables
      USE palmlib
      USE palm_user_param

      implicit none

C-----------------------------------------------------------------------
C
C     module variables
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C     various netCDF ids for files variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), private :: ! netCDF ids
     &         ncstat, nc_file_id,
     &         nc_srcgrdsize_id, nc_dstgrdsize_id,
     &         nc_srcgrdcorn_id, nc_dstgrdcorn_id,
     &         nc_srcgrdrank_id, nc_dstgrdrank_id,
     &         nc_srcgrddims_id, nc_dstgrddims_id,
     &         nc_numlinks_id, nc_numwgts_id, 
     &         nc_srcgrdimask_id, nc_dstgrdimask_id,
     &         nc_srcgrdcntrlat_id, nc_srcgrdcntrlon_id,
     &         nc_srcgrdcrnrlat_id, nc_srcgrdcrnrlon_id,
     &         nc_srcgrdarea_id, nc_srcgrdfrac_id,
     &         nc_dstgrdcntrlat_id, nc_dstgrdcntrlon_id,
     &         nc_dstgrdcrnrlat_id, nc_dstgrdcrnrlon_id,
     &         nc_dstgrdarea_id, nc_dstgrdfrac_id,
     &         nc_srcgrdadd_id, nc_dstgrdadd_id, nc_rmpmatrix_id

C***********************************************************************

      contains

C***********************************************************************

      subroutine read_remap(map_name, interp_file)

C-----------------------------------------------------------------------
C
C     this driver routine reads some global attributes and then
C     calls a specific read routine based on file conventions
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(in) ::
     &  interp_file        ! filename for remap data

C-----------------------------------------------------------------------
C
C     output variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(out) ::
     &  map_name            ! name for mapping

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      character(char_len) :: 
     &   map_method       ! character string for map_type
     &,  normalize_opt    ! character string for normalization option
     &,  convention       ! character string for output convention

C-----------------------------------------------------------------------
C
C     open file and read some global information
C
C-----------------------------------------------------------------------

      ncstat = nf_open(interp_file, NF_NOWRITE, nc_file_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** map name
      !***
      map_name = ' '
      ncstat = nf_get_att_text(nc_file_id, NF_GLOBAL, 'title',
     &                         map_name)
      call netcdf_error_handler(ncstat)

      IF (ip_iprint .gt. 0) THEN
          print *,'Reading remapping:',trim(map_name)
          print *,'From file:',trim(interp_file)
          WRITE(PL_OUT,*) 'Reading remapping:',trim(map_name)
          WRITE(PL_OUT,*) 'From file:',trim(interp_file)
      ENDIF

      !***
      !*** normalization option
      !***
      normalize_opt = ' '
      ncstat = nf_get_att_text(nc_file_id, NF_GLOBAL, 'normalization',
     &                         normalize_opt)
      call netcdf_error_handler(ncstat)

      select case(normalize_opt)
      case ('none')
        norm_opt = norm_opt_none
      case ('fracarea')
        norm_opt = norm_opt_frcarea
      case ('destarea')
        norm_opt = norm_opt_dstarea
      case default
        print *,'normalize_opt = ',normalize_opt
        stop 'Invalid normalization option'
      end select

      !***
      !*** map method
      !***
      map_method = ' '
      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'map_method',
     &                          map_method)
      call netcdf_error_handler(ncstat)

      select case(map_method)
      case('Conservative remapping')
        map_type = map_type_conserv
      case('Bilinear remapping')
        map_type = map_type_bilinear
      case('Distance weighted avg of nearest neighbors')
        map_type = map_type_distwgt
      case('Bicubic remapping')
        map_type = map_type_bicubic
      case default
        print *,'map_type = ',map_method
        stop 'Invalid Map Type'
      end select

      !***
      !*** file convention
      !***
      convention = ' '
      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'conventions',
     &                          convention)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     call appropriate read routine based on output convention
C
C-----------------------------------------------------------------------

      select case(convention)
      case ('SCRIP')
        call read_remap_scrip
      case ('NCAR-CSM')
        call read_remap_csm
      case default
        print *,'convention = ',convention
        stop 'unknown output file convention'
      end select

C-----------------------------------------------------------------------

      end subroutine read_remap

C***********************************************************************

      subroutine read_remap2(map_name, interp_file)

C-----------------------------------------------------------------------
C
C     this driver routine reads some global attributes and then
C     calls a specific read routine based on file conventions
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(in) ::
     &  interp_file        ! filename for remap data

C-----------------------------------------------------------------------
C
C     output variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(out) ::
     &  map_name            ! name for mapping

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      character(char_len) :: 
     &   map_method       ! character string for map_type
     &,  normalize_opt    ! character string for normalization option
     &,  convention       ! character string for output convention

C-----------------------------------------------------------------------
C
C     open file and read some global information
C
C-----------------------------------------------------------------------

      ncstat = nf_open(interp_file, NF_NOWRITE, nc_file_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** map name
      !***
      map_name = ' '
      ncstat = nf_get_att_text(nc_file_id, NF_GLOBAL, 'title',
     &                         map_name)
      call netcdf_error_handler(ncstat)

      IF (ip_iprint .gt. 0) THEN
          print *,'Reading remapping:',trim(map_name)
          print *,'From file:',trim(interp_file)
          WRITE(PL_OUT,*) 'Reading remapping:',trim(map_name)
          WRITE(PL_OUT,*) 'From file:',trim(interp_file)
      ENDIF

      !***
      !*** normalization option
      !***
      normalize_opt = ' '
      ncstat = nf_get_att_text(nc_file_id, NF_GLOBAL, 'normalization',
     &                         normalize_opt)
      call netcdf_error_handler(ncstat)

      select case(normalize_opt)
      case ('none')
        norm_opt = norm_opt_none
      case ('fracarea')
        norm_opt = norm_opt_frcarea
      case ('destarea')
        norm_opt = norm_opt_dstarea
      case default
        print *,'normalize_opt = ',normalize_opt
        stop 'Invalid normalization option'
      end select

      !***
      !*** map method
      !***
      map_method = ' '
      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'map_method',
     &                          map_method)
      call netcdf_error_handler(ncstat)

      select case(map_method)
      case('Conservative remapping')
        map_type = map_type_conserv
      case('Bilinear remapping')
        map_type = map_type_bilinear
      case('Distance weighted avg of nearest neighbors')
        map_type = map_type_distwgt
      case('Bicubic remapping')
        map_type = map_type_bicubic
      case default
        print *,'map_type = ',map_method
        stop 'Invalid Map Type'
      end select

      !***
      !*** file convention
      !***
      convention = ' '
      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'conventions',
     &                          convention)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     call appropriate read routine based on output convention
C
C-----------------------------------------------------------------------

      select case(convention)
      case ('SCRIP')
        call read_remap_scrip2
      case ('NCAR-CSM')
        call read_remap_csm
      case default
        print *,'convention = ',convention
        stop 'unknown output file convention'
      end select

C-----------------------------------------------------------------------

      end subroutine read_remap2

C***********************************************************************

      subroutine read_remap_scrip

C-----------------------------------------------------------------------
C
C     the routine reads a netCDF file to extract remapping info
C     in SCRIP format
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      character (char_len) ::
     &  grid1_name           ! grid name for source grid
     &, grid2_name           ! grid name for dest   grid

      integer (kind=int_kind) ::  
     &  n                    ! dummy index

      integer (kind=int_kind), dimension(:), allocatable ::
     &  grid1_mask_int,      ! integer masks to determine
     &  grid2_mask_int       ! cells that participate in map

C-----------------------------------------------------------------------
C
C     read some additional global attributes
C
C-----------------------------------------------------------------------

      !***
      !*** source and destination grid names
      !***

      grid1_name = ' '
      grid2_name = ' '
      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'source_grid',
     &                          grid1_name)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'dest_grid',
     &                          grid2_name)
      call netcdf_error_handler(ncstat)

      IF (ip_iprint .gt. 0) THEN
          print *,' '
          print *,'Remapping between:',trim(grid1_name)
          print *,'and ',trim(grid2_name)
          print *,' '
      ENDIF

C-----------------------------------------------------------------------
C
C     read dimension information
C
C-----------------------------------------------------------------------

      ncstat = nf_inq_dimid(nc_file_id, 'src_grid_size', 
     &                      nc_srcgrdsize_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdsize_id, grid1_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'dst_grid_size', 
     &                      nc_dstgrdsize_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdsize_id, grid2_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'src_grid_corners', 
     &                      nc_srcgrdcorn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdcorn_id, 
     &                       grid1_corners)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'dst_grid_corners', 
     &                      nc_dstgrdcorn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdcorn_id, 
     &                       grid2_corners)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'src_grid_rank', 
     &                      nc_srcgrdrank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdrank_id, 
     &                       grid1_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'dst_grid_rank', 
     &                      nc_dstgrdrank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdrank_id, 
     &                       grid2_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'num_links', 
     &                      nc_numlinks_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_numlinks_id, 
     &                       num_links_map1)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'num_wgts', 
     &                      nc_numwgts_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_numwgts_id, num_wts)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     allocate arrays
C
C-----------------------------------------------------------------------

       IF(.not.allocated(grid1_dims      ))
     & allocate(grid1_dims      (grid1_rank))
       IF(.not.allocated(grid1_center_lat))
     & allocate(grid1_center_lat(grid1_size)) 
       IF(.not.allocated(grid1_center_lon))
     & allocate(grid1_center_lon(grid1_size))
       IF(.not.allocated(grid1_area      ))
     & allocate(grid1_area      (grid1_size))
       IF(.not.allocated(grid1_frac      ))
     & allocate(grid1_frac      (grid1_size))
       IF(.not.allocated(grid1_mask      ))
     & allocate(grid1_mask      (grid1_size))
       IF(.not.allocated(grid1_mask_int  ))
     & allocate(grid1_mask_int  (grid1_size))
       IF(.not.allocated(grid1_corner_lat))
     & allocate(grid1_corner_lat(grid1_corners, grid1_size))
       IF(.not.allocated(grid1_corner_lon))
     & allocate(grid1_corner_lon(grid1_corners, grid1_size) )

       IF(.not.allocated(grid2_dims      ))
     & allocate(grid2_dims      (grid2_rank))
       IF(.not.allocated(grid2_center_lat))
     & allocate(grid2_center_lat(grid2_size)) 
       IF(.not.allocated(grid2_center_lon))
     & allocate(grid2_center_lon(grid2_size))
       IF(.not.allocated(grid2_area      ))
     & allocate(grid2_area      (grid2_size))
       IF(.not.allocated(grid2_frac      ))
     & allocate(grid2_frac      (grid2_size))
       IF(.not.allocated(grid2_mask      ))
     & allocate(grid2_mask      (grid2_size))
       IF(.not.allocated(grid2_mask_int  ))
     & allocate(grid2_mask_int  (grid2_size))
       IF(.not.allocated(grid2_corner_lat))
     & allocate(grid2_corner_lat(grid2_corners, grid2_size))
       IF(.not.allocated(grid2_corner_lon))
     & allocate(grid2_corner_lon(grid2_corners, grid2_size) )

       IF(.not.allocated(grid1_add_map1))
     & allocate(grid1_add_map1(num_links_map1))
       IF(.not.allocated(grid2_add_map1))
     & allocate(grid2_add_map1(num_links_map1))
       IF(.not.allocated(wts_map1))
     & allocate(wts_map1(num_wts,num_links_map1) )

C-----------------------------------------------------------------------
C
C     get variable ids
C
C-----------------------------------------------------------------------

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_dims', 
     &                      nc_srcgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_imask', 
     &                      nc_srcgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_center_lat', 
     &                                   nc_srcgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_center_lon', 
     &                                   nc_srcgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_corner_lat', 
     &                                   nc_srcgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_corner_lon', 
     &                                   nc_srcgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_area', 
     &                                   nc_srcgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_frac', 
     &                                   nc_srcgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_dims', 
     &                      nc_dstgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_imask', 
     &                      nc_dstgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_center_lat', 
     &                                   nc_dstgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_center_lon', 
     &                                   nc_dstgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_corner_lat', 
     &                                   nc_dstgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_corner_lon', 
     &                                   nc_dstgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_area', 
     &                                   nc_dstgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_frac', 
     &                                   nc_dstgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_address', 
     &                                   nc_srcgrdadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_address', 
     &                                   nc_dstgrdadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'remap_matrix', 
     &                                   nc_rmpmatrix_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     read all variables
C
C-----------------------------------------------------------------------

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrddims_id, 
     &                        grid1_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrdimask_id, 
     &                        grid1_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcntrlat_id, 
     &                                       grid1_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcntrlon_id, 
     &                                       grid1_center_lon)
      call netcdf_error_handler(ncstat)

      grid1_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_srcgrdcntrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')
        grid1_center_lat = grid1_center_lat*deg2rad
        grid1_center_lon = grid1_center_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid1 center lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcrnrlat_id, 
     &                                       grid1_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcrnrlon_id, 
     &                                       grid1_corner_lon)
      call netcdf_error_handler(ncstat)

      grid1_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_srcgrdcrnrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')
        grid1_corner_lat = grid1_corner_lat*deg2rad
        grid1_corner_lon = grid1_corner_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid1 corner lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdarea_id, 
     &                                       grid1_area)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdfrac_id, 
     &                                       grid1_frac)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrddims_id, 
     &                        grid2_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrdimask_id, 
     &                        grid2_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcntrlat_id, 
     &                                       grid2_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcntrlon_id, 
     &                                       grid2_center_lon)
      call netcdf_error_handler(ncstat)

      grid2_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_dstgrdcntrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')
        grid2_center_lat = grid2_center_lat*deg2rad
        grid2_center_lon = grid2_center_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid2 center lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcrnrlat_id, 
     &                                       grid2_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcrnrlon_id, 
     &                                       grid2_corner_lon)
      call netcdf_error_handler(ncstat)

      grid2_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_dstgrdcrnrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')
        grid2_corner_lat = grid2_corner_lat*deg2rad
        grid2_corner_lon = grid2_corner_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid2 corner lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdarea_id, 
     &                                       grid2_area)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdfrac_id, 
     &                                       grid2_frac)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrdadd_id, 
     &                        grid1_add_map1)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrdadd_id, 
     &                        grid2_add_map1)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                                       wts_map1)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     initialize logical mask 
C
C-----------------------------------------------------------------------

      where (grid1_mask_int == 1)
        grid1_mask = .true.
      elsewhere
        grid1_mask = .false.
      endwhere
      where (grid2_mask_int == 1)
        grid2_mask = .true.
      elsewhere
        grid2_mask = .false.
      endwhere
      deallocate(grid1_mask_int, grid2_mask_int)

C-----------------------------------------------------------------------
C
C     close input file
C
C-----------------------------------------------------------------------

      ncstat = nf_close(nc_file_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------

      end subroutine read_remap_scrip

C***********************************************************************

      subroutine read_remap_scrip2

C-----------------------------------------------------------------------
C
C     the routine reads a netCDF file to extract remapping info
C     in SCRIP format
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      character (char_len) ::
     &  grid1_name           ! grid name for source grid
     &, grid2_name           ! grid name for dest   grid

      integer (kind=int_kind) ::  
     &  n                    ! dummy index

      integer (kind=int_kind), dimension(:), allocatable ::
     &  grid1_mask_int,      ! integer masks to determine
     &  grid2_mask_int       ! cells that participate in map

C-----------------------------------------------------------------------
C
C     read some additional global attributes
C
C-----------------------------------------------------------------------

      !***
      !*** source and destination grid names
      !***

      grid1_name = ' '
      grid2_name = ' '
      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'source_grid',
     &                          grid1_name)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'dest_grid',
     &                          grid2_name)
      call netcdf_error_handler(ncstat)

      IF (ip_iprint .gt. 0) THEN
          print *,' '
          print *,'Remapping between:',trim(grid1_name)
          print *,'and ',trim(grid2_name)
          print *,' '
      ENDIF

C-----------------------------------------------------------------------
C
C     read dimension information
C
C-----------------------------------------------------------------------

      ncstat = nf_inq_dimid(nc_file_id, 'src_grid_size', 
     &                      nc_srcgrdsize_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdsize_id, grid1_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'dst_grid_size', 
     &                      nc_dstgrdsize_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdsize_id, grid2_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'src_grid_corners', 
     &                      nc_srcgrdcorn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdcorn_id, 
     &                       grid1_corners)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'dst_grid_corners', 
     &                      nc_dstgrdcorn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdcorn_id, 
     &                       grid2_corners)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'src_grid_rank', 
     &                      nc_srcgrdrank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdrank_id, 
     &                       grid1_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'dst_grid_rank', 
     &                      nc_dstgrdrank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdrank_id, 
     &                       grid2_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'num_links', 
     &                      nc_numlinks_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_numlinks_id, 
     &                       num_links_map2)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'num_wgts', 
     &                      nc_numwgts_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_numwgts_id, num_wts)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     allocate arrays
C
C-----------------------------------------------------------------------

       IF(.not.allocated( grid1_dims      ))
     & allocate( grid1_dims      (grid1_rank))
       IF(.not.allocated(grid1_center_lat))
     & allocate(grid1_center_lat(grid1_size)) 
       IF(.not.allocated(grid1_center_lon))
     & allocate(grid1_center_lon(grid1_size))
       IF(.not.allocated(grid1_area      ))
     & allocate(grid1_area      (grid1_size))
       IF(.not.allocated(grid1_frac      ))
     & allocate(grid1_frac      (grid1_size))
       IF(.not.allocated(grid1_mask      ))
     & allocate(grid1_mask      (grid1_size))
       IF(.not.allocated(grid1_mask_int  ))
     & allocate(grid1_mask_int  (grid1_size))
       IF(.not.allocated(grid1_corner_lat))
     & allocate(grid1_corner_lat(grid1_corners, grid1_size))
       IF(.not.allocated(grid1_corner_lon))
     & allocate(grid1_corner_lon(grid1_corners, grid1_size) )

       IF(.not.allocated( grid2_dims      ))
     & allocate( grid2_dims      (grid2_rank))
       IF(.not.allocated(grid2_center_lat))
     & allocate(grid2_center_lat(grid2_size)) 
       IF(.not.allocated(grid2_center_lon))
     & allocate(grid2_center_lon(grid2_size))
       IF(.not.allocated(grid2_area      ))
     & allocate(grid2_area      (grid2_size))
       IF(.not.allocated(grid2_frac      ))
     & allocate(grid2_frac      (grid2_size))
       IF(.not.allocated(grid2_mask      ))
     & allocate(grid2_mask      (grid2_size))
       IF(.not.allocated(grid2_mask_int  ))
     & allocate(grid2_mask_int  (grid2_size))
       IF(.not.allocated(grid2_corner_lat))
     & allocate(grid2_corner_lat(grid2_corners, grid2_size))
       IF(.not.allocated(grid2_corner_lon))
     & allocate(grid2_corner_lon(grid2_corners, grid2_size) )

       IF(.not.allocated( grid1_add_map2))
     & allocate( grid1_add_map2(num_links_map2))
       IF(.not.allocated(grid2_add_map2))
     & allocate(grid2_add_map2(num_links_map2))
       IF(.not.allocated(wts_map2))
     & allocate(wts_map2(num_wts,num_links_map2) )

C-----------------------------------------------------------------------
C
C     get variable ids
C
C-----------------------------------------------------------------------

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_dims', 
     &                      nc_srcgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_imask', 
     &                      nc_srcgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_center_lat', 
     &                                   nc_srcgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_center_lon', 
     &                                   nc_srcgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_corner_lat', 
     &                                   nc_srcgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_corner_lon', 
     &                                   nc_srcgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_area', 
     &                                   nc_srcgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_frac', 
     &                                   nc_srcgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_dims', 
     &                      nc_dstgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_imask', 
     &                      nc_dstgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_center_lat', 
     &                                   nc_dstgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_center_lon', 
     &                                   nc_dstgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_corner_lat', 
     &                                   nc_dstgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_corner_lon', 
     &                                   nc_dstgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_area', 
     &                                   nc_dstgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_frac', 
     &                                   nc_dstgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'src_address', 
     &                                   nc_srcgrdadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_address', 
     &                                   nc_dstgrdadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'remap_matrix', 
     &                                   nc_rmpmatrix_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     read all variables
C
C-----------------------------------------------------------------------

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrddims_id, 
     &                        grid1_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrdimask_id, 
     &                        grid1_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcntrlat_id, 
     &                                       grid1_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcntrlon_id, 
     &                                       grid1_center_lon)
      call netcdf_error_handler(ncstat)

      grid1_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_srcgrdcntrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')
        grid1_center_lat = grid1_center_lat*deg2rad
        grid1_center_lon = grid1_center_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid1 center lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcrnrlat_id, 
     &                                       grid1_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcrnrlon_id, 
     &                                       grid1_corner_lon)
      call netcdf_error_handler(ncstat)

      grid1_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_srcgrdcrnrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')
        grid1_corner_lat = grid1_corner_lat*deg2rad
        grid1_corner_lon = grid1_corner_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid1 corner lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdarea_id, 
     &                                       grid1_area)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdfrac_id, 
     &                                       grid1_frac)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrddims_id, 
     &                        grid2_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrdimask_id, 
     &                        grid2_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcntrlat_id, 
     &                                       grid2_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcntrlon_id, 
     &                                       grid2_center_lon)
      call netcdf_error_handler(ncstat)

      grid2_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_dstgrdcntrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')
        grid2_center_lat = grid2_center_lat*deg2rad
        grid2_center_lon = grid2_center_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid2 center lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcrnrlat_id, 
     &                                       grid2_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcrnrlon_id, 
     &                                       grid2_corner_lon)
      call netcdf_error_handler(ncstat)

      grid2_units = ' '
      ncstat = nf_get_att_text(nc_file_id, nc_dstgrdcrnrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')
        grid2_corner_lat = grid2_corner_lat*deg2rad
        grid2_corner_lon = grid2_corner_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid2 corner lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdarea_id, 
     &                                       grid2_area)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdfrac_id, 
     &                                       grid2_frac)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrdadd_id, 
     &                        grid1_add_map2)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrdadd_id, 
     &                        grid2_add_map2)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                                       wts_map2)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     initialize logical mask 
C
C-----------------------------------------------------------------------

      where (grid1_mask_int == 1)
        grid1_mask = .true.
      elsewhere
        grid1_mask = .false.
      endwhere
      where (grid2_mask_int == 1)
        grid2_mask = .true.
      elsewhere
        grid2_mask = .false.
      endwhere
      deallocate(grid1_mask_int, grid2_mask_int)

C-----------------------------------------------------------------------
C
C     close input file
C
C-----------------------------------------------------------------------

      ncstat = nf_close(nc_file_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------

      end subroutine read_remap_scrip2

C***********************************************************************

      subroutine read_remap_csm

C-----------------------------------------------------------------------
C
C     the routine reads a netCDF file to extract remapping info
C     in NCAR-CSM format
C
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      character (char_len) ::
     &  grid1_name           ! grid name for source grid
     &, grid2_name           ! grid name for dest   grid

      integer (kind=int_kind) ::
     &  nc_numwgts1_id    ! extra netCDF id for num_wgts > 1 
     &, nc_rmpmatrix2_id  ! extra netCDF id for high-order remap matrix

      real (kind=dbl_kind), dimension(:),allocatable ::
     &  wts1              ! CSM wants single array for 1st-order wts

      real (kind=dbl_kind), dimension(:,:),allocatable ::
     &  wts2              ! write remaining weights in different array

      integer (kind=int_kind) ::  
     &  n                    ! dummy index

      integer (kind=int_kind), dimension(:), allocatable ::
     &  grid1_mask_int,      ! integer masks to determine
     &  grid2_mask_int       ! cells that participate in map

C-----------------------------------------------------------------------
C
C     read some additional global attributes
C
C-----------------------------------------------------------------------

      !***
      !*** source and destination grid names
      !***

      grid1_name = ' '
      grid2_name = ' '
      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'domain_a',
     &                          grid1_name)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_att_text (nc_file_id, NF_GLOBAL, 'domain_b',
     &                          grid2_name)
      call netcdf_error_handler(ncstat)

      print *,' '
      print *,'Remapping between:',trim(grid1_name)
      print *,'and ',trim(grid2_name)
      print *,' '

C-----------------------------------------------------------------------
C
C     read dimension information
C
C-----------------------------------------------------------------------

      ncstat = nf_inq_dimid(nc_file_id, 'n_a', nc_srcgrdsize_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdsize_id, grid1_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'n_b', nc_dstgrdsize_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdsize_id, grid2_size)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'nv_a', nc_srcgrdcorn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdcorn_id, 
     &                       grid1_corners)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'nv_b', nc_dstgrdcorn_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdcorn_id, 
     &                       grid2_corners)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'src_grid_rank', 
     &                      nc_srcgrdrank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_srcgrdrank_id, 
     &                       grid1_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'dst_grid_rank', 
     &                      nc_dstgrdrank_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_dstgrdrank_id, 
     &                       grid2_rank)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'n_s', 
     &                      nc_numlinks_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_numlinks_id, 
     &                       num_links_map1)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_dimid(nc_file_id, 'num_wgts', 
     &                      nc_numwgts_id)
      call netcdf_error_handler(ncstat)
      ncstat = nf_inq_dimlen(nc_file_id, nc_numwgts_id, num_wts)
      call netcdf_error_handler(ncstat)

      if (num_wts > 1) then
        ncstat = nf_inq_dimid(nc_file_id, 'num_wgts1', 
     &                        nc_numwgts1_id)
        call netcdf_error_handler(ncstat)
      endif

C-----------------------------------------------------------------------
C
C     allocate arrays
C
C-----------------------------------------------------------------------

       IF(.not.allocated(grid1_dims      ))
     & allocate(grid1_dims      (grid1_rank))
       IF(.not.allocated(grid1_center_lat))
     & allocate(grid1_center_lat(grid1_size)) 
       IF(.not.allocated(grid1_center_lon))
     & allocate(grid1_center_lon(grid1_size))
       IF(.not.allocated(grid1_area      ))
     & allocate(grid1_area      (grid1_size))
       IF(.not.allocated(grid1_frac      ))
     & allocate(grid1_frac      (grid1_size))
       IF(.not.allocated(grid1_mask      ))
     & allocate(grid1_mask      (grid1_size))
       IF(.not.allocated(grid1_mask_int  ))
     & allocate(grid1_mask_int  (grid1_size))
       IF(.not.allocated(grid1_corner_lat))
     & allocate(grid1_corner_lat(grid1_corners, grid1_size))
       IF(.not.allocated(grid1_corner_lon))
     & allocate(grid1_corner_lon(grid1_corners, grid1_size) )

       IF(.not.allocated(grid2_dims      ))
     & allocate(grid2_dims      (grid2_rank))
       IF(.not.allocated(grid2_center_lat))
     & allocate(grid2_center_lat(grid2_size)) 
       IF(.not.allocated(grid2_center_lon))
     & allocate(grid2_center_lon(grid2_size))
       IF(.not.allocated(grid2_area      ))
     & allocate(grid2_area      (grid2_size))
       IF(.not.allocated(grid2_frac      ))
     & allocate(grid2_frac      (grid2_size))
       IF(.not.allocated(grid2_mask      ))
     & allocate(grid2_mask      (grid2_size))
       IF(.not.allocated(grid2_mask_int  ))
     & allocate(grid2_mask_int  (grid2_size))
       IF(.not.allocated(grid2_corner_lat))
     & allocate(grid2_corner_lat(grid2_corners, grid2_size))
       IF(.not.allocated(grid2_corner_lon))
     & allocate(grid2_corner_lon(grid2_corners, grid2_size) )

       IF(.not.allocated(grid1_add_map1))
     & allocate(grid1_add_map1(num_links_map1))
       IF(.not.allocated(grid2_add_map1))
     & allocate(grid2_add_map1(num_links_map1))
       IF(.not.allocated(wts_map1))
     & allocate(wts_map1(num_wts,num_links_map1))
       IF(.not.allocated(wts1))
     & allocate(wts1(num_links_map1))
       IF(.not.allocated(wts2))
     & allocate(wts2(num_wts-1,num_links_map1) )

C-----------------------------------------------------------------------
C
C     get variable ids
C
C-----------------------------------------------------------------------

      ncstat = nf_inq_varid(nc_file_id, 'src_grid_dims', 
     &                      nc_srcgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'mask_a', 
     &                      nc_srcgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'yc_a', nc_srcgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'xc_a', nc_srcgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'yv_a', nc_srcgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'xv_a', nc_srcgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'area_a', nc_srcgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'frac_a', nc_srcgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'dst_grid_dims', 
     &                      nc_dstgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'mask_b', 
     &                      nc_dstgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'yc_b', nc_dstgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'xc_b', nc_dstgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'yv_b', nc_dstgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'xv_b', nc_dstgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'area_b', nc_dstgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'frac_b', nc_dstgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'col', nc_srcgrdadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'row', nc_dstgrdadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_inq_varid(nc_file_id, 'S', nc_rmpmatrix_id)
      call netcdf_error_handler(ncstat)

      if (num_wts > 1) then
        ncstat = nf_inq_varid(nc_file_id, 'S2', nc_rmpmatrix2_id)
        call netcdf_error_handler(ncstat)
      endif

C-----------------------------------------------------------------------
C
C     read all variables
C
C-----------------------------------------------------------------------

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrddims_id, 
     &                        grid1_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrdimask_id, 
     &                        grid1_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcntrlat_id, 
     &                                       grid1_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcntrlon_id, 
     &                                       grid1_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_att_text(nc_file_id, nc_srcgrdcntrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')
        grid1_center_lat = grid1_center_lat*deg2rad
        grid1_center_lon = grid1_center_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid1 center lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcrnrlat_id, 
     &                                       grid1_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdcrnrlon_id, 
     &                                       grid1_corner_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_att_text(nc_file_id, nc_srcgrdcrnrlat_id, 'units',
     &                         grid1_units)
      call netcdf_error_handler(ncstat)

      select case (grid1_units(1:7))
      case ('degrees')
        grid1_corner_lat = grid1_corner_lat*deg2rad
        grid1_corner_lon = grid1_corner_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid1 corner lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdarea_id, 
     &                                       grid1_area)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_srcgrdfrac_id, 
     &                                       grid1_frac)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrddims_id, 
     &                        grid2_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrdimask_id, 
     &                        grid2_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcntrlat_id, 
     &                                       grid2_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcntrlon_id, 
     &                                       grid2_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_att_text(nc_file_id, nc_dstgrdcntrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')
        grid2_center_lat = grid2_center_lat*deg2rad
        grid2_center_lon = grid2_center_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid2 center lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcrnrlat_id, 
     &                                       grid2_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdcrnrlon_id, 
     &                                       grid2_corner_lon)
      call netcdf_error_handler(ncstat)


      ncstat = nf_get_att_text(nc_file_id, nc_dstgrdcrnrlat_id, 'units',
     &                         grid2_units)
      call netcdf_error_handler(ncstat)

      select case (grid2_units(1:7))
      case ('degrees')
        grid2_corner_lat = grid2_corner_lat*deg2rad
        grid2_corner_lon = grid2_corner_lon*deg2rad
      case ('radians')
        !*** no conversion necessary
      case default
        print *,'unknown units supplied for grid2 corner lat/lon: '
        print *,'proceeding assuming radians'
      end select

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdarea_id, 
     &                                       grid2_area)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_dstgrdfrac_id, 
     &                                       grid2_frac)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_srcgrdadd_id, 
     &                        grid1_add_map1)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_int(nc_file_id, nc_dstgrdadd_id, 
     &                        grid2_add_map1)
      call netcdf_error_handler(ncstat)

      ncstat = nf_get_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                                       wts1)
      wts_map1(1,:) = wts1
      deallocate(wts1)

      if (num_wts > 1) then
        ncstat = nf_get_var_double(nc_file_id, nc_rmpmatrix2_id, 
     &                                         wts2)
        wts_map1(2:,:) = wts2
        deallocate(wts2)
      endif
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     initialize logical mask 
C
C-----------------------------------------------------------------------

      where (grid1_mask_int == 1)
        grid1_mask = .true.
      elsewhere
        grid1_mask = .false.
      endwhere
      where (grid2_mask_int == 1)
        grid2_mask = .true.
      elsewhere
        grid2_mask = .false.
      endwhere
      deallocate(grid1_mask_int, grid2_mask_int)

C-----------------------------------------------------------------------
C
C     close input file
C
C-----------------------------------------------------------------------

      ncstat = nf_close(nc_file_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------

      end subroutine read_remap_csm

C***********************************************************************

      end module remap_read

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     this routine performs a remapping based on addresses and weights
C     computed in a setup phase
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: remap.f,v 1.5 2000/04/19 21:56:25 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module remap_mod

C-----------------------------------------------------------------------
C
C     this module contains the routines for performing the actual
C     remappings
C
C-----------------------------------------------------------------------

      use kinds_mod    ! defines common data types
      use constants    ! defines common constants

      implicit none

C***********************************************************************

      contains

C***********************************************************************

      subroutine remap(dst_array, map_wts, dst_add, src_add, 
     &                 src_array, src_grad1, src_grad2, src_grad3)

C-----------------------------------------------------------------------
C
C     performs the remapping based on weights computed elsewhere
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input arrays
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(:), intent(in) ::
     &     dst_add,     ! destination address for each link
     &     src_add      ! source      address for each link

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &     map_wts      ! remapping weights for each link

      real (kind=dbl_kind), dimension(:), intent(in) ::
     &     src_array    ! array with source field to be remapped

      real (kind=dbl_kind), dimension(:), intent(in), optional ::
     &     src_grad1    ! gradient arrays on source grid necessary for
     &,    src_grad2    ! higher-order remappings
     &,    src_grad3

C-----------------------------------------------------------------------
C
C     output variables
C
C-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(:), intent(inout) ::
     &     dst_array    ! array for remapped field on destination grid

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, iorder

C-----------------------------------------------------------------------
C
C     check the order of the interpolation
C
C-----------------------------------------------------------------------

      if (present(src_grad1)) then
        iorder = 2
      else
        iorder = 1
      endif

C-----------------------------------------------------------------------
C
C     first order remapping
C
C-----------------------------------------------------------------------

      dst_array = zero

      select case (iorder)
      case(1)

        do n=1,size(dst_add)
          dst_array(dst_add(n)) = dst_array(dst_add(n)) + 
     &                            src_array(src_add(n))*map_wts(1,n)
        end do

C-----------------------------------------------------------------------
C
C     second order remapping
C
C-----------------------------------------------------------------------

      case(2)

        if (size(map_wts,DIM=1) == 3) then
          do n=1,size(dst_add)
            dst_array(dst_add(n)) = dst_array(dst_add(n)) +
     &                              src_array(src_add(n))*map_wts(1,n) +
     &                              src_grad1(src_add(n))*map_wts(2,n) +
     &                              src_grad2(src_add(n))*map_wts(3,n)
          end do
        else if (size(map_wts,DIM=1) == 4) then
          do n=1,size(dst_add)
            dst_array(dst_add(n)) = dst_array(dst_add(n)) +
     &                              src_array(src_add(n))*map_wts(1,n) +
     &                              src_grad1(src_add(n))*map_wts(2,n) +
     &                              src_grad2(src_add(n))*map_wts(3,n) +
     &                              src_grad3(src_add(n))*map_wts(4,n)
          end do
        endif

      end select

C-----------------------------------------------------------------------

      end subroutine remap

C***********************************************************************

      end module remap_mod

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This program creates a remapping grid file for Gaussian lat/lon
C     grids (for spectral transform codes).
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: convertgauss.f,v 1.3 2000/04/19 22:05:57 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C***********************************************************************

      subroutine convert_gauss(nx,ny,grid_size,grid_name,grid_file_out)

C-----------------------------------------------------------------------
C
C     This file creates a remapping grid file for a Gaussian grid
C
C-----------------------------------------------------------------------

      USE palm_user_param
      use kinds_mod
      use constants
      use iounits
      use netcdf_mod

      implicit none

C-----------------------------------------------------------------------
C
C     variables that describe the grid
C
C     T42: nx=128 ny=64
C     T62: nx=192 ny=94
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: 
     &             nx, ny,grid_size

      integer (kind=int_kind), parameter ::
     &             grid_rank = 2,
     &             grid_corners = 4

      character(char_len), intent(in) :: 
     &             grid_name,
     &             grid_file_out

      integer (kind=int_kind), dimension(grid_rank) ::
     &             grid_dims

C-----------------------------------------------------------------------
C
C     grid coordinates and masks
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(grid_size) ::
     &             grid_imask

      real (kind=dbl_kind), dimension(grid_size) ::
     &             grid_center_lat,  ! lat/lon coordinates for
     &             grid_center_lon   ! each grid center in degrees

      real (kind=dbl_kind), dimension(grid_corners,grid_size) ::
     &             grid_corner_lat,  ! lat/lon coordinates for
     &             grid_corner_lon   ! each grid corner in degrees

C-----------------------------------------------------------------------
C
C     other local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: i, j, iunit, atm_add

      integer (kind=int_kind) ::
     &        ncstat,            ! general netCDF status variable
     &        nc_grid_id,        ! netCDF grid dataset id
     &        nc_gridsize_id,    ! netCDF grid size dim id
     &        nc_gridcorn_id,    ! netCDF grid corner dim id
     &        nc_gridrank_id,    ! netCDF grid rank dim id
     &        nc_griddims_id,    ! netCDF grid dimension size id
     &        nc_grdcntrlat_id,  ! netCDF grid center lat id
     &        nc_grdcntrlon_id,  ! netCDF grid center lon id
     &        nc_grdimask_id,    ! netCDF grid mask id
     &        nc_grdcrnrlat_id,  ! netCDF grid corner lat id
     &        nc_grdcrnrlon_id   ! netCDF grid corner lon id

      integer (kind=int_kind), dimension(2) ::
     &        nc_dims2_id        ! netCDF dim id array for 2-d arrays

      real (kind=dbl_kind) :: dlon, minlon, maxlon, centerlon,
     &                              minlat, maxlat, centerlat

      real (kind=dbl_kind), dimension(ny) :: gauss_root, gauss_wgt
      real (kind=dbl_kind) :: w
      integer (kind=int_kind) :: lwork, ierror

C-----------------------------------------------------------------------
C
C     compute longitudes of cell centers and corners.  set up alon
C     array for search routine.
C
C-----------------------------------------------------------------------

      grid_dims(1) = nx
      grid_dims(2) = ny

      dlon = 360./nx

      do i=1,nx

          minlon = (i-1)*dlon 
          maxlon =  i   *dlon
          centerlon = minlon + half*dlon
C        centerlon = (i-1)*dlon
C        minlon = centerlon - half*dlon
C        maxlon = centerlon + half*dlon

        do j=1,ny
          atm_add = (j-1)*nx + i

          grid_center_lon(atm_add  ) = centerlon
          grid_corner_lon(1,atm_add) = minlon
          grid_corner_lon(2,atm_add) = maxlon
          grid_corner_lon(3,atm_add) = maxlon
          grid_corner_lon(4,atm_add) = minlon
        end do

      end do

C-----------------------------------------------------------------------
C
C     compute Gaussian latitudes and store in gauss_wgt.
C
C-----------------------------------------------------------------------

C      call gquad(ny, gauss_root, gauss_wgt)
C      do j=1,ny
C        gauss_wgt(j) = pih - gauss_root(ny+1-j)
C      end do

      CALL gaqd(ny,gauss_root,gauss_wgt,w,lwork,ierror)
      do j=1,ny
        gauss_wgt(j) = gauss_root(j) - (gauss_root(1)+gauss_root(ny))/
     $      2._dbl_kind
      end do

C-----------------------------------------------------------------------
C
C     compute latitudes at cell centers and corners.  set up alat 
C     array for search routine.
C
C-----------------------------------------------------------------------

      do j=1,ny
        centerlat = gauss_wgt(j)

        if (j .eq. 1) then
          minlat = -pih
        else
          minlat = ATAN((COS(gauss_wgt(j-1)) - 
     &                   COS(gauss_wgt(j  )))/
     &                  (SIN(gauss_wgt(j  )) - 
     &                   SIN(gauss_wgt(j-1))))
        endif

        if (j .eq. ny) then
          maxlat = pih
        else
          maxlat = ATAN((COS(gauss_wgt(j  )) - 
     &                   COS(gauss_wgt(j+1)))/
     &                  (SIN(gauss_wgt(j+1)) - 
     &                   SIN(gauss_wgt(j  ))))
        endif

        do i=1,nx
          atm_add = (j-1)*nx + i
          grid_center_lat(atm_add  ) = centerlat*360./pi2
          grid_corner_lat(1,atm_add) = minlat*360./pi2
          grid_corner_lat(2,atm_add) = minlat*360./pi2
          grid_corner_lat(3,atm_add) = maxlat*360./pi2
          grid_corner_lat(4,atm_add) = maxlat*360./pi2
        end do

      end do

C-----------------------------------------------------------------------
C
C     define mask
C
C-----------------------------------------------------------------------

      grid_imask = 1

C-----------------------------------------------------------------------
C
C     set up attributes for netCDF file
C
C-----------------------------------------------------------------------

      !***
      !*** create netCDF dataset for this grid
      !***

      ncstat = nf_create (grid_file_out, NF_CLOBBER,
     &                    nc_grid_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, NF_GLOBAL, 'title',
     &                          len_trim(grid_name), grid_name)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid size dimension
      !***

      ncstat = nf_def_dim (nc_grid_id, 'grid_size', grid_size, 
     &                     nc_gridsize_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner dimension
      !***

      ncstat = nf_def_dim (nc_grid_id, 'grid_corners', grid_corners, 
     &                     nc_gridcorn_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid rank dimension
      !***

      ncstat = nf_def_dim (nc_grid_id, 'grid_rank', grid_rank, 
     &                     nc_gridrank_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid dimension size array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_dims', NF_INT,
     &                     1, nc_gridrank_id, nc_griddims_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid center latitude array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_center_lat', NF_DOUBLE,
     &                     1, nc_gridsize_id, nc_grdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcntrlat_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid center longitude array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_center_lon', NF_DOUBLE,
     &                     1, nc_gridsize_id, nc_grdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcntrlon_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid mask
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_imask', NF_INT,
     &                     1, nc_gridsize_id, nc_grdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdimask_id, 'units',
     &                          8, 'unitless')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner latitude array
      !***

      nc_dims2_id(1) = nc_gridcorn_id
      nc_dims2_id(2) = nc_gridsize_id

      ncstat = nf_def_var (nc_grid_id, 'grid_corner_lat', NF_DOUBLE,
     &                     2, nc_dims2_id, nc_grdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcrnrlat_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner longitude array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_corner_lon', NF_DOUBLE,
     &                     2, nc_dims2_id, nc_grdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcrnrlon_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** end definition stage
      !***

      ncstat = nf_enddef(nc_grid_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     write grid data
C
C-----------------------------------------------------------------------

      ncstat = nf_put_var_int(nc_grid_id, nc_griddims_id, grid_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_grid_id, nc_grdimask_id, grid_imask)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcntrlat_id, 
     &                           grid_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcntrlon_id, 
     &                           grid_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcrnrlat_id, 
     &                           grid_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcrnrlon_id, 
     &                           grid_corner_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_close(nc_grid_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------

      end subroutine convert_gauss

C***********************************************************************

      subroutine gquad(l,root,w)

C-----------------------------------------------------------------------
C
C     This subroutine finds the l roots (in theta) and gaussian weights 
C     associated with the legendre polynomial of degree l > 1.
C
C-----------------------------------------------------------------------

      use kinds_mod
      use constants

      implicit none

C-----------------------------------------------------------------------
C
C     intent(in)
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) :: l

C-----------------------------------------------------------------------
C
C     intent(out)
C
C-----------------------------------------------------------------------

      real (kind=dbl_kind), dimension(l), intent(out) ::
     &      root, w

C-----------------------------------------------------------------------
C
C     local
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: l1, l2, l22, l3, k, i, j

      real (kind=dbl_kind) :: 
     &     del,co,p1,p2,p3,t1,t2,slope,s,c,pp1,pp2,p00

C-----------------------------------------------------------------------
C
C     Define useful constants.
C
C-----------------------------------------------------------------------

      del= pi/float(4*l)
      l1 = l+1
      co = float(2*l+3)/float(l1**2)
      p2 = 1.0
      t2 = -del
      l2 = l/2
      k = 1
      p00 = one/sqrt(two)

C-----------------------------------------------------------------------
C
C     Start search for each root by looking for crossing point.
C
C-----------------------------------------------------------------------

      do i=1,l2
   10    t1 = t2
         t2 = t1+del
         p1 = p2
         s = sin(t2)
         c = cos(t2)
         pp1 = 1.0
         p3 = p00
         do j=1,l1
            pp2 = pp1
            pp1 = p3
            p3 = 2.0*sqrt((float(j**2)-0.250)/float(j**2))*c*pp1-
     &           sqrt(float((2*j+1)*(j-1)*(j-1))/
     &           float((2*j-3)*j*j))*pp2
         end do
         p2 = pp1
         if ((k*p2).gt.0) goto 10

C-----------------------------------------------------------------------
C
C        Now converge using Newton-Raphson.
C
C-----------------------------------------------------------------------

         k = -k
   20    continue
            slope = (t2-t1)/(p2-p1)
            t1 = t2
            t2 = t2-slope*p2
            p1 = p2
            s = sin(t2)
            c = cos(t2)
            pp1 = 1.0
            p3 = p00
            do j=1,l1
               pp2 = pp1
               pp1 = p3
               p3 = 2.0*sqrt((float(j**2)-0.250)/float(j**2))*c*pp1-
     &              sqrt(float((2*j+1)*(j-1)*(j-1))/
     &              float((2*j-3)*j*j))*pp2
            end do
            p2 = pp1
         if (abs(p2).gt.1.e-10) goto 20
         root(i) = t2
         w(i) = co*(sin(t2)/p3)**2
      end do

C-----------------------------------------------------------------------
C
C     If l is odd, take care of odd point.
C
C-----------------------------------------------------------------------

      l22 = 2*l2
      if (l22 .ne. l) then
         l2 = l2+1
         t2 = pi/2.0
         root(l2) = t2
         s = sin(t2)
         c = cos(t2)
         pp1 = 1.0
         p3 = p00
         do j=1,l1
            pp2 = pp1
            pp1 = p3
            p3 = 2.0*sqrt((float(j**2)-0.250)/float(j**2))*c*pp1-
     &           sqrt(float((2*j+1)*(j-1)*(j-1))/
     &           float((2*j-3)*j*j))*pp2
         end do
         p2 = pp1
         w(l2) = co/p3**2
      endif

C-----------------------------------------------------------------------
C
C     Use symmetry to compute remaining roots and weights.
C
C-----------------------------------------------------------------------

      l3 = l2+1
      do i=l3,l
         root(i) = pi-root(l-i+1)
         w(i) = w(l-i+1)
      end do

C-----------------------------------------------------------------------

      end subroutine gquad

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This program creates a remapping grid file for a lat/lon grid.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: create_latlon.f,v 1.1 2000/04/19 22:05:58 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C***********************************************************************

      subroutine create_latlon(nx,ny,grid_size,grid_name,grid_file_out)

C-----------------------------------------------------------------------
C
C     This file creates a remapping grid file for a Gaussian grid
C
C-----------------------------------------------------------------------

      USE palm_user_param
      use kinds_mod
      use constants
      use iounits
      use netcdf_mod

      implicit none

C-----------------------------------------------------------------------
C
C     variables that describe the grid
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &             nx , ny ,grid_size 
      integer (kind=int_kind), parameter ::
     &             grid_rank = 2,
     &             grid_corners = 4

      character(char_len), intent(in) :: 
     &             grid_name,
     &             grid_file_out

      integer (kind=int_kind), dimension(grid_rank) ::
     &             grid_dims

C-----------------------------------------------------------------------
C
C     grid coordinates and masks
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(grid_size) ::
     &             grid_imask

      real (kind=dbl_kind), dimension(grid_size) ::
     &             grid_center_lat,  ! lat/lon coordinates for
     &             grid_center_lon   ! each grid center in degrees

      real (kind=dbl_kind), dimension(grid_corners,grid_size) ::
     &             grid_corner_lat,  ! lat/lon coordinates for
     &             grid_corner_lon   ! each grid corner in degrees

C-----------------------------------------------------------------------
C
C     other local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: i, j, iunit, atm_add

      integer (kind=int_kind) ::
     &        ncstat,            ! general netCDF status variable
     &        nc_grid_id,        ! netCDF grid dataset id
     &        nc_gridsize_id,    ! netCDF grid size dim id
     &        nc_gridcorn_id,    ! netCDF grid corner dim id
     &        nc_gridrank_id,    ! netCDF grid rank dim id
     &        nc_griddims_id,    ! netCDF grid dimension size id
     &        nc_grdcntrlat_id,  ! netCDF grid center lat id
     &        nc_grdcntrlon_id,  ! netCDF grid center lon id
     &        nc_grdimask_id,    ! netCDF grid mask id
     &        nc_grdcrnrlat_id,  ! netCDF grid corner lat id
     &        nc_grdcrnrlon_id   ! netCDF grid corner lon id

      integer (kind=int_kind), dimension(2) ::
     &        nc_dims2_id        ! netCDF dim id array for 2-d arrays

      real (kind=dbl_kind) :: dlon, minlon, maxlon, centerlon,
     &                        dlat, minlat, maxlat, centerlat

C-----------------------------------------------------------------------
C
C     compute longitudes and latitudes of cell centers and corners.
C
C-----------------------------------------------------------------------

      grid_dims(1) = nx
      grid_dims(2) = ny

      dlon = 360./nx
      dlat = 180./ny

      do j=1,ny

        minlat = -90._dbl_kind + (j-1)*dlat
        maxlat = -90._dbl_kind +  j   *dlat
        centerlat = minlat + half*dlat

        do i=1,nx
C          centerlon = (i-1)*dlon 
C          minlon = centerlon - half*dlon
C          maxlon = centerlon + half*dlon
          minlon = (i-1)*dlon 
          maxlon =  i   *dlon
          centerlon = minlon + half*dlon
 
          atm_add = (j-1)*nx + i

          grid_center_lat(atm_add  ) = centerlat
          grid_corner_lat(1,atm_add) = minlat
          grid_corner_lat(2,atm_add) = minlat
          grid_corner_lat(3,atm_add) = maxlat
          grid_corner_lat(4,atm_add) = maxlat

          grid_center_lon(atm_add  ) = centerlon
          grid_corner_lon(1,atm_add) = minlon
          grid_corner_lon(2,atm_add) = maxlon
          grid_corner_lon(3,atm_add) = maxlon
          grid_corner_lon(4,atm_add) = minlon
        end do
      end do

C-----------------------------------------------------------------------
C
C     define mask
C
C-----------------------------------------------------------------------

      grid_imask = 1

C-----------------------------------------------------------------------
C
C     set up attributes for netCDF file
C
C-----------------------------------------------------------------------

      !***
      !*** create netCDF dataset for this grid
      !***

      ncstat = nf_create (grid_file_out, NF_CLOBBER,
     &                    nc_grid_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, NF_GLOBAL, 'title',
     &                          len_trim(grid_name), grid_name)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid size dimension
      !***

      ncstat = nf_def_dim (nc_grid_id, 'grid_size', grid_size, 
     &                     nc_gridsize_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner dimension
      !***

      ncstat = nf_def_dim (nc_grid_id, 'grid_corners', grid_corners, 
     &                     nc_gridcorn_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid rank dimension
      !***

      ncstat = nf_def_dim (nc_grid_id, 'grid_rank', grid_rank, 
     &                     nc_gridrank_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid dimension size array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_dims', NF_INT,
     &                     1, nc_gridrank_id, nc_griddims_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid center latitude array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_center_lat', NF_DOUBLE,
     &                     1, nc_gridsize_id, nc_grdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcntrlat_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid center longitude array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_center_lon', NF_DOUBLE,
     &                     1, nc_gridsize_id, nc_grdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcntrlon_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid mask
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_imask', NF_INT,
     &                     1, nc_gridsize_id, nc_grdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdimask_id, 'units',
     &                          8, 'unitless')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner latitude array
      !***

      nc_dims2_id(1) = nc_gridcorn_id
      nc_dims2_id(2) = nc_gridsize_id

      ncstat = nf_def_var (nc_grid_id, 'grid_corner_lat', NF_DOUBLE,
     &                     2, nc_dims2_id, nc_grdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcrnrlat_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner longitude array
      !***

      ncstat = nf_def_var (nc_grid_id, 'grid_corner_lon', NF_DOUBLE,
     &                     2, nc_dims2_id, nc_grdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_grid_id, nc_grdcrnrlon_id, 'units',
     &                          7, 'degrees')
      call netcdf_error_handler(ncstat)

      !***
      !*** end definition stage
      !***

      ncstat = nf_enddef(nc_grid_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     write grid data
C
C-----------------------------------------------------------------------

      ncstat = nf_put_var_int(nc_grid_id, nc_griddims_id, grid_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_grid_id, nc_grdimask_id, grid_imask)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcntrlat_id, 
     &                           grid_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcntrlon_id, 
     &                           grid_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcrnrlat_id, 
     &                           grid_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_grid_id, nc_grdcrnrlon_id, 
     &                           grid_corner_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_close(nc_grid_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------

      end subroutine create_latlon

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This module contains routines for writing the remapping data to 
C     a file.  Before writing the data for each mapping, the links are 
C     sorted by destination grid address.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: remap_write.f,v 1.7 2001/08/21 21:06:42 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module remap_write

C-----------------------------------------------------------------------

      use kinds_mod     ! defines common data types
      use constants     ! defines common scalar constants
      use grids         ! module containing grid information
      use remap_vars    ! module containing remap information
      use netcdf_mod    ! module with netCDF stuff

      implicit none

C-----------------------------------------------------------------------
C
C     module variables
C
C-----------------------------------------------------------------------

      character(char_len), private :: 
     &   map_method       ! character string for map_type
     &,  normalize_opt    ! character string for normalization option
     &,  history          ! character string for history information
     &,  convention       ! character string for output convention

      character(8), private :: 
     &   cdate            ! character date string

      integer (kind=int_kind), dimension(:), allocatable, private ::
     &   src_mask_int     ! integer masks to determine
     &,  dst_mask_int     ! cells that participate in map

C-----------------------------------------------------------------------
C
C     various netCDF identifiers used by output routines
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), private ::
     &   ncstat               ! error flag for netCDF calls 
     &,  nc_file_id           ! id for netCDF file
     &,  nc_srcgrdsize_id     ! id for source grid size
     &,  nc_dstgrdsize_id     ! id for destination grid size
     &,  nc_srcgrdcorn_id     ! id for number of source grid corners
     &,  nc_dstgrdcorn_id     ! id for number of dest grid corners
     &,  nc_srcgrdrank_id     ! id for source grid rank
     &,  nc_dstgrdrank_id     ! id for dest grid rank
     &,  nc_numlinks_id       ! id for number of links in mapping
     &,  nc_numwgts_id        ! id for number of weights for mapping
     &,  nc_srcgrddims_id     ! id for source grid dimensions
     &,  nc_dstgrddims_id     ! id for dest grid dimensions
     &,  nc_srcgrdcntrlat_id  ! id for source grid center latitude
     &,  nc_dstgrdcntrlat_id  ! id for dest grid center latitude
     &,  nc_srcgrdcntrlon_id  ! id for source grid center longitude
     &,  nc_dstgrdcntrlon_id  ! id for dest grid center longitude
     &,  nc_srcgrdimask_id    ! id for source grid mask
     &,  nc_dstgrdimask_id    ! id for dest grid mask
     &,  nc_srcgrdcrnrlat_id  ! id for latitude of source grid corners
     &,  nc_srcgrdcrnrlon_id  ! id for longitude of source grid corners
     &,  nc_dstgrdcrnrlat_id  ! id for latitude of dest grid corners
     &,  nc_dstgrdcrnrlon_id  ! id for longitude of dest grid corners
     &,  nc_srcgrdarea_id     ! id for area of source grid cells
     &,  nc_dstgrdarea_id     ! id for area of dest grid cells
     &,  nc_srcgrdfrac_id     ! id for area fraction on source grid
     &,  nc_dstgrdfrac_id     ! id for area fraction on dest grid
     &,  nc_srcadd_id         ! id for map source address
     &,  nc_dstadd_id         ! id for map destination address
     &,  nc_rmpmatrix_id      ! id for remapping matrix

      integer (kind=int_kind), dimension(2), private ::
     &   nc_dims2_id  ! netCDF ids for 2d array dims

C***********************************************************************

      contains

C***********************************************************************

      subroutine write_remap(map1_name, map2_name, 
     &                       interp_file1, interp_file2, output_opt)

C-----------------------------------------------------------------------
C
C     calls correct output routine based on output format choice
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(in) ::
     &            map1_name,    ! name for mapping grid1 to grid2
     &            map2_name,    ! name for mapping grid2 to grid1
     &            interp_file1, ! filename for map1 remap data
     &            interp_file2, ! filename for map2 remap data
     &            output_opt    ! option for output conventions

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     define some common variables to be used in all routines
C
C-----------------------------------------------------------------------

      select case(norm_opt)
      case (norm_opt_none)
        normalize_opt = 'none'
      case (norm_opt_frcarea)
        normalize_opt = 'fracarea'
      case (norm_opt_dstarea)
        normalize_opt = 'destarea'
      end select

      select case(map_type)
      case(map_type_conserv)
        map_method = 'Conservative remapping'
      case(map_type_bilinear)
        map_method = 'Bilinear remapping'
      case(map_type_distwgt)
        map_method = 'Distance weighted avg of nearest neighbors'
      case(map_type_bicubic)
        map_method = 'Bicubic remapping'
      case default
        stop 'Invalid Map Type'
      end select

      call date_and_time(date=cdate)
      write (history,1000) cdate(5:6),cdate(7:8),cdate(1:4)
 1000 format('Created: ',a2,'-',a2,'-',a4)

C-----------------------------------------------------------------------
C
C     sort address and weight arrays
C
C-----------------------------------------------------------------------

      call sort_add(grid2_add_map1, grid1_add_map1, wts_map1)
      if (num_maps > 1) then
        call sort_add(grid1_add_map2, grid2_add_map2, wts_map2)
      endif

C-----------------------------------------------------------------------
C
C     call appropriate output routine
C
C-----------------------------------------------------------------------

      select case(output_opt)
      case ('scrip')
        call write_remap_scrip(map1_name, interp_file1, 1)
      case ('ncar-csm')
        call write_remap_csm  (map1_name, interp_file1, 1)
      case default
        stop 'unknown output file convention'
      end select

C-----------------------------------------------------------------------
C
C     call appropriate output routine for second mapping if required
C
C-----------------------------------------------------------------------

      if (num_maps > 1) then
        select case(output_opt)
        case ('scrip')
          call write_remap_scrip(map2_name, interp_file2, 2)
        case ('ncar-csm')
          call write_remap_csm  (map2_name, interp_file2, 2)
        case default
          stop 'unknown output file convention'
        end select
      endif

C-----------------------------------------------------------------------

      end subroutine write_remap

C***********************************************************************

      subroutine write_remap_scrip(map_name, interp_file, direction)

C-----------------------------------------------------------------------
C
C     writes remap data to a netCDF file using SCRIP conventions
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(in) ::
     &            map_name     ! name for mapping 
     &,           interp_file  ! filename for remap data

      integer (kind=int_kind), intent(in) ::
     &  direction              ! direction of map (1=grid1 to grid2
                               !                   2=grid2 to grid1)

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      character(char_len) ::
     &  grid1_ctmp        ! character temp for grid1 names
     &, grid2_ctmp        ! character temp for grid2 names

      integer (kind=int_kind) ::
     &  itmp1             ! integer temp
     &, itmp2             ! integer temp
     &, itmp3             ! integer temp
     &, itmp4             ! integer temp

C-----------------------------------------------------------------------
C
C     create netCDF file for mapping and define some global attributes
C
C-----------------------------------------------------------------------

      ncstat = nf_create (interp_file, NF_CLOBBER, nc_file_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** map name
      !***
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'title',
     &                          len_trim(map_name), map_name)
      call netcdf_error_handler(ncstat)

      !***
      !*** normalization option
      !***
      ncstat = nf_put_att_text(nc_file_id, NF_GLOBAL, 'normalization',
     &                         len_trim(normalize_opt), normalize_opt)
      call netcdf_error_handler(ncstat)

      !***
      !*** map method
      !***
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'map_method',
     &                          len_trim(map_method), map_method)
      call netcdf_error_handler(ncstat)

      !***
      !*** history
      !***
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'history',
     &                          len_trim(history), history)
      call netcdf_error_handler(ncstat)

      !***
      !*** file convention
      !***
      convention = 'SCRIP'
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'conventions',
     &                          len_trim(convention), convention)
      call netcdf_error_handler(ncstat)

      !***
      !*** source and destination grid names
      !***

      if (direction == 1) then
        grid1_ctmp = 'source_grid'
        grid2_ctmp = 'dest_grid'
      else
        grid1_ctmp = 'dest_grid'
        grid2_ctmp = 'source_grid'
      endif

      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, trim(grid1_ctmp),
     &                          len_trim(grid1_name), grid1_name)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, trim(grid2_ctmp),
     &                          len_trim(grid2_name), grid2_name)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     prepare netCDF dimension info
C
C-----------------------------------------------------------------------

      !***
      !*** define grid size dimensions
      !***

      if (direction == 1) then
        itmp1 = grid1_size
        itmp2 = grid2_size
      else
        itmp1 = grid2_size
        itmp2 = grid1_size
      endif

      ncstat = nf_def_dim (nc_file_id, 'src_grid_size', itmp1, 
     &                     nc_srcgrdsize_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'dst_grid_size', itmp2, 
     &                     nc_dstgrdsize_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner dimension
      !***

      if (direction == 1) then
        itmp1 = grid1_corners
        itmp2 = grid2_corners
      else
        itmp1 = grid2_corners
        itmp2 = grid1_corners
      endif

      ncstat = nf_def_dim (nc_file_id, 'src_grid_corners', 
     &                     itmp1, nc_srcgrdcorn_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'dst_grid_corners', 
     &                     itmp2, nc_dstgrdcorn_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid rank dimension
      !***

      if (direction == 1) then
        itmp1 = grid1_rank
        itmp2 = grid2_rank
      else
        itmp1 = grid2_rank
        itmp2 = grid1_rank
      endif

      ncstat = nf_def_dim (nc_file_id, 'src_grid_rank', 
     &                     itmp1, nc_srcgrdrank_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'dst_grid_rank', 
     &                     itmp2, nc_dstgrdrank_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define map size dimensions
      !***

      if (direction == 1) then
        itmp1 = num_links_map1
      else
        itmp1 = num_links_map2
      endif

      ncstat = nf_def_dim (nc_file_id, 'num_links', 
     &                     itmp1, nc_numlinks_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'num_wgts', 
     &                     num_wts, nc_numwgts_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid dimensions
      !***

      ncstat = nf_def_var (nc_file_id, 'src_grid_dims', NF_INT,
     &                     1, nc_srcgrdrank_id, nc_srcgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_dims', NF_INT,
     &                     1, nc_dstgrdrank_id, nc_dstgrddims_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     define all arrays for netCDF descriptors
C
C-----------------------------------------------------------------------

      !***
      !*** define grid center latitude array
      !***

      ncstat = nf_def_var (nc_file_id, 'src_grid_center_lat', 
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_center_lat', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid center longitude array
      !***

      ncstat = nf_def_var (nc_file_id, 'src_grid_center_lon', 
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_center_lon', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner lat/lon arrays
      !***

      nc_dims2_id(1) = nc_srcgrdcorn_id
      nc_dims2_id(2) = nc_srcgrdsize_id

      ncstat = nf_def_var (nc_file_id, 'src_grid_corner_lat', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_srcgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'src_grid_corner_lon', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_srcgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      nc_dims2_id(1) = nc_dstgrdcorn_id
      nc_dims2_id(2) = nc_dstgrdsize_id

      ncstat = nf_def_var (nc_file_id, 'dst_grid_corner_lat', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_dstgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_corner_lon', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_dstgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define units for all coordinate arrays
      !***

      if (direction == 1) then
        grid1_ctmp = grid1_units
        grid2_ctmp = grid2_units
      else
        grid1_ctmp = grid2_units
        grid2_ctmp = grid1_units
      endif

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcntrlat_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcntrlat_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcntrlon_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcntrlon_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcrnrlat_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcrnrlon_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcrnrlat_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcrnrlon_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid mask
      !***

      ncstat = nf_def_var (nc_file_id, 'src_grid_imask', NF_INT,
     &                     1, nc_srcgrdsize_id, nc_srcgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdimask_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_imask', NF_INT,
     &                     1, nc_dstgrdsize_id, nc_dstgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdimask_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid area arrays
      !***

      ncstat = nf_def_var (nc_file_id, 'src_grid_area', 
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdarea_id, 
     &                          'units', 14, 'square radians')
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_area', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdarea_id, 
     &                          'units', 14, 'square radians')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid fraction arrays
      !***

      ncstat = nf_def_var (nc_file_id, 'src_grid_frac', 
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdfrac_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_frac', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdfrac_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      !***
      !*** define mapping arrays
      !***

      ncstat = nf_def_var (nc_file_id, 'src_address', 
     &                     NF_INT, 1, nc_numlinks_id, 
     &                     nc_srcadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_address', 
     &                     NF_INT, 1, nc_numlinks_id, 
     &                     nc_dstadd_id)
      call netcdf_error_handler(ncstat)

      nc_dims2_id(1) = nc_numwgts_id
      nc_dims2_id(2) = nc_numlinks_id

      ncstat = nf_def_var (nc_file_id, 'remap_matrix', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_rmpmatrix_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** end definition stage
      !***

      ncstat = nf_enddef(nc_file_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     compute integer masks
C
C-----------------------------------------------------------------------

      if (direction == 1) THEN
          IF(.not.allocated(src_mask_int))
     $  allocate (src_mask_int(grid1_size))
          IF(.not.allocated(dst_mask_int))
     &  allocate (dst_mask_int(grid2_size))

        where (grid2_mask)
          dst_mask_int = 1
        elsewhere
          dst_mask_int = 0
        endwhere

        where (grid1_mask)
          src_mask_int = 1
        elsewhere
          src_mask_int = 0
        endwhere
      else
        IF(.not.allocated(src_mask_int))
     $  allocate (src_mask_int(grid2_size))
        IF(.not.allocated(dst_mask_int))
     &  allocate (dst_mask_int(grid1_size))

        where (grid1_mask)
          dst_mask_int = 1
        elsewhere
          dst_mask_int = 0
        endwhere

        where (grid2_mask)
          src_mask_int = 1
        elsewhere
          src_mask_int = 0
        endwhere
      endif

C-----------------------------------------------------------------------
C
C     change units of lat/lon coordinates if input units different
C     from radians
C
C-----------------------------------------------------------------------

      if (grid1_units(1:7) == 'degrees' .and. direction == 1) then
        grid1_center_lat = grid1_center_lat/deg2rad
        grid1_center_lon = grid1_center_lon/deg2rad
        grid1_corner_lat = grid1_corner_lat/deg2rad
        grid1_corner_lon = grid1_corner_lon/deg2rad
      endif

      if (grid2_units(1:7) == 'degrees' .and. direction == 1) then
        grid2_center_lat = grid2_center_lat/deg2rad
        grid2_center_lon = grid2_center_lon/deg2rad
        grid2_corner_lat = grid2_corner_lat/deg2rad
        grid2_corner_lon = grid2_corner_lon/deg2rad
      endif

C-----------------------------------------------------------------------
C
C     write mapping data
C
C-----------------------------------------------------------------------

      if (direction == 1) then
        itmp1 = nc_srcgrddims_id
        itmp2 = nc_dstgrddims_id
      else
        itmp2 = nc_srcgrddims_id
        itmp1 = nc_dstgrddims_id
      endif

      ncstat = nf_put_var_int(nc_file_id, itmp1, grid1_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_file_id, itmp2, grid2_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_file_id, nc_srcgrdimask_id, 
     &                        src_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_file_id, nc_dstgrdimask_id,
     &                        dst_mask_int)
      call netcdf_error_handler(ncstat)

      deallocate(src_mask_int, dst_mask_int)

      if (direction == 1) then
        itmp1 = nc_srcgrdcntrlat_id
        itmp2 = nc_srcgrdcntrlon_id
        itmp3 = nc_srcgrdcrnrlat_id
        itmp4 = nc_srcgrdcrnrlon_id
      else
        itmp1 = nc_dstgrdcntrlat_id
        itmp2 = nc_dstgrdcntrlon_id
        itmp3 = nc_dstgrdcrnrlat_id
        itmp4 = nc_dstgrdcrnrlon_id
      endif

      ncstat = nf_put_var_double(nc_file_id, itmp1, grid1_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp2, grid1_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp3, grid1_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp4, grid1_corner_lon)
      call netcdf_error_handler(ncstat)

      if (direction == 1) then
        itmp1 = nc_dstgrdcntrlat_id
        itmp2 = nc_dstgrdcntrlon_id
        itmp3 = nc_dstgrdcrnrlat_id
        itmp4 = nc_dstgrdcrnrlon_id
      else
        itmp1 = nc_srcgrdcntrlat_id
        itmp2 = nc_srcgrdcntrlon_id
        itmp3 = nc_srcgrdcrnrlat_id
        itmp4 = nc_srcgrdcrnrlon_id
      endif

      ncstat = nf_put_var_double(nc_file_id, itmp1, grid2_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp2, grid2_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp3, grid2_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp4, grid2_corner_lon)
      call netcdf_error_handler(ncstat)

      if (direction == 1) then
        itmp1 = nc_srcgrdarea_id
        itmp2 = nc_srcgrdfrac_id
        itmp3 = nc_dstgrdarea_id
        itmp4 = nc_dstgrdfrac_id
      else
        itmp1 = nc_dstgrdarea_id
        itmp2 = nc_dstgrdfrac_id
        itmp3 = nc_srcgrdarea_id
        itmp4 = nc_srcgrdfrac_id
      endif

      if (luse_grid1_area) then
        ncstat = nf_put_var_double(nc_file_id, itmp1, grid1_area_in)
      else
        ncstat = nf_put_var_double(nc_file_id, itmp1, grid1_area)
      endif
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp2, grid1_frac)
      call netcdf_error_handler(ncstat)

      if (luse_grid2_area) then
        ncstat = nf_put_var_double(nc_file_id, itmp3, grid2_area_in)
      else
        ncstat = nf_put_var_double(nc_file_id, itmp3, grid2_area)
      endif
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp4, grid2_frac)
      call netcdf_error_handler(ncstat)

      if (direction == 1) then
        ncstat = nf_put_var_int(nc_file_id, nc_srcadd_id, 
     &                          grid1_add_map1)
        call netcdf_error_handler(ncstat)

        ncstat = nf_put_var_int(nc_file_id, nc_dstadd_id, 
     &                          grid2_add_map1)
        call netcdf_error_handler(ncstat)

        ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                             wts_map1)
        call netcdf_error_handler(ncstat)
      else
        ncstat = nf_put_var_int(nc_file_id, nc_srcadd_id, 
     &                          grid2_add_map2)
        call netcdf_error_handler(ncstat)

        ncstat = nf_put_var_int(nc_file_id, nc_dstadd_id, 
     &                          grid1_add_map2)
        call netcdf_error_handler(ncstat)

        ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                             wts_map2)
        call netcdf_error_handler(ncstat)
      endif

      ncstat = nf_close(nc_file_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------

      end subroutine write_remap_scrip

C***********************************************************************

      subroutine write_remap_csm(map_name, interp_file, direction)

C-----------------------------------------------------------------------
C
C     writes remap data to a netCDF file using NCAR-CSM conventions
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      character(char_len), intent(in) ::
     &            map_name     ! name for mapping 
     &,           interp_file  ! filename for remap data

      integer (kind=int_kind), intent(in) ::
     &  direction              ! direction of map (1=grid1 to grid2
                               !                   2=grid2 to grid1)

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      character(char_len) ::
     &  grid1_ctmp        ! character temp for grid1 names
     &, grid2_ctmp        ! character temp for grid2 names

      integer (kind=int_kind) ::
     &  itmp1             ! integer temp
     &, itmp2             ! integer temp
     &, itmp3             ! integer temp
     &, itmp4             ! integer temp
     &, nc_numwgts1_id    ! extra netCDF id for additional weights
     &, nc_src_isize_id   ! extra netCDF id for ni_a
     &, nc_src_jsize_id   ! extra netCDF id for nj_a
     &, nc_dst_isize_id   ! extra netCDF id for ni_b
     &, nc_dst_jsize_id   ! extra netCDF id for nj_b
     &, nc_rmpmatrix2_id  ! extra netCDF id for high-order remap matrix

      real (kind=dbl_kind), dimension(:),allocatable ::
     &  wts1              ! CSM wants single array for 1st-order wts

      real (kind=dbl_kind), dimension(:,:),allocatable ::
     &  wts2              ! write remaining weights in different array

C-----------------------------------------------------------------------
C
C     create netCDF file for mapping and define some global attributes
C
C-----------------------------------------------------------------------

      ncstat = nf_create (interp_file, NF_CLOBBER, nc_file_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** map name
      !***
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'title',
     &                          len_trim(map_name), map_name)
      call netcdf_error_handler(ncstat)

      !***
      !*** normalization option
      !***
      ncstat = nf_put_att_text(nc_file_id, NF_GLOBAL, 'normalization',
     &                         len_trim(normalize_opt), normalize_opt)
      call netcdf_error_handler(ncstat)

      !***
      !*** map method
      !***
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'map_method',
     &                          len_trim(map_method), map_method)
      call netcdf_error_handler(ncstat)

      !***
      !*** history
      !***
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'history',
     &                          len_trim(history), history)
      call netcdf_error_handler(ncstat)

      !***
      !*** file convention
      !***
      convention = 'NCAR-CSM'
      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, 'conventions',
     &                          len_trim(convention), convention)
      call netcdf_error_handler(ncstat)

      !***
      !*** source and destination grid names
      !***

      if (direction == 1) then
        grid1_ctmp = 'domain_a'
        grid2_ctmp = 'domain_b'
      else
        grid1_ctmp = 'domain_b'
        grid2_ctmp = 'domain_a'
      endif

      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, trim(grid1_ctmp),
     &                          len_trim(grid1_name), grid1_name)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, NF_GLOBAL, trim(grid2_ctmp),
     &                          len_trim(grid2_name), grid2_name)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     prepare netCDF dimension info
C
C-----------------------------------------------------------------------

      !***
      !*** define grid size dimensions
      !***

      if (direction == 1) then
        itmp1 = grid1_size
        itmp2 = grid2_size
      else
        itmp1 = grid2_size
        itmp2 = grid1_size
      endif

      ncstat = nf_def_dim (nc_file_id, 'n_a', itmp1, nc_srcgrdsize_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'n_b', itmp2, nc_dstgrdsize_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner dimension
      !***

      if (direction == 1) then
        itmp1 = grid1_corners
        itmp2 = grid2_corners
      else
        itmp1 = grid2_corners
        itmp2 = grid1_corners
      endif

      ncstat = nf_def_dim (nc_file_id, 'nv_a', itmp1, nc_srcgrdcorn_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'nv_b', itmp2, nc_dstgrdcorn_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid rank dimension
      !***

      if (direction == 1) then
        itmp1 = grid1_rank
        itmp2 = grid2_rank
      else
        itmp1 = grid2_rank
        itmp2 = grid1_rank
      endif

      ncstat = nf_def_dim (nc_file_id, 'src_grid_rank', 
     &                     itmp1, nc_srcgrdrank_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'dst_grid_rank', 
     &                     itmp2, nc_dstgrdrank_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define first two dims as if 2-d cartesian domain
      !***

      if (direction == 1) then
        itmp1 = grid1_dims(1)
        if (grid1_rank > 1) then
          itmp2 = grid1_dims(2)
        else
          itmp2 = 0
        endif
        itmp3 = grid2_dims(1)
        if (grid2_rank > 1) then
          itmp4 = grid2_dims(2)
        else
          itmp4 = 0
        endif
      else
        itmp1 = grid2_dims(1)
        if (grid2_rank > 1) then
          itmp2 = grid2_dims(2)
        else
          itmp2 = 0
        endif
        itmp3 = grid1_dims(1)
        if (grid1_rank > 1) then
          itmp4 = grid1_dims(2)
        else
          itmp4 = 0
        endif
      endif

      ncstat = nf_def_dim (nc_file_id, 'ni_a', itmp1, nc_src_isize_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'nj_a', itmp2, nc_src_jsize_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'ni_b', itmp3, nc_dst_isize_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'nj_b', itmp4, nc_dst_jsize_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define map size dimensions
      !***

      if (direction == 1) then
        itmp1 = num_links_map1
      else
        itmp1 = num_links_map2
      endif

      ncstat = nf_def_dim (nc_file_id, 'n_s', itmp1, nc_numlinks_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_dim (nc_file_id, 'num_wgts', 
     &                     num_wts, nc_numwgts_id)
      call netcdf_error_handler(ncstat)

      if (num_wts > 1) then
        ncstat = nf_def_dim (nc_file_id, 'num_wgts1', 
     &                       num_wts-1, nc_numwgts1_id)
        call netcdf_error_handler(ncstat)
      endif

      !***
      !*** define grid dimensions
      !***

      ncstat = nf_def_var (nc_file_id, 'src_grid_dims', NF_INT,
     &                     1, nc_srcgrdrank_id, nc_srcgrddims_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'dst_grid_dims', NF_INT,
     &                     1, nc_dstgrdrank_id, nc_dstgrddims_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     define all arrays for netCDF descriptors
C
C-----------------------------------------------------------------------

      !***
      !*** define grid center latitude array
      !***

      ncstat = nf_def_var (nc_file_id, 'yc_a',
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'yc_b', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdcntrlat_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid center longitude array
      !***

      ncstat = nf_def_var (nc_file_id, 'xc_a', 
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'xc_b', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdcntrlon_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid corner lat/lon arrays
      !***

      nc_dims2_id(1) = nc_srcgrdcorn_id
      nc_dims2_id(2) = nc_srcgrdsize_id

      ncstat = nf_def_var (nc_file_id, 'yv_a', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_srcgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'xv_a', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_srcgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      nc_dims2_id(1) = nc_dstgrdcorn_id
      nc_dims2_id(2) = nc_dstgrdsize_id

      ncstat = nf_def_var (nc_file_id, 'yv_b', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_dstgrdcrnrlat_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'xv_b', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_dstgrdcrnrlon_id)
      call netcdf_error_handler(ncstat)

      !***
      !*** CSM wants all in degrees
      !***

      grid1_units = 'degrees'
      grid2_units = 'degrees'

      if (direction == 1) then
        grid1_ctmp = grid1_units
        grid2_ctmp = grid2_units
      else
        grid1_ctmp = grid2_units
        grid2_ctmp = grid1_units
      endif

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcntrlat_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcntrlat_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcntrlon_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcntrlon_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcrnrlat_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdcrnrlon_id, 
     &                          'units', 7, grid1_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcrnrlat_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdcrnrlon_id, 
     &                          'units', 7, grid2_ctmp)
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid mask
      !***

      ncstat = nf_def_var (nc_file_id, 'mask_a', NF_INT,
     &                     1, nc_srcgrdsize_id, nc_srcgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdimask_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'mask_b', NF_INT,
     &                     1, nc_dstgrdsize_id, nc_dstgrdimask_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdimask_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid area arrays
      !***

      ncstat = nf_def_var (nc_file_id, 'area_a', 
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdarea_id, 
     &                          'units', 14, 'square radians')
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'area_b', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdarea_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdarea_id, 
     &                          'units', 14, 'square radians')
      call netcdf_error_handler(ncstat)

      !***
      !*** define grid fraction arrays
      !***

      ncstat = nf_def_var (nc_file_id, 'frac_a', 
     &                     NF_DOUBLE, 1, nc_srcgrdsize_id, 
     &                     nc_srcgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_srcgrdfrac_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'frac_b', 
     &                     NF_DOUBLE, 1, nc_dstgrdsize_id, 
     &                     nc_dstgrdfrac_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_att_text (nc_file_id, nc_dstgrdfrac_id, 
     &                          'units', 8, 'unitless')
      call netcdf_error_handler(ncstat)

      !***
      !*** define mapping arrays
      !***

      ncstat = nf_def_var (nc_file_id, 'col', 
     &                     NF_INT, 1, nc_numlinks_id, 
     &                     nc_srcadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'row', 
     &                     NF_INT, 1, nc_numlinks_id, 
     &                     nc_dstadd_id)
      call netcdf_error_handler(ncstat)

      ncstat = nf_def_var (nc_file_id, 'S', 
     &                     NF_DOUBLE, 1, nc_numlinks_id, 
     &                     nc_rmpmatrix_id)
      call netcdf_error_handler(ncstat)

      if (num_wts > 1) then
        nc_dims2_id(1) = nc_numwgts1_id
        nc_dims2_id(2) = nc_numlinks_id

        ncstat = nf_def_var (nc_file_id, 'S2', 
     &                     NF_DOUBLE, 2, nc_dims2_id, 
     &                     nc_rmpmatrix2_id)
        call netcdf_error_handler(ncstat)
      endif

      !***
      !*** end definition stage
      !***

      ncstat = nf_enddef(nc_file_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------
C
C     compute integer masks
C
C-----------------------------------------------------------------------

      if (direction == 1) then
          IF(.not.allocated(src_mask_int))
     $  allocate (src_mask_int(grid1_size))
          IF(.not.allocated(dst_mask_int))
     &  allocate (dst_mask_int(grid2_size))

        where (grid2_mask)
          dst_mask_int = 1
        elsewhere
          dst_mask_int = 0
        endwhere

        where (grid1_mask)
          src_mask_int = 1
        elsewhere
          src_mask_int = 0
        endwhere
      else
          IF(.not.allocated(src_mask_int))
     $  allocate (src_mask_int(grid2_size))
          IF(.not.allocated(dst_mask_int))
     &  allocate (dst_mask_int(grid1_size))

        where (grid1_mask)
          dst_mask_int = 1
        elsewhere
          dst_mask_int = 0
        endwhere

        where (grid2_mask)
          src_mask_int = 1
        elsewhere
          src_mask_int = 0
        endwhere
      endif

C-----------------------------------------------------------------------
C
C     change units of lat/lon coordinates if input units different
C     from radians. if this is the second mapping, the conversion has
C     alread been done.
C
C-----------------------------------------------------------------------

      if (grid1_units(1:7) == 'degrees' .and. direction == 1) then
        grid1_center_lat = grid1_center_lat/deg2rad
        grid1_center_lon = grid1_center_lon/deg2rad
        grid1_corner_lat = grid1_corner_lat/deg2rad
        grid1_corner_lon = grid1_corner_lon/deg2rad
      endif

      if (grid2_units(1:7) == 'degrees' .and. direction == 1) then
        grid2_center_lat = grid2_center_lat/deg2rad
        grid2_center_lon = grid2_center_lon/deg2rad
        grid2_corner_lat = grid2_corner_lat/deg2rad
        grid2_corner_lon = grid2_corner_lon/deg2rad
      endif

C-----------------------------------------------------------------------
C
C     write mapping data
C
C-----------------------------------------------------------------------

      if (direction == 1) then
        itmp1 = nc_srcgrddims_id
        itmp2 = nc_dstgrddims_id
      else
        itmp2 = nc_srcgrddims_id
        itmp1 = nc_dstgrddims_id
      endif

      ncstat = nf_put_var_int(nc_file_id, itmp1, grid1_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_file_id, itmp2, grid2_dims)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_file_id, nc_srcgrdimask_id, 
     &                        src_mask_int)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_int(nc_file_id, nc_dstgrdimask_id,
     &                        dst_mask_int)
      call netcdf_error_handler(ncstat)

      deallocate(src_mask_int, dst_mask_int)

      if (direction == 1) then
        itmp1 = nc_srcgrdcntrlat_id
        itmp2 = nc_srcgrdcntrlon_id
        itmp3 = nc_srcgrdcrnrlat_id
        itmp4 = nc_srcgrdcrnrlon_id
      else
        itmp1 = nc_dstgrdcntrlat_id
        itmp2 = nc_dstgrdcntrlon_id
        itmp3 = nc_dstgrdcrnrlat_id
        itmp4 = nc_dstgrdcrnrlon_id
      endif

      ncstat = nf_put_var_double(nc_file_id, itmp1, grid1_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp2, grid1_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp3, grid1_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp4, grid1_corner_lon)
      call netcdf_error_handler(ncstat)

      if (direction == 1) then
        itmp1 = nc_dstgrdcntrlat_id
        itmp2 = nc_dstgrdcntrlon_id
        itmp3 = nc_dstgrdcrnrlat_id
        itmp4 = nc_dstgrdcrnrlon_id
      else
        itmp1 = nc_srcgrdcntrlat_id
        itmp2 = nc_srcgrdcntrlon_id
        itmp3 = nc_srcgrdcrnrlat_id
        itmp4 = nc_srcgrdcrnrlon_id
      endif

      ncstat = nf_put_var_double(nc_file_id, itmp1, grid2_center_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp2, grid2_center_lon)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp3, grid2_corner_lat)
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp4, grid2_corner_lon)
      call netcdf_error_handler(ncstat)

      if (direction == 1) then
        itmp1 = nc_srcgrdarea_id
        itmp2 = nc_srcgrdfrac_id
        itmp3 = nc_dstgrdarea_id
        itmp4 = nc_dstgrdfrac_id
      else
        itmp1 = nc_dstgrdarea_id
        itmp2 = nc_dstgrdfrac_id
        itmp3 = nc_srcgrdarea_id
        itmp4 = nc_srcgrdfrac_id
      endif

      if (luse_grid1_area) then
        ncstat = nf_put_var_double(nc_file_id, itmp1, grid1_area_in)
      else
        ncstat = nf_put_var_double(nc_file_id, itmp1, grid1_area)
      endif
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp2, grid1_frac)
      call netcdf_error_handler(ncstat)

      if (luse_grid2_area) then
        ncstat = nf_put_var_double(nc_file_id, itmp3, grid2_area)
      else
        ncstat = nf_put_var_double(nc_file_id, itmp3, grid2_area)
      endif
      call netcdf_error_handler(ncstat)

      ncstat = nf_put_var_double(nc_file_id, itmp4, grid2_frac)
      call netcdf_error_handler(ncstat)

      if (direction == 1) then
        ncstat = nf_put_var_int(nc_file_id, nc_srcadd_id, 
     &                          grid1_add_map1)
        call netcdf_error_handler(ncstat)

        ncstat = nf_put_var_int(nc_file_id, nc_dstadd_id, 
     &                          grid2_add_map1)
        call netcdf_error_handler(ncstat)

        if (num_wts == 1) then
          ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                               wts_map1)
          call netcdf_error_handler(ncstat)
        else
          allocate(wts1(num_links_map1),wts2(num_wts-1,num_links_map1))

          wts1 = wts_map1(1,:)
          wts2 = wts_map1(2:,:)

          ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                               wts1)
          call netcdf_error_handler(ncstat)
          ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix2_id, 
     &                               wts2)
          call netcdf_error_handler(ncstat)
          deallocate(wts1,wts2)
        endif
      else
        ncstat = nf_put_var_int(nc_file_id, nc_srcadd_id, 
     &                          grid2_add_map2)
        call netcdf_error_handler(ncstat)

        ncstat = nf_put_var_int(nc_file_id, nc_dstadd_id, 
     &                          grid1_add_map2)
        call netcdf_error_handler(ncstat)

        if (num_wts == 1) then
          ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                               wts_map2)
          call netcdf_error_handler(ncstat)
        else
          allocate(wts1(num_links_map2),wts2(num_wts-1,num_links_map2))

          wts1 = wts_map2(1,:)
          wts2 = wts_map2(2:,:)

          ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix_id, 
     &                               wts1)
          call netcdf_error_handler(ncstat)
          ncstat = nf_put_var_double(nc_file_id, nc_rmpmatrix2_id, 
     &                               wts2)
          call netcdf_error_handler(ncstat)
          deallocate(wts1,wts2)
        endif
      endif

      ncstat = nf_close(nc_file_id)
      call netcdf_error_handler(ncstat)

C-----------------------------------------------------------------------

      end subroutine write_remap_csm

C***********************************************************************

      subroutine sort_add(add1, add2, weights)

C-----------------------------------------------------------------------
C
C     this routine sorts address and weight arrays based on the
C     destination address with the source address as a secondary
C     sorting criterion.  the method is a standard heap sort.
C
C-----------------------------------------------------------------------

      use kinds_mod     ! defines common data types
      use constants     ! defines common scalar constants

      implicit none

C-----------------------------------------------------------------------
C
C     Input and Output arrays
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(inout), dimension(:) ::
     &        add1,       ! destination address array (num_links)
     &        add2        ! source      address array

      real (kind=dbl_kind), intent(inout), dimension(:,:) ::
     &        weights     ! remapping weights (num_wts, num_links)

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) ::
     &          num_links,          ! num of links for this mapping
     &          num_wts,            ! num of weights for this mapping
     &          add1_tmp, add2_tmp, ! temp for addresses during swap
     &          nwgt,
     &          lvl, final_lvl,     ! level indexes for heap sort levels
     &          chk_lvl1, chk_lvl2, max_lvl

      real (kind=dbl_kind), dimension(SIZE(weights,DIM=1)) ::
     &          wgttmp              ! temp for holding wts during swap

C-----------------------------------------------------------------------
C
C     determine total number of links to sort and number of weights
C
C-----------------------------------------------------------------------

      num_links = SIZE(add1)
      num_wts   = SIZE(weights, DIM=1)

C-----------------------------------------------------------------------
C
C     start at the lowest level (N/2) of the tree and sift lower 
C     values to the bottom of the tree, promoting the larger numbers
C
C-----------------------------------------------------------------------

      do lvl=num_links/2,1,-1

        final_lvl = lvl
        add1_tmp = add1(lvl)
        add2_tmp = add2(lvl)
        wgttmp(:) = weights(:,lvl)

        !***
        !*** loop until proper level is found for this link, or reach
        !*** bottom
        !***

        sift_loop1: do

          !***
          !*** find the largest of the two daughters
          !***

          chk_lvl1 = 2*final_lvl
          chk_lvl2 = 2*final_lvl+1
          if (chk_lvl1 .EQ. num_links) chk_lvl2 = chk_lvl1

          if ((add1(chk_lvl1) >  add1(chk_lvl2)) .OR.
     &       ((add1(chk_lvl1) == add1(chk_lvl2)) .AND.
     &        (add2(chk_lvl1) >  add2(chk_lvl2)))) then
            max_lvl = chk_lvl1
          else 
            max_lvl = chk_lvl2
          endif

          !***
          !*** if the parent is greater than both daughters,
          !*** the correct level has been found
          !***

          if ((add1_tmp .GT. add1(max_lvl)) .OR.
     &       ((add1_tmp .EQ. add1(max_lvl)) .AND.
     &        (add2_tmp .GT. add2(max_lvl)))) then
            add1(final_lvl) = add1_tmp
            add2(final_lvl) = add2_tmp
            weights(:,final_lvl) = wgttmp(:)
            exit sift_loop1

          !***
          !*** otherwise, promote the largest daughter and push
          !*** down one level in the tree.  if haven't reached
          !*** the end of the tree, repeat the process.  otherwise
          !*** store last values and exit the loop
          !***

          else 
            add1(final_lvl) = add1(max_lvl)
            add2(final_lvl) = add2(max_lvl)
            weights(:,final_lvl) = weights(:,max_lvl)

            final_lvl = max_lvl
            if (2*final_lvl > num_links) then
              add1(final_lvl) = add1_tmp
              add2(final_lvl) = add2_tmp
              weights(:,final_lvl) = wgttmp(:)
              exit sift_loop1
            endif
          endif
        end do sift_loop1
      end do

C-----------------------------------------------------------------------
C
C     now that the heap has been sorted, strip off the top (largest)
C     value and promote the values below
C
C-----------------------------------------------------------------------

      do lvl=num_links,3,-1

        !***
        !*** move the top value and insert it into the correct place
        !***

        add1_tmp = add1(lvl)
        add1(lvl) = add1(1)

        add2_tmp = add2(lvl)
        add2(lvl) = add2(1)

        wgttmp(:) = weights(:,lvl)
        weights(:,lvl) = weights(:,1)

        !***
        !*** as above this loop sifts the tmp values down until proper 
        !*** level is reached
        !***

        final_lvl = 1

        sift_loop2: do

          !***
          !*** find the largest of the two daughters
          !***

          chk_lvl1 = 2*final_lvl
          chk_lvl2 = 2*final_lvl+1
          if (chk_lvl2 >= lvl) chk_lvl2 = chk_lvl1

          if ((add1(chk_lvl1) >  add1(chk_lvl2)) .OR.
     &       ((add1(chk_lvl1) == add1(chk_lvl2)) .AND.
     &        (add2(chk_lvl1) >  add2(chk_lvl2)))) then
            max_lvl = chk_lvl1
          else 
            max_lvl = chk_lvl2
          endif

          !***
          !*** if the parent is greater than both daughters,
          !*** the correct level has been found
          !***

          if ((add1_tmp >  add1(max_lvl)) .OR.
     &       ((add1_tmp == add1(max_lvl)) .AND.
     &        (add2_tmp >  add2(max_lvl)))) then
            add1(final_lvl) = add1_tmp
            add2(final_lvl) = add2_tmp
            weights(:,final_lvl) = wgttmp(:)
            exit sift_loop2

          !***
          !*** otherwise, promote the largest daughter and push
          !*** down one level in the tree.  if haven't reached
          !*** the end of the tree, repeat the process.  otherwise
          !*** store last values and exit the loop
          !***

          else 
            add1(final_lvl) = add1(max_lvl)
            add2(final_lvl) = add2(max_lvl)
            weights(:,final_lvl) = weights(:,max_lvl)

            final_lvl = max_lvl
            if (2*final_lvl >= lvl) then
              add1(final_lvl) = add1_tmp
              add2(final_lvl) = add2_tmp
              weights(:,final_lvl) = wgttmp(:)
              exit sift_loop2
            endif
          endif
        end do sift_loop2
      end do

      !***
      !*** swap the last two entries
      !***


      add1_tmp = add1(2)
      add1(2)  = add1(1)
      add1(1)  = add1_tmp

      add2_tmp = add2(2)
      add2(2)  = add2(1)
      add2(1)  = add2_tmp

      wgttmp (:)   = weights(:,2)
      weights(:,2) = weights(:,1)
      weights(:,1) = wgttmp (:)

C-----------------------------------------------------------------------

      end subroutine sort_add

C***********************************************************************

      end module remap_write

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     this module contains necessary routines for performing an 
C     bilinear interpolation.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: remap_bilinear.f,v 1.6 2001/08/22 18:20:40 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      module remap_bilinear

C-----------------------------------------------------------------------

      use kinds_mod     ! defines common data types
      use constants     ! defines common constants
      use grids         ! module containing grid info
      use remap_vars    ! module containing remap info

      implicit none

C-----------------------------------------------------------------------

      integer (kind=int_kind), parameter ::
     &    max_iter = 100   ! max iteration count for i,j iteration

      real (kind=dbl_kind), parameter ::
     &     converge = 1.e-10_dbl_kind  ! convergence criterion

C***********************************************************************

      contains

C***********************************************************************

      subroutine remap_bilin

C-----------------------------------------------------------------------
C
C     this routine computes the weights for a bilinear interpolation.
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: n,icount,
     &     dst_add,        ! destination address
     &     iter,           ! iteration counter
     &     nmap            ! index of current map being computed

      integer (kind=int_kind), dimension(4) :: 
     &     src_add         ! address for the four source points

      real (kind=dbl_kind), dimension(4)  ::
     &     src_lats,       ! latitudes  of four bilinear corners
     &     src_lons,       ! longitudes of four bilinear corners
     &     wgts            ! bilinear weights for four corners

      real (kind=dbl_kind) ::
     &     plat, plon,       ! lat/lon coords of destination point
     &     iguess, jguess,   ! current guess for bilinear coordinate
     &     thguess, phguess, ! current guess for lat/lon coordinate
     &     deli, delj,       ! corrections to i,j
     &     dth1, dth2, dth3, ! some latitude  differences
     &     dph1, dph2, dph3, ! some longitude differences
     &     dthp, dphp,       ! difference between point and sw corner
     &     mat1, mat2, mat3, mat4, ! matrix elements
     &     determinant,      ! matrix determinant
     &     sum_wgts          ! sum of weights for normalization

C-----------------------------------------------------------------------
C
C     compute mappings from grid1 to grid2
C
C-----------------------------------------------------------------------

      nmap = 1
      if (grid1_rank /= 2) then
        stop 'Can not do bilinear interpolation when grid_rank /= 2'
      endif

      !***
      !*** loop over destination grid 
      !***

      grid_loop1: do dst_add = 1, grid2_size

        if (.not. grid2_mask(dst_add)) cycle grid_loop1

        plat = grid2_center_lat(dst_add)
        plon = grid2_center_lon(dst_add)

        !***
        !*** find nearest square of grid points on source grid
        !***

        call grid_search_bilin(src_add, src_lats, src_lons, 
     &                         plat, plon, grid1_dims,
     &                         grid1_center_lat, grid1_center_lon,
     &                         grid1_bound_box, bin_addr1, bin_addr2)

        !***
        !*** check to see if points are land points
        !***

        if (src_add(1) > 0) then
          do n=1,4
            if (.not. grid1_mask(src_add(n))) src_add(1) = 0
          end do
        endif

        !***
        !*** if point found, find local i,j coordinates for weights
        !***

        if (src_add(1) > 0) then

          grid2_frac(dst_add) = one

          !***
          !*** iterate to find i,j for bilinear approximation
          !***

          dth1 = src_lats(2) - src_lats(1)
          dth2 = src_lats(4) - src_lats(1)
          dth3 = src_lats(3) - src_lats(2) - dth2

          dph1 = src_lons(2) - src_lons(1)
          dph2 = src_lons(4) - src_lons(1)
          dph3 = src_lons(3) - src_lons(2)

          if (dph1 >  three*pih) dph1 = dph1 - pi2
          if (dph2 >  three*pih) dph2 = dph2 - pi2
          if (dph3 >  three*pih) dph3 = dph3 - pi2
          if (dph1 < -three*pih) dph1 = dph1 + pi2
          if (dph2 < -three*pih) dph2 = dph2 + pi2
          if (dph3 < -three*pih) dph3 = dph3 + pi2

          dph3 = dph3 - dph2

          iguess = half
          jguess = half

          iter_loop1: do iter=1,max_iter

            dthp = plat - src_lats(1) - dth1*iguess -
     &                    dth2*jguess - dth3*iguess*jguess
            dphp = plon - src_lons(1)

            if (dphp >  three*pih) dphp = dphp - pi2
            if (dphp < -three*pih) dphp = dphp + pi2

            dphp = dphp - dph1*iguess - dph2*jguess - 
     &                    dph3*iguess*jguess

            mat1 = dth1 + dth3*jguess
            mat2 = dth2 + dth3*iguess
            mat3 = dph1 + dph3*jguess
            mat4 = dph2 + dph3*iguess

            determinant = mat1*mat4 - mat2*mat3

            deli = (dthp*mat4 - mat2*dphp)/determinant
            delj = (mat1*dphp - dthp*mat3)/determinant

            if (abs(deli) < converge .and. 
     &          abs(delj) < converge) exit iter_loop1

            iguess = iguess + deli
            jguess = jguess + delj

          end do iter_loop1

          if (iter <= max_iter) then

            !***
            !*** successfully found i,j - compute weights
            !***

            wgts(1) = (one-iguess)*(one-jguess)
            wgts(2) = iguess*(one-jguess)
            wgts(3) = iguess*jguess
            wgts(4) = (one-iguess)*jguess

            call store_link_bilin(dst_add, src_add, wgts, nmap)

          else
            print *,'Point coords: ',plat,plon
            print *,'Dest grid lats: ',src_lats
            print *,'Dest grid lons: ',src_lons
            print *,'Dest grid addresses: ',src_add
            print *,'Current i,j : ',iguess, jguess
            stop 'Iteration for i,j exceed max iteration count'
          endif

        !***
        !*** search for bilinear failed - use a distance-weighted
        !*** average instead (this is typically near the pole)
        !***

        else if (src_add(1) < 0) then

          src_add = abs(src_add)
          icount = 0
          do n=1,4
            if (grid1_mask(src_add(n))) then
              icount = icount + 1
            else
              src_lats(n) = zero
            endif
          end do

          if (icount > 0) then
            !*** renormalize weights

            sum_wgts = sum(src_lats)
            wgts(1) = src_lats(1)/sum_wgts
            wgts(2) = src_lats(2)/sum_wgts
            wgts(3) = src_lats(3)/sum_wgts
            wgts(4) = src_lats(4)/sum_wgts

            grid2_frac(dst_add) = one
            call store_link_bilin(dst_add, src_add, wgts, nmap)
          endif

        endif
      end do grid_loop1

C-----------------------------------------------------------------------
C
C     compute mappings from grid2 to grid1 if necessary
C
C-----------------------------------------------------------------------

      if (num_maps > 1) then

      nmap = 2
      if (grid2_rank /= 2) then
        stop 'Can not do bilinear interpolation when grid_rank /= 2'
      endif

      !***
      !*** loop over destination grid 
      !***

      grid_loop2: do dst_add = 1, grid1_size

        if (.not. grid1_mask(dst_add)) cycle grid_loop2

        plat = grid1_center_lat(dst_add)
        plon = grid1_center_lon(dst_add)

        !***
        !*** find nearest square of grid points on source grid
        !***

        call grid_search_bilin(src_add, src_lats, src_lons, 
     &                         plat, plon, grid2_dims,
     &                         grid2_center_lat, grid2_center_lon,
     &                         grid2_bound_box, bin_addr2, bin_addr1)

        !***
        !*** check to see if points are land points
        !***

        if (src_add(1) > 0) then
          do n=1,4
            if (.not. grid2_mask(src_add(n))) src_add(1) = 0
          end do
        endif

        !***
        !*** if point found, find i,j coordinates for weights
        !***

        if (src_add(1) > 0) then

          grid1_frac(dst_add) = one

          !***
          !*** iterate to find i,j for bilinear approximation
          !***

          dth1 = src_lats(2) - src_lats(1)
          dth2 = src_lats(4) - src_lats(1)
          dth3 = src_lats(3) - src_lats(2) - dth2

          dph1 = src_lons(2) - src_lons(1)
          dph2 = src_lons(4) - src_lons(1)
          dph3 = src_lons(3) - src_lons(2)

          if (dph1 >  pi) dph1 = dph1 - pi2
          if (dph2 >  pi) dph2 = dph2 - pi2
          if (dph3 >  pi) dph3 = dph3 - pi2
          if (dph1 < -pi) dph1 = dph1 + pi2
          if (dph2 < -pi) dph2 = dph2 + pi2
          if (dph3 < -pi) dph3 = dph3 + pi2

          dph3 = dph3 - dph2

          iguess = zero
          jguess = zero

          iter_loop2: do iter=1,max_iter

            dthp = plat - src_lats(1) - dth1*iguess -
     &                    dth2*jguess - dth3*iguess*jguess
            dphp = plon - src_lons(1)

            if (dphp >  pi) dphp = dphp - pi2
            if (dphp < -pi) dphp = dphp + pi2

            dphp = dphp - dph1*iguess - dph2*jguess - 
     &                    dph3*iguess*jguess

            mat1 = dth1 + dth3*jguess
            mat2 = dth2 + dth3*iguess
            mat3 = dph1 + dph3*jguess
            mat4 = dph2 + dph3*iguess

            determinant = mat1*mat4 - mat2*mat3

            deli = (dthp*mat4 - mat2*dphp)/determinant
            delj = (mat1*dphp - dthp*mat3)/determinant

            if (abs(deli) < converge .and. 
     &          abs(delj) < converge) exit iter_loop2

            iguess = iguess + deli
            jguess = jguess + delj

          end do iter_loop2

          if (iter <= max_iter) then

            !***
            !*** successfully found i,j - compute weights
            !***

            wgts(1) = (one-iguess)*(one-jguess)
            wgts(2) = iguess*(one-jguess)
            wgts(3) = iguess*jguess
            wgts(4) = (one-iguess)*jguess

            call store_link_bilin(dst_add, src_add, wgts, nmap)

          else
            print *,'Point coords: ',plat,plon
            print *,'Dest grid lats: ',src_lats
            print *,'Dest grid lons: ',src_lons
            print *,'Dest grid addresses: ',src_add
            print *,'Current i,j : ',iguess, jguess
            stop 'Iteration for i,j exceed max iteration count'
          endif

        !***
        !*** search for bilinear failed - us a distance-weighted
        !*** average instead
        !***

        else if (src_add(1) < 0) then

          src_add = abs(src_add)
          icount = 0
          do n=1,4
            if (grid2_mask(src_add(n))) then
              icount = icount + 1
            else
              src_lats(n) = zero
            endif
          end do

          if (icount > 0) then
            !*** renormalize weights

            sum_wgts = sum(src_lats)
            wgts(1) = src_lats(1)/sum_wgts
            wgts(2) = src_lats(2)/sum_wgts
            wgts(3) = src_lats(3)/sum_wgts
            wgts(4) = src_lats(4)/sum_wgts

            grid1_frac(dst_add) = one
            call store_link_bilin(dst_add, src_add, wgts, nmap)
          endif

        endif
      end do grid_loop2

      endif ! nmap=2

C-----------------------------------------------------------------------

      end subroutine remap_bilin

C***********************************************************************

      subroutine grid_search_bilin(src_add, src_lats, src_lons, 
     &                             plat, plon, src_grid_dims,
     &                             src_center_lat, src_center_lon,
     &                             src_grid_bound_box,
     &                             src_bin_add, dst_bin_add)

C-----------------------------------------------------------------------
C
C     this routine finds the location of the search point plat, plon
C     in the source grid and returns the corners needed for a bilinear
C     interpolation.
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     output variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), dimension(4), intent(out) ::
     &        src_add  ! address of each corner point enclosing P

      real (kind=dbl_kind), dimension(4), intent(out) ::
     &        src_lats, ! latitudes  of the four corner points
     &        src_lons  ! longitudes of the four corner points

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      real (kind=dbl_kind), intent(in) ::
     &        plat,   ! latitude  of the search point
     &        plon    ! longitude of the search point

      integer (kind=int_kind), dimension(2), intent(in) ::
     &        src_grid_dims  ! size of each src grid dimension

      real (kind=dbl_kind), dimension(:), intent(in) ::
     &        src_center_lat, ! latitude  of each src grid center 
     &        src_center_lon  ! longitude of each src grid center

      real (kind=dbl_kind), dimension(:,:), intent(in) ::
     &        src_grid_bound_box ! bound box for source grid

      integer (kind=int_kind), dimension(:,:), intent(in) ::
     &        src_bin_add,    ! latitude bins for restricting
     &        dst_bin_add     ! searches

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, next_n, srch_add,   ! dummy indices
     &    nx, ny,            ! dimensions of src grid
     &    min_add, max_add,  ! addresses for restricting search
     &    i, j, jp1, ip1, n_add, e_add, ne_add  ! addresses

      real (kind=dbl_kind) ::  ! vectors for cross-product check
     &      vec1_lat, vec1_lon,
     &      vec2_lat, vec2_lon, cross_product, cross_product_last,
     &      coslat_dst, sinlat_dst, coslon_dst, sinlon_dst,
     &      dist_min, distance ! for computing dist-weighted avg

C-----------------------------------------------------------------------
C
C     restrict search first using bins
C
C-----------------------------------------------------------------------

      src_add = 0

      min_add = size(src_center_lat)
      max_add = 1
      do n=1,num_srch_bins
        if (plat >= bin_lats(1,n) .and. plat <= bin_lats(2,n) .and.
     &      plon >= bin_lons(1,n) .and. plon <= bin_lons(2,n)) then
          min_add = min(min_add, src_bin_add(1,n))
          max_add = max(max_add, src_bin_add(2,n))
        endif
      end do
 
C-----------------------------------------------------------------------
C
C     now perform a more detailed search 
C
C-----------------------------------------------------------------------

      nx = src_grid_dims(1)
      ny = src_grid_dims(2)

      srch_loop: do srch_add = min_add,max_add

        !*** first check bounding box

        if (plat <= src_grid_bound_box(2,srch_add) .and. 
     &      plat >= src_grid_bound_box(1,srch_add) .and.
     &      plon <= src_grid_bound_box(4,srch_add) .and. 
     &      plon >= src_grid_bound_box(3,srch_add)) then

          !***
          !*** we are within bounding box so get really serious
          !***

          !*** determine neighbor addresses

          j = (srch_add - 1)/nx +1
          i = srch_add - (j-1)*nx

          if (i < nx) then
            ip1 = i + 1
          else
            ip1 = 1
          endif

          if (j < ny) then
            jp1 = j+1
          else
            jp1 = 1
          endif

          n_add = (jp1 - 1)*nx + i
          e_add = (j - 1)*nx + ip1
          ne_add = (jp1 - 1)*nx + ip1

          src_lats(1) = src_center_lat(srch_add)
          src_lats(2) = src_center_lat(e_add)
          src_lats(3) = src_center_lat(ne_add)
          src_lats(4) = src_center_lat(n_add)

          src_lons(1) = src_center_lon(srch_add)
          src_lons(2) = src_center_lon(e_add)
          src_lons(3) = src_center_lon(ne_add)
          src_lons(4) = src_center_lon(n_add)

          !***
          !*** for consistency, we must make sure all lons are in
          !*** same 2pi interval
          !***

          vec1_lon = src_lons(1) - plon
          if (vec1_lon >  pi) then
            src_lons(1) = src_lons(1) - pi2
          else if (vec1_lon < -pi) then
            src_lons(1) = src_lons(1) + pi2
          endif
          do n=2,4
            vec1_lon = src_lons(n) - src_lons(1)
            if (vec1_lon >  pi) then
              src_lons(n) = src_lons(n) - pi2
            else if (vec1_lon < -pi) then
              src_lons(n) = src_lons(n) + pi2
            endif
          end do

          corner_loop: do n=1,4
            next_n = MOD(n,4) + 1

            !***
            !*** here we take the cross product of the vector making 
            !*** up each box side with the vector formed by the vertex
            !*** and search point.  if all the cross products are 
            !*** positive, the point is contained in the box.
            !***

            vec1_lat = src_lats(next_n) - src_lats(n)
            vec1_lon = src_lons(next_n) - src_lons(n)
            vec2_lat = plat - src_lats(n)
            vec2_lon = plon - src_lons(n)

            !***
            !*** check for 0,2pi crossings
            !***

            if (vec1_lon >  three*pih) then
              vec1_lon = vec1_lon - pi2
            else if (vec1_lon < -three*pih) then
              vec1_lon = vec1_lon + pi2
            endif
            if (vec2_lon >  three*pih) then
              vec2_lon = vec2_lon - pi2
            else if (vec2_lon < -three*pih) then
              vec2_lon = vec2_lon + pi2
            endif

            cross_product = vec1_lon*vec2_lat - vec2_lon*vec1_lat

            !***
            !*** if cross product is less than zero, this cell
            !*** doesn't work
            !***

            if (n == 1) cross_product_last = cross_product
            if (cross_product*cross_product_last < zero) 
     &          exit corner_loop
            cross_product_last = cross_product

          end do corner_loop

          !***
          !*** if cross products all same sign, we found the location
          !***

          if (n > 4) then
            src_add(1) = srch_add
            src_add(2) = e_add
            src_add(3) = ne_add
            src_add(4) = n_add

            return
          endif

          !***
          !*** otherwise move on to next cell
          !***

        endif !bounding box check
      end do srch_loop

      !***
      !*** if no cell found, point is likely either in a box that
      !*** straddles either pole or is outside the grid.  fall back
      !*** to a distance-weighted average of the four closest
      !*** points.  go ahead and compute weights here, but store
      !*** in src_lats and return -add to prevent the parent
      !*** routine from computing bilinear weights
      !***

      !print *,'Could not find location for ',plat,plon
      !print *,'Using nearest-neighbor average for this point'

      coslat_dst = cos(plat)
      sinlat_dst = sin(plat)
      coslon_dst = cos(plon)
      sinlon_dst = sin(plon)

      dist_min = bignum
      src_lats = bignum
      do srch_add = min_add,max_add
        distance = acos(coslat_dst*cos(src_center_lat(srch_add))*
     &                 (coslon_dst*cos(src_center_lon(srch_add)) +
     &                  sinlon_dst*sin(src_center_lon(srch_add)))+
     &                  sinlat_dst*sin(src_center_lat(srch_add)))

        if (distance < dist_min) then
          sort_loop: do n=1,4
            if (distance < src_lats(n)) then
              do i=4,n+1,-1
                src_add (i) = src_add (i-1)
                src_lats(i) = src_lats(i-1)
              end do
              src_add (n) = -srch_add
              src_lats(n) = distance
              dist_min = src_lats(4)
              exit sort_loop
            endif
          end do sort_loop
        endif
      end do

      src_lons = one/(src_lats + tiny)
      distance = sum(src_lons)
      src_lats = src_lons/distance

C-----------------------------------------------------------------------

      end subroutine grid_search_bilin 

C***********************************************************************

      subroutine store_link_bilin(dst_add, src_add, weights, nmap)

C-----------------------------------------------------------------------
C
C     this routine stores the address and weight for four links 
C     associated with one destination point in the appropriate address 
C     and weight arrays and resizes those arrays if necessary.
C
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C
C     input variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind), intent(in) ::
     &        dst_add,  ! address on destination grid
     &        nmap      ! identifies which direction for mapping

      integer (kind=int_kind), dimension(4), intent(in) ::
     &        src_add   ! addresses on source grid

      real (kind=dbl_kind), dimension(4), intent(in) ::
     &        weights ! array of remapping weights for these links

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: n, ! dummy index
     &       num_links_old          ! placeholder for old link number

C-----------------------------------------------------------------------
C
C     increment number of links and check to see if remap arrays need
C     to be increased to accomodate the new link.  then store the
C     link.
C
C-----------------------------------------------------------------------

      select case (nmap)
      case(1)

        num_links_old  = num_links_map1
        num_links_map1 = num_links_old + 4

        if (num_links_map1 > max_links_map1) 
     &     call resize_remap_vars(1,resize_increment)

        do n=1,4
          grid1_add_map1(num_links_old+n) = src_add(n)
          grid2_add_map1(num_links_old+n) = dst_add
          wts_map1    (1,num_links_old+n) = weights(n)
        end do

      case(2)

        num_links_old  = num_links_map2
        num_links_map2 = num_links_old + 4

        if (num_links_map2 > max_links_map2) 
     &     call resize_remap_vars(2,resize_increment)

        do n=1,4
          grid1_add_map2(num_links_old+n) = dst_add
          grid2_add_map2(num_links_old+n) = src_add(n)
          wts_map2    (1,num_links_old+n) = weights(n)
        end do

      end select

C-----------------------------------------------------------------------

      end subroutine store_link_bilin

C***********************************************************************

      end module remap_bilinear

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C
C     This routine is the driver for computing the addresses and weights 
C     for interpolating between two grids on a sphere.
C
C-----------------------------------------------------------------------
C
C     CVS:$Id: scrip.f,v 1.6 2001/08/21 21:06:44 pwjones Exp $
C
C     Copyright (c) 1997, 1998 the Regents of the University of 
C       California.
C
C     This software and ancillary information (herein called software) 
C     called SCRIP is made available under the terms described here.  
C     The software has been approved for release with associated 
C     LA-CC Number 98-45.
C
C     Unless otherwise indicated, this software has been authored
C     by an employee or employees of the University of California,
C     operator of the Los Alamos National Laboratory under Contract
C     No. W-7405-ENG-36 with the U.S. Department of Energy.  The U.S.
C     Government has rights to use, reproduce, and distribute this
C     software.  The public may copy and use this software without
C     charge, provided that this Notice and any statement of authorship
C     are reproduced on all copies.  Neither the Government nor the
C     University makes any warranty, express or implied, or assumes
C     any liability or responsibility for the use of this software.
C
C     If software is modified to produce derivative works, such modified
C     software should be clearly marked, so as not to confuse it with 
C     the version available from Los Alamos National Laboratory.
C
C***********************************************************************

      subroutine scrip

C-----------------------------------------------------------------------

      use kinds_mod                  ! module defining data types
      use constants                  ! module for common constants
      use iounits                    ! I/O unit manager
      use grids                      ! module with grid information
      use remap_vars                 ! common remapping variables
      use remap_bilinear             ! routines for bilinear interp
      use remap_write                ! routines for remap output

      implicit none

C-----------------------------------------------------------------------
C
C     input namelist variables
C
C-----------------------------------------------------------------------

      character (char_len) :: 
     &           grid1_file,   ! filename of grid file containing grid1
     &           grid2_file,   ! filename of grid file containing grid2
     &           interp_file1, ! filename for output remap data (map1)
     &           interp_file2, ! filename for output remap data (map2)
     &           map1_name,    ! name for mapping from grid1 to grid2
     &           map2_name,    ! name for mapping from grid2 to grid1
     &           map_method,   ! choice for mapping method
     &           normalize_opt,! option for normalizing weights
     &           output_opt    ! option for output conventions

      integer (kind=int_kind) ::
     &           nmap          ! number of mappings to compute (1 or 2)

      namelist /remap_inputs/ grid1_file, grid2_file, 
     &                        interp_file1, interp_file2,
     &                        map1_name, map2_name, num_maps,
     &                        luse_grid1_area, luse_grid2_area,
     &                        map_method, normalize_opt, output_opt,
     &                        restrict_type, num_srch_bins

C-----------------------------------------------------------------------
C
C     local variables
C
C-----------------------------------------------------------------------

      integer (kind=int_kind) :: n,     ! dummy counter
     &                           iunit  ! unit number for namelist file

C-----------------------------------------------------------------------
C
C     read input namelist
C
C-----------------------------------------------------------------------

      grid1_file    = 'model_grid.nc'
      grid2_file    = 'control_grid.nc'
      interp_file1  = 'rmp_model_to_control_bilin.nc'
      interp_file2  = 'rmp_control_to_model_bilin.nc'
      map1_name     = 'MODEL to CONTROL Bilinear Mapping'
      map2_name     = 'CONTROL to MODEL Bilinear Mapping'
      luse_grid1_area = .false.
      luse_grid2_area = .false.
      num_maps      = 2
      map_type      = 1
      normalize_opt = 'frac'
      output_opt    = 'scrip'
      restrict_type = 'latitude'
      num_srch_bins = 90
      map_method = 'bilinear'

C      call get_unit(iunit)
C      open(iunit, file='scrip_in', status='old', form='formatted')
C      read(iunit, nml=remap_inputs)
C      call release_unit(iunit)

      select case(map_method)
      case ('bilinear')
        map_type = map_type_bilinear
        luse_grid_centers = .true.
      case default
        stop 'unknown mapping method'
      end select

      select case(normalize_opt(1:4))
      case ('none')
        norm_opt = norm_opt_none
      case ('frac')
        norm_opt = norm_opt_frcarea
      case ('dest')
        norm_opt = norm_opt_dstarea
      case default
        stop 'unknown normalization option'
      end select

C-----------------------------------------------------------------------
C
C     initialize grid information for both grids
C
C-----------------------------------------------------------------------

      call grid_init(grid1_file, grid2_file)

C      write(stdout, *) ' Computing remappings between: ',grid1_name
C      write(stdout, *) '                          and  ',grid2_name

C-----------------------------------------------------------------------
C
C     initialize some remapping variables.
C
C-----------------------------------------------------------------------

      call init_remap_vars

C-----------------------------------------------------------------------
C
C     call appropriate interpolation setup routine based on type of
C     remapping requested.
C
C-----------------------------------------------------------------------

      select case(map_type)
      case(map_type_bilinear)
        call remap_bilin
      case default
        stop 'Invalid Map Type'
      end select

C-----------------------------------------------------------------------
C
C     reduce size of remapping arrays and then write remapping info
C     to a file.
C
C-----------------------------------------------------------------------

      if (num_links_map1 /= max_links_map1) then
        call resize_remap_vars(1, num_links_map1-max_links_map1)
      endif
      if ((num_maps > 1) .and. (num_links_map2 /= max_links_map2)) then
        call resize_remap_vars(2, num_links_map2-max_links_map2)
      endif

      call write_remap(map1_name, map2_name, 
     &                 interp_file1, interp_file2, output_opt)

C-----------------------------------------------------------------------

      end subroutine scrip

C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

